@@ -20,6 +20,7 @@
 
 #include <QTextEdit>
 #include <QMultiMap>
+#include <QTimer>
 
 #include <klocale.h>
 #include <kiconloader.h>
@@ -69,6 +70,10 @@ KateCompletionModel::KateCompletionModel(KateCompletionWidget* parent)
   m_emptyGroups.append(m_argumentHints);
   m_emptyGroups.append(m_bestMatches);
 
+  m_updateBestMatchesTimer = new QTimer(this);
+  m_updateBestMatchesTimer->setSingleShot(true);
+  connect(m_updateBestMatchesTimer, SIGNAL(timeout()), this, SLOT(updateBestMatches()));
+  
   m_groupHash.insert(0, m_ungrouped);
   m_groupHash.insert(-1, m_argumentHints);
   m_groupHash.insert(BestMatchesProperty, m_argumentHints);
@@ -408,29 +413,79 @@ void KateCompletionModel::createGroups()
   clearGroups();
 
   foreach (CodeCompletionModel* sourceModel, m_completionModels)
-    for (int i = 0; i < sourceModel->rowCount(); ++i) {
-      QModelIndex sourceIndex = sourceModel->index(i, CodeCompletionModel::Name, QModelIndex());
-
-      int completionFlags = sourceIndex.data(CodeCompletionModel::CompletionRole).toInt();
-      QString scopeIfNeeded = (groupingMethod() & Scope) ? sourceModel->index(i, CodeCompletionModel::Scope, QModelIndex()).data(Qt::DisplayRole).toString() : QString();
-
-      int argumentHintDepth = sourceIndex.data(CodeCompletionModel::ArgumentHintDepth).toInt();
-
-      Group* g;
-      if( argumentHintDepth )
-        g = m_argumentHints;
-      else
-        g = fetchGroup(completionFlags, scopeIfNeeded);
-
-      g->addItem(Item(this, ModelRow(sourceModel, i)));
-    }
+    for (int i = 0; i < sourceModel->rowCount(); ++i)
+      createItem(sourceModel, i);
 
   //debugStats();
 
   resort();
   reset();
+  updateBestMatches();
+  emit contentGeometryChanged();
+}
 
-  rematch();
+KateCompletionModel::Group* KateCompletionModel::createItem(CodeCompletionModel* sourceModel, int row, bool notifyModel)
+{
+  QModelIndex sourceIndex = sourceModel->index(row, CodeCompletionModel::Name, QModelIndex());
+
+  int completionFlags = sourceIndex.data(CodeCompletionModel::CompletionRole).toInt();
+  QString scopeIfNeeded = (groupingMethod() & Scope) ? sourceModel->index(row, CodeCompletionModel::Scope, QModelIndex()).data(Qt::DisplayRole).toString() : QString();
+
+  int argumentHintDepth = sourceIndex.data(CodeCompletionModel::ArgumentHintDepth).toInt();
+
+  Group* g;
+  if( argumentHintDepth )
+    g = m_argumentHints;
+  else
+    g = fetchGroup(completionFlags, scopeIfNeeded);
+
+  Item item = Item(this, ModelRow(sourceModel, row));
+
+  if(g != m_argumentHints)
+    item.match(m_currentMatch);
+
+  g->addItem(item, notifyModel);
+
+  return g;
+}
+
+void KateCompletionModel::slotRowsInserted( const QModelIndex & parent, int start, int end )
+{
+  if (!parent.isValid()) {
+    QSet<Group*> affectedGroups;
+
+    for (int i = start; i <= end; ++i)
+      affectedGroups << createItem(static_cast<CodeCompletionModel*>(sender()), i, true);
+
+    foreach (Group* g, affectedGroups)
+      hideOrShowGroup(g);
+  } else {
+    kWarning() << "Heirachical code completion models not supported.";
+  }
+  emit contentGeometryChanged();
+}
+
+void KateCompletionModel::slotRowsRemoved( const QModelIndex & parent, int start, int end )
+{
+  if (!parent.isValid()) {
+    QSet<Group*> affectedGroups;
+
+    CodeCompletionModel* source = static_cast<CodeCompletionModel*>(sender());
+
+    for (int i = start; i <= end; ++i) {
+      Group* g = groupForIndex(mapFromSource(parent));
+      Q_ASSERT(g);
+
+      if (g->removeItem(ModelRow(source, i)))
+        affectedGroups << g;
+    }
+
+    foreach (Group* g, affectedGroups)
+      hideOrShowGroup(g);
+    contentGeometryChanged();
+  } else {
+    kWarning() << "Heirachical code completion models not supported.";
+  }
 }
 
 KateCompletionModel::Group* KateCompletionModel::fetchGroup( int attribute, const QString& scope )
@@ -484,7 +539,7 @@ KateCompletionModel::Group* KateCompletionModel::fetchGroup( int attribute, cons
       at = "Private";
 
     if (accessIncludeStatic() && attribute & KTextEditor::CodeCompletionModel::Static)
-      at.append(i18n(" Static"));
+      at.append(" Static");
 
     if (accessIncludeConst() && attribute & KTextEditor::CodeCompletionModel::Const)
       at.append(" Const");
@@ -573,13 +628,11 @@ QModelIndex KateCompletionModel::parent( const QModelIndex & index ) const
 
     int row = m_rowTable.indexOf(g);
 
-    /**
-     * Workaround for crash caused by the assertion that I don't know how to fix, any better fix is encouraged.
-     * It happens when I click setup
-     * */
-    if( row == -1 )
+    if (row == -1) {
+      kWarning() << k_funcinfo << "Couldn't find parent for index" << index;
       return QModelIndex();
-//     Q_ASSERT(row != -1);
+    }
+    
     return createIndex(row, 0, 0);
   }
 
@@ -681,9 +734,11 @@ void KateCompletionModel::setCurrentCompletion( const QString & completion )
     changeCompletions(m_ungrouped, completion, changeType);
   else {
     foreach (Group* g, m_rowTable)
-      changeCompletions(g, completion, changeType);
+      if(g != m_argumentHints)
+        changeCompletions(g, completion, changeType);
     foreach (Group* g, m_emptyGroups)
-      changeCompletions(g, completion, changeType);
+      if(g != m_argumentHints)
+        changeCompletions(g, completion, changeType);
 
     updateBestMatches();
   }
@@ -699,14 +754,16 @@ void KateCompletionModel::rematch()
 
   } else {
     foreach (Group* g, m_rowTable)
-      changeCompletions(g, m_currentMatch, Change);
+      if(g != m_argumentHints)
+        changeCompletions(g, m_currentMatch, Change);
 
     foreach (Group* g, m_emptyGroups)
-      changeCompletions(g, m_currentMatch, Change);
+      if(g != m_argumentHints)
+        changeCompletions(g, m_currentMatch, Change);
 
     updateBestMatches();
   }
-  
+
   clearExpanding(); //We need to do this, or be aware of expanding-widgets while filtering.
 }
 
@@ -822,6 +879,7 @@ void KateCompletionModel::changeCompletions( Group * g, const QString & newCompl
   COMPLETE_ADD
 
   hideOrShowGroup(g);
+  emit contentGeometryChanged();
 }
 
 int KateCompletionModel::Group::orderNumber() const {
@@ -855,8 +913,11 @@ bool KateCompletionModel::Group::orderBefore(Group* other) const {
 
 void KateCompletionModel::hideOrShowGroup(Group* g)
 {
-  if( g == m_argumentHints )
+  if( g == m_argumentHints ) {
+    emit argumentHintsChanged();
+    m_updateBestMatchesTimer->start(200); //We have new argument-hints, so we have new best matches
     return; //Never show argument-hints in the normal completion-list
+  }
 
   if (!g->isEmpty) {
     if (g->rows.isEmpty()) {
@@ -890,14 +951,11 @@ void KateCompletionModel::hideOrShowGroup(Group* g)
       }
       if (hasGroups())
         beginInsertRows(QModelIndex(), row, row);
+      else
+        beginInsertRows(QModelIndex(), 0, g->rows.count());
       m_rowTable.insert(row, g);
-      if (hasGroups())
-        endInsertRows();
+      endInsertRows();
       m_emptyGroups.removeAll(g);
-
-      // Cheating a bit here... we already display these rows
-      //beginInsertRows(indexForGroup(g), 0, g->rows.count() - 1);
-      //endInsertRows();
     }
   }
 }
@@ -929,20 +987,22 @@ void KateCompletionModel::deleteRows( Group* g, QMutableListIterator<ModelRow> &
 
 void KateCompletionModel::addRows( Group * g, QMutableListIterator<ModelRow> & filtered, int startRow, const QList<ModelRow> & newItems )
 {
-  bool notify = true;
+  //bool notify = true;
 
   QModelIndex groupIndex = indexForGroup(g);
-  if (hasGroups() && !groupIndex.isValid())
+  //if (hasGroups() && !groupIndex.isValid())
     // Group is currently hidden... don't emit begin/endInsertRows.
-    notify = false;
+    //notify = false;
 
-  if (notify)
+  kDebug() << "Group" << g->title << "addRows" << startRow << "to " << (startRow + newItems.count() - 1);
+
+  //if (notify)
     beginInsertRows(groupIndex, startRow, startRow + newItems.count() - 1);
 
   for (int i = 0; i < newItems.count(); ++i)
     filtered.insert(newItems[i]);
 
-  if (notify)
+  //if (notify)
     endInsertRows();
 }
 
@@ -957,14 +1017,6 @@ bool KateCompletionModel::indexIsItem( const QModelIndex & index ) const
   return false;
 }
 
-void KateCompletionModel::slotRowsInserted( const QModelIndex & /*parent*/, int /*start*/, int /*end*/ )
-{
-}
-
-void KateCompletionModel::slotRowsRemoved( const QModelIndex & /*parent*/, int /*start*/, int /*end*/ )
-{
-}
-
 void KateCompletionModel::slotModelReset()
 {
   createGroups();
@@ -1070,15 +1122,26 @@ int KateCompletionModel::translateColumn( int sourceColumn ) const
   if (m_columnMerges.isEmpty())
     return sourceColumn;
 
+  /* Debugging - dump column merge list
+
+  QString columnMerge;
+  foreach (const QList<int>& list, m_columnMerges) {
+    columnMerge += '[';
+    foreach (int column, list) {
+      columnMerge += QString::number(column) + " ";
+    }
+    columnMerge += "] ";
+  }
+
+  kDebug() << k_funcinfo << columnMerge;*/
+
   int c = 0;
   foreach (const QList<int>& list, m_columnMerges) {
     foreach (int column, list) {
       if (column == sourceColumn)
-        ///@todo Ugly workaround: "return c" is correct here. This is a workaround for a bug I don't understand, which you can notice in katecompletionwidget.cpp:
-        ///For some reason, the name-column is mapped to the post-fix column when column-merging is enabled, so the widget is not positioned correctly.
-        return c > 0 ? c-1 : c;
-      c++;
+        return c;
     }
+    c++;
   }
   return -1;
 }
@@ -1241,7 +1304,7 @@ KateCompletionModel::Item::Item( KateCompletionModel* m, ModelRow sr )
   , matchFilters(true)
 {
   inheritanceDepth = m_sourceRow.first->index(m_sourceRow.second, 0).data(CodeCompletionModel::InheritanceDepth).toInt();
-  
+
   filter();
   match();
 }
@@ -1276,23 +1339,57 @@ QString KateCompletionModel::Item::completionSortingName( ) const
   return m_completionSortingName;
 }
 
-void KateCompletionModel::Group::addItem( Item i )
+void KateCompletionModel::Group::addItem( Item i, bool notifyModel )
 {
+  if (isEmpty)
+    notifyModel = false;
+
+  QModelIndex groupIndex;
+  if (notifyModel)
+    groupIndex = model->indexForGroup(this);
+
   if (model->isSortingEnabled()) {
-    QList<Item>::Iterator it = model->isSortingReverse() ? qUpperBound(--prefilter.end(), prefilter.end(), i) : qUpperBound(prefilter.begin(), prefilter.end(), i);
+    QList<Item>::Iterator it = model->isSortingReverse() ? qLowerBound(prefilter.begin(), prefilter.end(), i) : qUpperBound(prefilter.begin(), prefilter.end(), i);
     if (it != prefilter.end()) {
+      const Item& item = *it;
       prefilter.insert(it, i);
       if (i.isVisible()) {
-        int index = rows.indexOf(it->sourceRow());
-        if (index != -1)
+        int index = rows.indexOf(item.sourceRow());
+        if (index == -1) {
+          if (notifyModel)
+            model->beginInsertRows(groupIndex, rows.count(), rows.count());
+
           rows.append(i.sourceRow());
-        else
+
+        } else {
+          if (notifyModel)
+            model->beginInsertRows(groupIndex, index, index);
+
           rows.insert(index, i.sourceRow());
+        }
+
+        if (notifyModel)
+          model->endInsertRows();
       }
     } else {
       prefilter.append(i);
-      if (i.isVisible())
-        rows.append(i.sourceRow());
+      if (i.isVisible()) {
+        if (model->isSortingReverse()) {
+          if (notifyModel)
+            model->beginInsertRows(groupIndex, 0, 0);
+
+          rows.prepend(i.sourceRow());
+
+        } else {
+          if (notifyModel)
+            model->beginInsertRows(groupIndex, rows.count(), rows.count());
+
+          rows.append(i.sourceRow());
+        }
+
+        if (notifyModel)
+          model->endInsertRows();
+      }
     }
 
   } else {
@@ -1301,6 +1398,27 @@ void KateCompletionModel::Group::addItem( Item i )
   }
 }
 
+bool KateCompletionModel::Group::removeItem(const ModelRow& row)
+{
+  for (int pi = 0; pi < prefilter.count(); ++pi)
+    if (prefilter[pi].sourceRow() == row) {
+      int index = rows.indexOf(row);
+      if (index != -1)
+        model->beginRemoveRows(model->indexForGroup(this), index, index);
+
+      rows.removeAt(index);
+      prefilter.removeAt(pi);
+
+      if (index != -1)
+        model->endRemoveRows();
+
+      return index != -1;
+    }
+
+  Q_ASSERT(false);
+  return false;
+}
+
 KateCompletionModel::Group::Group( KateCompletionModel * m )
   : model(m)
   , isEmpty(true)
@@ -1326,7 +1444,6 @@ void KateCompletionModel::Group::resort( )
       rows.append(i.sourceRow());
 
   model->hideOrShowGroup(this);
-
   //Q_ASSERT(rows.count() == oldRowCount);
 }
 
@@ -1353,6 +1470,8 @@ void KateCompletionModel::resort( )
 
   foreach (Group* g, m_emptyGroups)
     g->resort();
+  
+  emit contentGeometryChanged();
 }
 
 bool KateCompletionModel::Item::isValid( ) const
@@ -1424,13 +1543,15 @@ void KateCompletionModel::refilter( )
   m_ungrouped->refilter();
 
   foreach (Group* g, m_rowTable)
-    g->refilter();
+    if(g != m_argumentHints)
+      g->refilter();
 
   foreach (Group* g, m_emptyGroups)
-    g->refilter();
+    if(g != m_argumentHints)
+      g->refilter();
 
   updateBestMatches();
-  
+
   clearExpanding(); //We need to do this, or be aware of expanding-widgets while filtering.
 }
 
@@ -1660,6 +1781,7 @@ void KateCompletionModel::removeCompletionModel(CodeCompletionModel * model)
 //Updates the best-matches group
 void KateCompletionModel::updateBestMatches() {
 
+  m_updateBestMatchesTimer->stop();
   //Maps match-qualities to ModelRows paired together with the BestMatchesCount returned by the items.
   typedef QMultiMap<int, QPair<int, ModelRow> > BestMatchMap;
   BestMatchMap matches;
