@@ -88,7 +88,7 @@ KateCompletionWidget::KateCompletionWidget(KateView* parent)
   m_sortText = new QLabel(i18n("Sort: None"), m_statusBar);
 
   m_filterButton = new QToolButton(m_statusBar);
-  m_filterButton->setIcon(KIcon("search-filter"));
+  m_filterButton->setIcon(KIcon("view-filter"));
   m_filterButton->setCheckable(true);
   connect(m_filterButton, SIGNAL(toggled(bool)), m_presentationModel, SLOT(setFilteringEnabled(bool)));
 
@@ -104,7 +104,7 @@ KateCompletionWidget::KateCompletionWidget(KateView* parent)
   m_updateFocusTimer = new QTimer(this);
   m_updateFocusTimer->setSingleShot(true);
   connect(m_updateFocusTimer, SIGNAL(timeout()), this, SLOT(updateFocus()));
-  
+
   QSizeGrip* sg = new QSizeGrip(m_statusBar);
 
   QHBoxLayout* statusLayout = new QHBoxLayout(m_statusBar);
@@ -131,6 +131,7 @@ KateCompletionWidget::KateCompletionWidget(KateView* parent)
   // Keep branches expanded
   connect(m_presentationModel, SIGNAL(modelReset()), this, SLOT(modelReset()), Qt::QueuedConnection);
   connect(m_presentationModel, SIGNAL(rowsInserted(const QModelIndex&, int, int)), SLOT(rowsInserted(const QModelIndex&, int, int)));
+  connect(m_argumentHintModel, SIGNAL(contentStateChanged(bool)), this, SLOT(argumentHintsChanged(bool)));
 
   connect(view(), SIGNAL(cursorPositionChanged(KTextEditor::View*, const KTextEditor::Cursor&)), SLOT(cursorPositionChanged()));
   connect(view()->doc()->history(), SIGNAL(editDone(KateEditInfo*)), SLOT(editDone(KateEditInfo*)), Qt::QueuedConnection);
@@ -142,7 +143,7 @@ KateCompletionWidget::KateCompletionWidget(KateView* parent)
 
   //We need to do this, because else the focus goes to nirvana without any control when the completion-widget is clicked.
   setFocusPolicy(Qt::ClickFocus);
-  
+
   foreach (QWidget* childWidget, findChildren<QWidget*>())
     childWidget->setFocusPolicy(Qt::NoFocus);
 }
@@ -150,6 +151,17 @@ KateCompletionWidget::KateCompletionWidget(KateView* parent)
 KateCompletionWidget::~KateCompletionWidget() {
 }
 
+void KateCompletionWidget::modelContentChanged() {
+  int realItemCount = 0;
+  foreach (KTextEditor::CodeCompletionModel* model, m_presentationModel->completionModels())
+    realItemCount += model->rowCount();
+  //kDebug() << "content changed, item count " << realItemCount << "\n" << kBacktrace();
+  if( !m_isSuspended && !isVisible() && realItemCount != 0 ) {
+    //kDebug() << "showing";
+    updateAndShow();
+  }
+}
+
 void KateCompletionWidget::focusInEvent ( QFocusEvent * event ) {
   //Instantly give the focus back
   view()->activateWindow(); //When the completion-widget is clicked, the application-window is deactivated, so re-activate it
@@ -204,8 +216,9 @@ KateCompletionModel* KateCompletionWidget::model() {
 
 void KateCompletionWidget::rowsInserted(const QModelIndex& parent, int rowFrom, int rowEnd)
 {
-  for (int i = rowFrom; i <= rowEnd; ++i)
-    m_entryList->expand(m_presentationModel->index(i, 0, parent));
+  if (!parent.isValid())
+    for (int i = rowFrom; i <= rowEnd; ++i)
+      m_entryList->expand(m_presentationModel->index(i, 0, parent));
 }
 
 KateView * KateCompletionWidget::view( ) const
@@ -213,12 +226,25 @@ KateView * KateCompletionWidget::view( ) const
   return static_cast<KateView*>(const_cast<QObject*>(parent()));
 }
 
+void KateCompletionWidget::argumentHintsChanged(bool hasContent)
+{
+  m_dontShowArgumentHints = !hasContent;
+  
+  if( m_dontShowArgumentHints )
+    m_argumentHintTree->hide();
+  else
+    updateArgumentHintGeometry();
+}
+
 void KateCompletionWidget::startCompletion( const KTextEditor::Range & word, KTextEditor::CodeCompletionModel * model, KTextEditor::CodeCompletionModel::InvocationType invocationType)
 {
   m_isSuspended = false;
+  m_inCompletionList = true; //Always start at the top of the completion-list
+  
+  disconnect(this->model(), SIGNAL(contentGeometryChanged()), this, SLOT(modelContentChanged()));
 
   m_dontShowArgumentHints = true;
-  
+
   if (!word.isValid()) {
     kWarning(13035) << "Invalid range given to start code completion!";
     return;
@@ -240,9 +266,9 @@ void KateCompletionWidget::startCompletion( const KTextEditor::Range & word, KTe
     abortCompletion();
 
   m_completionRange = view()->doc()->smartManager()->newSmartRange(word);
-  m_completionRange->setInsertBehavior(KTextEditor::SmartRange::ExpandRight);
+  m_completionRange->setInsertBehavior(KTextEditor::SmartRange::ExpandRight | KTextEditor::SmartRange::ExpandLeft);
   if(!m_completionRange->isValid()) {
-    kWarning(13035) << "Could not construct valid smart-range from " << word;
+    kWarning(13035) << "Could not construct valid smart-range from" << word << "instead got" << *m_completionRange;
     abortCompletion();
     return;
   }
@@ -255,29 +281,34 @@ void KateCompletionWidget::startCompletion( const KTextEditor::Range & word, KTe
     m_presentationModel->setCompletionModel(model);
   else
     m_presentationModel->setCompletionModels(m_sourceModels);
-
-  setUpdatesEnabled(false);
   
+  if (!m_presentationModel->completionModels().isEmpty()) {
+    foreach (KTextEditor::CodeCompletionModel* model, m_presentationModel->completionModels())
+      model->completionInvoked(view(), word, invocationType);
+
+    m_presentationModel->setCurrentCompletion(view()->doc()->text(KTextEditor::Range(m_completionRange->start(), view()->cursorPosition())));
+  }
+  
+  connect(this->model(), SIGNAL(contentGeometryChanged()), this, SLOT(modelContentChanged()));
+  //Now that all models have been notified, check whether the widget should be displayed instantly
+  modelContentChanged();
+}
+
+void KateCompletionWidget::updateAndShow()
+{
+  setUpdatesEnabled(false);
+
   updatePosition(true);
 
   if (!m_presentationModel->completionModels().isEmpty()) {
     show();
-
-    foreach (KTextEditor::CodeCompletionModel* model, m_presentationModel->completionModels()) {
-      model->completionInvoked(view(), word, invocationType);
-    }
-
     m_entryList->resizeColumns(false, true);
-
-    m_presentationModel->setCurrentCompletion(view()->doc()->text(KTextEditor::Range(m_completionRange->start(), view()->cursorPosition())));
-
+    
     m_argumentHintModel->buildRows();
-    if( m_argumentHintModel->rowCount(QModelIndex()) != 0 ) {
-      m_dontShowArgumentHints = false;
-      updateArgumentHintGeometry();
-    }
+    if( m_argumentHintModel->rowCount(QModelIndex()) != 0 )
+      argumentHintsChanged(true);
   }
-  
+
   setUpdatesEnabled(true);
 }
 
@@ -466,12 +497,20 @@ void KateCompletionWidget::execute(bool shift)
     return abortCompletion();
 
   toExecute = m_presentationModel->mapToSource(toExecute);
+
+  if (!toExecute.isValid()) {
+    kWarning() << k_funcinfo << "Could not map index" << m_entryList->selectionModel()->currentIndex() << "to source index.";
+    return abortCompletion();
+  }
+
   KTextEditor::Cursor start = m_completionRange->start();
 
   // encapsulate all editing as being from the code completion, and undo-able in one step.
   view()->doc()->editStart(true, Kate::CodeCompletionEdit);
 
   KTextEditor::CodeCompletionModel* model = static_cast<KTextEditor::CodeCompletionModel*>(const_cast<QAbstractItemModel*>(toExecute.model()));
+  Q_ASSERT(model);
+
   model->executeCompletionItem(view()->document(), *m_completionRange, toExecute.row());
 
   view()->doc()->editEnd();
