@@ -633,6 +633,14 @@ void KateBuffer::removeLine(int i)
   if (i > editTagLineEnd)
     editTagLineEnd = i;
 
+  // make sure tags do not reach past the last line
+  // see https://bugs.kde.org/show_bug.cgi?id=152497
+  if (editTagLineEnd >= m_lines.size())
+    editTagLineEnd = m_lines.size() - 1;
+
+  if (editTagLineStart > editTagLineEnd)
+    editTagLineStart = editTagLineEnd;
+
   // line removed
   editTagLineFrom = true;
 
@@ -715,7 +723,7 @@ void KateBuffer::updatePreviousNotEmptyLine(int current_line,bool addindent,int
   bool retVal_folding = false;
   m_regionTree.updateLine (current_line, &foldingList, &retVal_folding, true,false);
 
-  emit tagLines (current_line, current_line);
+  // tagLines() is emitted from KatBuffer::doHighlight()!
 }
 
 void KateBuffer::addIndentBasedFoldingInformation(QVector<int> &foldingList,int linelength,bool addindent,int deindent)
