@@ -111,6 +111,7 @@ class KateDocument : public KTextEditor::Document,
                   QWidget *parentWidget = 0, QObject * = 0);
     ~KateDocument ();
 
+    using ReadWritePart::closeUrl;
     bool closeUrl();
 
     KTextEditor::Editor *editor ();
@@ -414,6 +415,12 @@ class KateDocument : public KTextEditor::Document,
     QVector<KTextEditor::Range> searchRegex (const KTextEditor::Range & inputRange,
         QRegExp & regexp, bool backwards = false);
 
+  private:
+    /**
+     * Return a widget suitable to be used as a dialog parent.
+     */
+    QWidget * dialogParent();
+
   /*
    * Public string processing helpers
    */
@@ -761,6 +768,12 @@ class KateDocument : public KTextEditor::Document,
 
   public:
     void addView(KTextEditor::View *);
+    /** removes the view from the list of views. The view is *not* deleted.
+     * That's your job. Or, easier, just delete the view in the first place.
+     * It will remove itself. TODO: this could be converted to a private slot
+     * connected to the view's destroyed() signal. It is not currently called
+     * anywhere except from the KateView destructor.
+     */
     void removeView(KTextEditor::View *);
     void setActiveView(KTextEditor::View*);
 
@@ -1033,7 +1046,7 @@ class KateDocument : public KTextEditor::Document,
   public:
     void makeAttribs (bool needInvalidate = true);
 
-    static bool checkOverwrite( KUrl u );
+    static bool checkOverwrite( KUrl u, QWidget *parent );
 
   /**
    * Configuration
