@@ -33,6 +33,7 @@
 #include <ktexteditor/variableinterface.h>
 #include <ktexteditor/smartinterface.h>
 #include <ktexteditor/smartrange.h>
+#include <ktexteditor/rangefeedback.h>
 
 #include <kconfig.h>
 #include <kdialog.h>
@@ -47,6 +48,7 @@
 #include <kpagewidgetmodel.h>
 #include <ktoggleaction.h>
 #include <kconfiggroup.h>
+#include <kcolorscheme.h>
 
 #include <QtCore/QRegExp>
 #include <QtCore/QString>
@@ -85,8 +87,7 @@ QVariant DocWordCompletionModel::data(const QModelIndex& index, int role) const
   switch ( role )
   {
     case Qt::DisplayRole:
-      if ( index.column() == Name )
-        return m_matches.at( index.row() );
+      return m_matches.at( index.row() );
     case CompletionRole:
       return (int)FirstProperty|LastProperty|Public;
     case ScopeIndex:
@@ -251,6 +252,7 @@ struct DocWordCompletionPluginViewPrivate
   KToggleAction *autopopup; // for accessing state
   uint treshold;        // the required length of a word before popping up the completion list automatically
   int directionalPos;   // be able to insert "" at the correct time
+  bool isCompleting; // true when the directional completion is doing a completion
 };
 
 DocWordCompletionPluginView::DocWordCompletionPluginView( uint treshold,
@@ -264,9 +266,9 @@ DocWordCompletionPluginView::DocWordCompletionPluginView( uint treshold,
     d( new DocWordCompletionPluginViewPrivate )
 {
 //   setObjectName( name );
-
+  d->isCompleting = false;
   d->treshold = treshold;
-  d->dcRange = KTextEditor::Range();
+  d->dcRange = KTextEditor::Range::invalid();
   KTextEditor::SmartInterface *si =
      qobject_cast<KTextEditor::SmartInterface*>( m_view->document() );
 
@@ -274,16 +276,12 @@ DocWordCompletionPluginView::DocWordCompletionPluginView( uint treshold,
     return;
 
   d->liRange = si->newSmartRange();
-//   d->liRange->setInsertBehavior(KTextEditor::SmartRange::ExpandRight|KTextEditor::SmartRange::ExpandLeft);
 
-  KTextEditor::Attribute::Ptr a( new KTextEditor::Attribute() );
-  QColor bg = Qt::blue;
-  bg.setAlpha(0x88);
-  a->setBackground( QBrush(bg) );
-//   a->setForeground( QBrush(Qt:white) );
+  KColorScheme colors(QPalette::Active);
+  KTextEditor::Attribute::Ptr a = KTextEditor::Attribute::Ptr( new KTextEditor::Attribute() );
+  a->setBackground( colors.background(KColorScheme::ActiveBackground) );
+  a->setForeground( colors.foreground(KColorScheme::ActiveText) ); // ### this does 0
   d->liRange->setAttribute( a );
-  si->addHighlightToView( m_view, d->liRange, false );
-//   si->addHighlightToDocument( d->liRange, false );
 
   view->insertChildClient( this );
 
@@ -380,7 +378,7 @@ void DocWordCompletionPluginView::popupCompletionList()
   if ( ! m_dWCompletionModel->rowCount(QModelIndex()) ) return;
 
   KTextEditor::CodeCompletionInterface *cci = qobject_cast<KTextEditor::CodeCompletionInterface *>( m_view );
-  if ( cci )
+  if ( cci && ! cci->isCompletionActive() )
     cci->startCompletion( r, m_dWCompletionModel );
 }
 
@@ -425,12 +423,18 @@ void DocWordCompletionPluginView::shellComplete()
 
   QString partial = findLongestUnique( matches, r.columnWidth() );
 
-  if ( partial.length() == r.columnWidth() )
+  if ( ! partial.length() )
     popupCompletionList();
 
   else
   {
     m_view->document()->insertText( r.end(), partial.mid( r.columnWidth() ) );
+    KTextEditor::SmartInterface *si = qobject_cast<KTextEditor::SmartInterface*>( m_view->document() );
+    if ( si ) {
+      si->addHighlightToView( m_view, d->liRange, true );
+      d->liRange->setRange( KTextEditor::Range( r.end(), partial.length() - r.columnWidth() ) );
+      connect( m_view, SIGNAL(cursorPositionChanged(KTextEditor::View*, const KTextEditor::Cursor&)), this, SLOT(slotCursorMoved()) );
+    }
   }
 }
 
@@ -445,13 +449,7 @@ void DocWordCompletionPluginView::complete( bool fw )
   int inc = fw ? 1 : -1;
   KTextEditor::Document *doc = m_view->document();
 
-  /* IF the current line is equal to the previous line
-     AND the position - the length of the last inserted string
-          is equal to the old position
-     AND the lastinsertedlength last characters of the word is
-          equal to the last inserted string
-          */
-  if ( r.start() == d->dcRange.start() && r.end() >= d->dcRange.end() )
+  if ( d->dcRange.isValid() )
   {
     //kDebug()<<"CONTINUE "<<d->dcRange;
     // this is a repeted activation
@@ -482,6 +480,14 @@ void DocWordCompletionPluginView::complete( bool fw )
     d->liRange->setRange( KTextEditor::Range( r.end(), 0 ) );
     d->dcCursor = r.start();
     d->directionalPos = inc;
+
+  KTextEditor::SmartInterface *si =
+     qobject_cast<KTextEditor::SmartInterface*>( m_view->document() );
+  if ( si )
+    si->addHighlightToView( m_view, d->liRange, true );
+
+    connect( m_view, SIGNAL(cursorPositionChanged(KTextEditor::View*, const KTextEditor::Cursor&)), this, SLOT(slotCursorMoved()) );
+
   }
 
   d->re.setPattern( "\\b" + doc->text( d->dcRange ) + "(\\w+)" );
@@ -499,21 +505,16 @@ void DocWordCompletionPluginView::complete( bool fw )
     {
       //kDebug()<<"USABLE MATCH";
       QString m = d->re.cap( 1 );
-      if ( m != doc->text( *d->liRange ) )
+      if ( m != doc->text( *d->liRange ) && (d->dcCursor.line() != d->dcRange.start().line() || pos != d->dcRange.start().column() ) )
       {
         // we got good a match! replace text and return.
+        d->isCompleting = true;
         doc->replaceText( *d->liRange, m );
-kDebug()<<"liRange:"<<*d->liRange;
-
         d->liRange->setRange( KTextEditor::Range( d->dcRange.end(), m.length() ) );
 
-kDebug()<<"lirange background:"<<d->liRange->attribute()->background();
         d->dcCursor.setColumn( pos ); // for next try
-  KTextEditor::SmartInterface *si =
-     qobject_cast<KTextEditor::SmartInterface*>( m_view->document() );
-foreach(KTextEditor::SmartRange* range, si->viewHighlights(m_view))
-kDebug()<<"highlighted range:"<<*range;
 
+        d->isCompleting = false;
         return;
       }
 
@@ -565,6 +566,20 @@ kDebug()<<"highlighted range:"<<*range;
   } // while true
 }
 
+void DocWordCompletionPluginView::slotCursorMoved()
+{
+  if ( d->isCompleting) return;
+
+  d->dcRange = KTextEditor::Range::invalid();
+
+  disconnect( m_view, SIGNAL(cursorPositionChanged(KTextEditor::View*, const KTextEditor::Cursor&)), this, SLOT(slotCursorMoved()) );
+
+  KTextEditor::SmartInterface *si =
+     qobject_cast<KTextEditor::SmartInterface*>( m_view->document() );
+  if ( si )
+    si->removeHighlightFromView( m_view, d->liRange );
+}
+
 // Contributed by <brain@hdsnet.hu> FIXME
 QString DocWordCompletionPluginView::findLongestUnique( const QStringList &matches, int lead ) const
 {
