@@ -1,11 +1,12 @@
 /*
    This file is part of the KDE libraries
-   Copyright (c) 2005-2007 David Jarvie <software@astrojar.org.uk>
+   Copyright (c) 2005-2007 David Jarvie <djarvie@kde.org>
    Copyright (c) 2005 S.R.Haque <srhaque@iee.org>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -113,6 +114,8 @@ class KSystemTimeZonesPrivate : public KTimeZones
 {
 public:
     static KSystemTimeZonesPrivate *instance();
+    static KTzfileTimeZoneSource *tzfileSource();
+    static void setLocalZone();
     static void cleanup();
     static void readConfig(bool init);
 #ifndef Q_OS_WIN
@@ -124,7 +127,6 @@ public:
     static QString m_zoneinfoDir;
     static QString m_zonetab;
     static KSystemTimeZoneSource *m_source;
-    static KTzfileTimeZoneSource *m_tzfileSource;
 
 private:
     KSystemTimeZonesPrivate() {}
@@ -135,6 +137,7 @@ private:
 
     static KSystemTimeZones *m_parent;
     static KSystemTimeZonesPrivate *m_instance;
+    static KTzfileTimeZoneSource *m_tzfileSource;
 };
 
 KTimeZone                KSystemTimeZonesPrivate::m_localZone;
@@ -146,6 +149,16 @@ KTzfileTimeZoneSource   *KSystemTimeZonesPrivate::m_tzfileSource = 0;
 KSystemTimeZones        *KSystemTimeZonesPrivate::m_parent = 0;
 KSystemTimeZonesPrivate *KSystemTimeZonesPrivate::m_instance = 0;
 
+KTzfileTimeZoneSource *KSystemTimeZonesPrivate::tzfileSource()
+{
+    if (!m_tzfileSource)
+    {
+        instance();
+        m_tzfileSource = new KTzfileTimeZoneSource(m_zoneinfoDir);
+    }
+    return m_tzfileSource;
+}
+
 
 KSystemTimeZones::KSystemTimeZones()
   : d(0)
@@ -180,12 +193,7 @@ KTimeZones *KSystemTimeZones::timeZones()
 
 KTimeZone KSystemTimeZones::readZone(const QString &name)
 {
-    if (!KSystemTimeZonesPrivate::m_tzfileSource)
-    {
-        KSystemTimeZonesPrivate::instance();
-        KSystemTimeZonesPrivate::m_tzfileSource = new KTzfileTimeZoneSource(KSystemTimeZonesPrivate::m_zoneinfoDir);
-    }
-    return KTzfileTimeZone(KSystemTimeZonesPrivate::m_tzfileSource, name);
+    return KTzfileTimeZone(KSystemTimeZonesPrivate::tzfileSource(), name);
 }
 
 const KTimeZones::ZoneMap KSystemTimeZones::zones()
@@ -217,11 +225,11 @@ void KSystemTimeZones::zonetabChanged(const QString &zonetab)
 
 void KSystemTimeZones::zoneDefinitionChanged(const QString &zone)
 {
-        // No need to do anything when the definition (as opposed to the
-        // identity) of the local zone changes, since the updated details
-        // will always be accessed by the system library calls to fetch
-        // local zone information.
-        Q_UNUSED(zone)
+    // No need to do anything when the definition (as opposed to the
+    // identity) of the local zone changes, since the updated details
+    // will always be accessed by the system library calls to fetch
+    // local zone information.
+    Q_UNUSED(zone)
 }
 
 // Perform initialization, create the unique KSystemTimeZones instance,
@@ -235,23 +243,8 @@ KSystemTimeZonesPrivate *KSystemTimeZonesPrivate::instance()
 
         // A KSystemTimeZones instance is required only to catch D-Bus signals.
         m_parent = new KSystemTimeZones;
-        // Ensure that the KDED time zones module has initialized
+        // Ensure that the KDED time zones module has initialized. The call loads the module on demand.
         QDBusInterface *ktimezoned = new QDBusInterface("org.kde.kded", "/modules/ktimezoned", KTIMEZONED_DBUS_IFACE);
-        if (!ktimezoned->isValid())
-        {
-kDebug(161)<<"instance(): load KDED module";
-            // Need to load the KDED time zones module
-            delete ktimezoned;
-            ktimezoned = 0;
-            QDBusInterface kded("org.kde.kded", "/kded", "org.kde.kded");
-            QDBusReply<bool> reply = kded.call("loadModule", "ktimezoned");
-            if (!reply.isValid())
-                kError(161) << "KSystemTimeZones: could not load ktimezoned kded module: " << reply.error().message() << endl;
-            else if (!reply)
-                kError(161) << "KSystemTimeZones: could not load ktimezoned kded module" << endl;
-            ktimezoned = new QDBusInterface("org.kde.kded", "/modules/ktimezoned", KTIMEZONED_DBUS_IFACE);
-kDebug(161)<<"instance(): ... loaded";
-        }
         QDBusReply<void> reply = ktimezoned->call("initialize", false);
         if (!reply.isValid())
             kError(161) << "KSystemTimeZones: ktimezoned initialize() D-Bus call failed: " << reply.error().message() << endl;
@@ -270,7 +263,7 @@ kDebug(161)<<"instance(): ... initialised";
         if (!m_zonetab.isEmpty())
             m_instance->readZoneTab(false);
 #endif
-        m_localZone = m_instance->zone(m_localZoneName);
+        setLocalZone();
         if (!m_localZone.isValid())
             m_localZone = KTimeZone::utc();   // ensure a time zone is always returned
 
@@ -289,10 +282,31 @@ void KSystemTimeZonesPrivate::readConfig(bool init)
     m_zonetab       = group.readEntry("Zonetab");
     m_localZoneName = group.readEntry("LocalZone");
     if (!init)
-        m_localZone = m_instance->zone(m_localZoneName);
+        setLocalZone();
     kDebug(161) << "readConfig(): local zone=" << m_localZoneName;
 }
 
+void KSystemTimeZonesPrivate::setLocalZone()
+{
+    if (m_localZoneName.startsWith('/'))
+    {
+        // The time zone is specified by a file outside the zoneinfo directory
+        m_localZone = KTzfileTimeZone(KSystemTimeZonesPrivate::tzfileSource(), m_localZoneName);
+        if (m_localZone.isValid() && m_instance)
+        {
+            // Add the new time zone to the list
+            KTimeZone oldzone = m_instance->zone(m_localZoneName);
+            if (!oldzone.isValid() || oldzone.type() != "KTzfileTimeZone")
+            {
+                m_instance->remove(oldzone);
+                m_instance->add(m_localZone);
+            }
+        }
+    }
+    else
+        m_localZone = m_instance->zone(m_localZoneName);
+}
+
 void KSystemTimeZonesPrivate::cleanup()
 {
     delete m_parent;
