@@ -47,9 +47,10 @@
 
 static QString cleanpath( const QString &_path, bool cleanDirSeparator, bool decodeDots )
 {
-  if (_path.isEmpty()) return QString();
+  if (_path.isEmpty())
+      return QString();
 
-  if (QDir::isRelativePath(_path))
+  if (QFileInfo(_path).isRelative())
      return _path; // Don't mangle mailto-style URLs
 
   QString path = _path;
@@ -104,8 +105,8 @@ static QString cleanpath( const QString &_path, bool cleanDirSeparator, bool dec
   }
 
 #ifdef Q_WS_WIN // prepend drive letter if exists (js)
-  if (orig_pos >= 2 && isalpha(path[0].toLatin1()) && path[1]==':') {
-    result.prepend(QString(path[0])+':');
+  if (orig_pos >= 2 && path[0].isLetter() && path[1] == QLatin1Char(':') ) {
+    result.prepend(QString(path[0]) + QLatin1Char(':') );
   }
 #endif
 
@@ -150,6 +151,11 @@ KUrl::List::List(const KUrl &url)
     append( url );
 }
 
+KUrl::List::List(const QList<KUrl> &list)
+    : QList<KUrl>(list)
+{
+}
+
 KUrl::List::List(const QStringList &list)
 {
   for (QStringList::ConstIterator it = list.begin();
@@ -188,10 +194,10 @@ void KUrl::List::populateMimeData( QMimeData* mimeData,
     }
 
     QByteArray uriListData;
-    for ( QList<QByteArray>::const_iterator it = urlStringList.begin(), end = urlStringList.end()
-                                                 ; it != end ; ++it ) {
-        uriListData += (*it);
-        uriListData += "\r\n";
+    for (int i = 0, n = urlStringList.count(); i < n; ++i) {
+      uriListData += urlStringList.at(i);
+        if (i < n-1)
+          uriListData += "\r\n";
     }
     mimeData->setData( "text/uri-list", uriListData );
 
@@ -306,25 +312,25 @@ KUrl::KUrl( const QString &str )
 {
   if ( !str.isEmpty() ) {
 #ifdef Q_WS_WIN
-   kDebug(126) << "KUrl::KUrl " << str.toAscii().data();
-    if ( str.startsWith("file:///") ) {
-      kDebug(126) << "KUrl::KUrl " << "converting file schema from: " << str.toAscii().data() << " to: " << QUrl::fromPercentEncoding( str.mid(8).toLatin1() );
-      setPath( QUrl::fromPercentEncoding( str.mid(8).toLatin1() ) );
-   	}
-    else if ( str.length() > 9 && str.startsWith("file://") && str[7].isLetter() && str[8] == QLatin1Char(':') ) {
-      kDebug(126) << "KUrl::KUrl " << "converting file schema from: " << str.toAscii().data() << " to: " << QUrl::fromPercentEncoding( str.mid(7).toLatin1() );
-      setPath( QUrl::fromPercentEncoding( str.mid(7).toLatin1() ) );
+    int len = str.length();
+    QString pathToSet;
+    kDebug(126) << "KUrl::KUrl ( const QString &str = " << str.toAscii().data() << " )";
+    if ( len > 9 && str.startsWith( QLatin1String( "file://" ) ) && str[7].isLetter() && str[8] == QLatin1Char(':') )
+      pathToSet = str.mid(7);
+    else if ( len > 2 && str[0] == QLatin1Char('/') && str[1].isLetter() && str[2] == QLatin1Char(':') )
+      pathToSet = str.mid(1);
+    else if ( len >= 2 && str[0].isLetter() && str[1] == QLatin1Char(':') )
+      pathToSet = str;
+    if ( !pathToSet.isEmpty() ) {
+      setPath( QDir::fromNativeSeparators( pathToSet ) );
     }
-    else if ( str.length() > 2 && str[0] == QLatin1Char('/') && str[1].isLetter() && str[2] == QLatin1Char(':') )
-      setPath(str.mid(2));
-    else if ( str.length() > 2 && str[0].isLetter() &&  str[1] == QLatin1Char(':') )
-      setPath( str );
 #else
     if ( str[0] == QLatin1Char('/') || str[0] == QLatin1Char('~') )
       setPath( str );
 #endif
-    else
-      setEncodedUrl( str.toUtf8(), QUrl::TolerantMode );
+    else {
+      _setEncodedUrl( str.toUtf8() );
+    }
   }
 }
 
@@ -344,7 +350,7 @@ KUrl::KUrl( const char * str )
       setPath( QString::fromUtf8( str ) );
 #endif
     else
-      setEncodedUrl( str, QUrl::TolerantMode );
+      _setEncodedUrl( str );
   }
 }
 
@@ -355,7 +361,7 @@ KUrl::KUrl( const QByteArray& str )
 #ifdef Q_WS_WIN
     kDebug(126) << "KUrl::KUrl " << " " << str.data();
     if ( str[0] == '/' && (str[1] >= 'A' && str[1] <= 'Z' || str[1] >= 'a' && str[1] <= 'z') && str[2] == ':' )
-    	  setPath( QString::fromUtf8( str.mid(2) ) );
+    	  setPath( QString::fromUtf8( str.mid( 1 ) ) );
     else if ( (str[0] >= 'A' && str[0] <= 'Z' || str[0] >= 'a' && str[0] <= 'z')  &&  str[1] == ':' )
         setPath( QString::fromUtf8( str ) );
 #else
@@ -363,7 +369,7 @@ KUrl::KUrl( const QByteArray& str )
       setPath( QString::fromUtf8( str ) );
 #endif
     else
-      setEncodedUrl( str, QUrl::TolerantMode );
+      _setEncodedUrl( str );
   }
 }
 
@@ -455,11 +461,7 @@ KUrl::KUrl( const KUrl& _u, const QString& _rel_url )
     else
     {
        if ( strPath.isEmpty() )
-#ifdef Q_WS_WIN
-          strPath = QLatin1Char('\\');
-#else
           strPath = QLatin1Char('/');
-#endif
     }
     setPath( strPath );
     //kDebug(126) << "url()=" << url() << " rUrl=" << rUrl;
@@ -602,7 +604,7 @@ void KUrl::setFileName( const QString& _txt )
   //QString path = m_strPath_encoded.isEmpty() ? m_strPath : m_strPath_encoded;
   QString path = this->path();
   if ( path.isEmpty() )
-    path = "/";
+    path = QDir::rootPath();
   else
   {
     int lastSlash = path.lastIndexOf( QLatin1Char('/') );
@@ -659,7 +661,7 @@ static QString trailingSlash( KUrl::AdjustPathOption trailing, const QString &pa
   }
   else if ( trailing == KUrl::RemoveTrailingSlash )
   {
-    if ( result == "/" )
+    if ( result == QLatin1String("/") )
       return result;
     int len = result.length();
     while (len > 1 && result[ len - 1 ] == QLatin1Char('/'))
@@ -756,19 +758,22 @@ void KUrl::setEncodedPathAndQuery( const QString& _txt )
 
 QString KUrl::path( AdjustPathOption trailing ) const
 {
+#ifdef Q_OS_WIN
+  // throw away the first '/' when it's a local file
+  return trailingSlash( trailing, isLocalFile() ? QUrl::toLocalFile() : QUrl::path() );
+#else
   return trailingSlash( trailing, QUrl::path() );
+#endif
 }
 
 QString KUrl::toLocalFile( AdjustPathOption trailing ) const
 {
-  if ( trailing == KUrl::LeaveTrailingSlash )
-    return QUrl::toLocalFile();
   return trailingSlash( trailing, QUrl::toLocalFile() );
 }
 
 bool KUrl::isLocalFile() const
 {
-  if ( ( scheme() != "file" ) || hasSubUrl() )
+  if ( ( scheme() != QLatin1String("file") ) || hasSubUrl() )
      return false;
 
   if (host().isEmpty() || (host() == QLatin1String("localhost")))
@@ -782,7 +787,7 @@ bool KUrl::isLocalFile() const
   for(char *p = hostname; *p; p++)
      *p = tolower(*p);
 
-  return (host() == hostname);
+  return (host() == QString::fromLatin1( hostname ));
 }
 
 void KUrl::setFileEncoding(const QString &encoding)
@@ -923,9 +928,6 @@ QString KUrl::prettyUrl( AdjustPathOption trailing ) const
 
   QString tmp = userName();
   if (!tmp.isEmpty()) {
-    if (!hasPass())
-      tmp = KStringHandler::csqueeze(tmp, 16);  
-      
     result += QUrl::toPercentEncoding(tmp);
     result += QLatin1Char('@');
   }
@@ -1237,7 +1239,11 @@ bool KUrl::cd( const QString& _dir )
   }
 
   // absolute path ?
+#ifdef Q_OS_WIN
+  if ( !QFileInfo(_dir).isRelative() )
+#else
   if ( _dir[0] == QLatin1Char('/') )
+#endif
   {
     //m_strPath_encoded.clear();
     setPath( _dir );
@@ -1361,10 +1367,10 @@ bool KUrl::hasHTMLRef() const
 
 void KUrl::setDirectory( const QString &dir)
 {
-  if ( dir.endsWith('/'))
+  if ( dir.endsWith(QLatin1Char('/')))
      setPath(dir);
   else
-     setPath(dir+'/');
+     setPath(dir + QLatin1Char('/'));
 }
 
 void KUrl::setQuery( const QString &_txt )
@@ -1394,6 +1400,13 @@ QString KUrl::query() const
   return QString( QChar( '?' ) ) + QString::fromAscii( encodedQuery() );
 }
 
+void KUrl::_setEncodedUrl(const QByteArray& url)
+{
+  setEncodedUrl(url, QUrl::TolerantMode);
+  if (!isValid()) // see unit tests referring to N183630/task 183874
+    setUrl(url, QUrl::TolerantMode);
+}
+
 bool urlcmp( const QString& _url1, const QString& _url2 )
 {
   return QUrl( _url1, QUrl::TolerantMode ) == QUrl( _url2, QUrl::TolerantMode );
