@@ -134,6 +134,16 @@ class KLocalizedStringPrivate;
  * it may happen that you add context or plural to previously basic message,
  * but forget to change the name of the call.
  *
+ * All message strings are expected to pass for well-formed XML, whether or
+ * not the output device supports some form of markup.
+ * Thus, predefined XML entities are always available: &amp;lt;, &amp;gt;,
+ * &amp;amp;, &amp;apos;, and &amp;quot;. E.g. if you need a non-tag
+ * less-than sign, use &amp;lt; entity instead.
+ * The exception to the well-formed XML requirement is the ampersand (&amp;),
+ * which is used a lot for marking accelerators, so you should not write it
+ * as &amp;amp; (except in the very unlikely case when the construct with
+ * the naked ampersand can be interpreted as an entity in itself).
+ *
  * \section spec_usage Specialized Usage
  *
  * There are some situations where i18n* calls are not sufficient or
@@ -178,6 +188,23 @@ class KLocalizedStringPrivate;
  *   ki18n("Welcome").toString(myLocale);
  * \endcode
  *
+ * Translators have a capability to script translations at runtime, which is
+ * for the most part transparent to the programmer. However, sometimes the
+ * programmer may help by providing some @e dynamic context to the message,
+ * using the inContext method of KLocalizedString.  Unlike the ordinary
+ * context, this one changes at runtime; translators have the means to fetch
+ * it and use it to script the translation properly. An example:
+ * \code
+ *   KLocalizedString ks = ki18nc("%1 is user name; may have "
+ *                                "dynamic context gender=[male,female]",
+ *                                "%1 went offline");
+ *   if (knownUsers.contains(user) && !knownUsers[user].gender.isEmpty()) {
+ *     ks = ks.inContext("gender", knownUsers[user].gender);
+ *   }
+ *   QString msg = ks.subs(user).toString();
+ * \endcode
+ *  Several dynamic contexts, with different keys, can be added like this.
+ *
  * \section subs_notes Placeholder Substitution
  *
  * Hopefully, for the most part placeholders are being substituted the way
@@ -198,10 +225,18 @@ class KLocalizedStringPrivate;
  * \li If none of the arguments supplied to a plural call is integer-valued,
  *     you will get an error mark in message at runtime (in debug mode).
  *
+ * \li Plain number arguments will be normally formatted as if they denote
+ *     amounts, according to language rules (thousands separation, etc.)
+ *     But sometimes a number is a numerical identifier (e.g. port number),
+ *     and to be treated as such, wrap the placeholder with the numid tag:
+ *     \code
+ *       QString msg = i18n("Using port <numid>%1</numid>", port);
+ *     \endcode
+ *
  * \section other_ref Further References
  *
  * <a href="http://techbase.kde.org/">KDE Techbase</a> contains a
- * <a href="http://techbase.kde.org/Development/Tutorials/Localization/i18n">
+ * <a href="http://techbase.kde.org/Development/Tutorials/Localization">
  * series of tutorials</a> on preparing the code for localization (and on
  * internationalization process in general), where the intended patterns of
  * usage of i18n API are covered in great detail.
@@ -419,6 +454,16 @@ public:
     KLocalizedString subs (const QString &a, int fieldWidth = 0,
                            const QChar &fillChar = QLatin1Char(' ')) const;
 
+    /**
+     * Adds dynamic context to the message.
+     *
+     * @param key context key
+     * @param text context value
+     * @return resultant KLocalizedString
+     */
+    KLocalizedString inContext (const QString &key,
+                                const QString &text) const;
+
     /**
      * @internal Called from KLocale on addition or removal of catalogs.
      */
