@@ -19,6 +19,7 @@
 
 #include <qtest_kde.h>
 #include "kconfigtest.h"
+#include <kdesktopfile.h>
 #include <kstandarddirs.h>
 #include "kconfigtest.moc"
 
@@ -849,6 +850,38 @@ void KConfigTest::testAddConfigSources()
     QCOMPARE(basegrp2.readEntry("New Entry Base Only", ""), QString("SomeValue"));
 }
 
+void KConfigTest::testConfigCopyTo()
+{
+    KConfig cf1("kconfigtest");
+    {
+        // Prepare source file
+        KConfigGroup group(&cf1, "CopyToTest");
+        group.writeEntry("Type", "Test");
+        cf1.sync();
+    }
+
+    {
+        // Copy to "destination"
+        const QString destination = KStandardDirs::locateLocal("config", "kconfigcopytotest");
+        QFile::remove(destination);
+        KConfig cf2;
+        cf1.copyTo(destination, &cf2);
+        KConfigGroup group2(&cf2, "CopyToTest");
+        QString testVal = group2.readEntry("Type");
+        QCOMPARE(testVal, QString("Test"));
+        group2.writeEntry("AnotherKey", "Test Worked"); // for sync to have something to sync
+        cf2.sync();
+        QVERIFY(QFile::exists(destination));
+    }
+
+    // Check copied config file on disk
+    KConfig cf3("kconfigcopytotest");
+    KConfigGroup group3(&cf3, "CopyToTest");
+    QString testVal = group3.readEntry("Type");
+    QCOMPARE(testVal, QString("Test"));
+    QCOMPARE(group3.readEntry("AnotherKey"), QString("Test Worked"));
+}
+
 void KConfigTest::testKAboutDataOrganizationDomain()
 {
     KAboutData data( "app", 0, ki18n("program"), "version",
@@ -879,3 +912,19 @@ QList<QByteArray> KConfigTest::readLines()
     } while(!line.isEmpty());
     return lines;
 }
+
+void KConfigTest::testCreateDir()
+{
+    // Test auto-creating the parent directory when needed (KConfigIniBackend::createEnclosing)
+    QString kdehome = QDir::home().canonicalPath() + "/.kde-unit-test";
+    QString subdir = kdehome + "/newsubdir";
+    QString file = subdir + "/foo.desktop";
+    QFile::remove(file);
+    QDir().rmdir(subdir);
+    QVERIFY(!QDir().exists(subdir));
+    KDesktopFile desktopFile(file);
+    desktopFile.desktopGroup().writeEntry("key", "value");
+    desktopFile.sync();
+    QVERIFY(QFile::exists(file));
+}
+
