@@ -53,9 +53,11 @@ private Q_SLOTS:
     void testGroupEscape();
     void testRevertAllEntries();
     void testChangeGroup();
+    void testConfigCopyTo();
     void testKdeglobals();
     void testSubGroup();
     void testAddConfigSources();
+    void testCreateDir();
 
     // unrelated
     void testKAboutDataOrganizationDomain();
