@@ -72,10 +72,21 @@ void KMimeTypeTest::initTestCase()
 
     if ( mustUpdateKSycoca ) {
         // Update ksycoca in ~/.kde-unit-test after creating the above
-        QProcess::execute( KGlobal::dirs()->findExe(KBUILDSYCOCA_EXENAME), QStringList() << "--noincremental" );
+        QProcess::execute( KGlobal::dirs()->findExe(KBUILDSYCOCA_EXENAME) );
     }
 }
 
+void KMimeTypeTest::cleanupTestCase()
+{
+    // If I want the konqueror unit tests to work, then I better not have a non-working part
+    // as the preferred part for text/plain...
+    const QString fakePart = KStandardDirs::locateLocal("services", "faketextpart.desktop");
+    QFile::remove(fakePart);
+    const QString fakePlugin = KStandardDirs::locateLocal("services", "faketextplugin.desktop");
+    QFile::remove(fakePlugin);
+    QProcess::execute( KGlobal::dirs()->findExe(KBUILDSYCOCA_EXENAME) );
+}
+
 static void checkIcon( const KUrl& url, const QString& expectedIcon )
 {
     QString icon = KMimeType::iconNameForUrl( url );
@@ -144,6 +155,7 @@ void KMimeTypeTest::testFindByPathUsingFileName_data()
     QTest::newRow("old kdelnk file is x-desktop too") << "foo.kdelnk" << "application/x-desktop";
     QTest::newRow("double-extension file") << "foo.tar.bz2" << "application/x-bzip-compressed-tar";
     QTest::newRow("single-extension file") << "foo.bz2" << "application/x-bzip";
+//    QTest::newRow(".doc should assume msword") << "somefile.doc" << "application/msword";
     QTest::newRow("glob that uses [] syntax, 1") << "Makefile" << "text/x-makefile";
     QTest::newRow("glob that uses [] syntax, 2") << "makefile" << "text/x-makefile";
     QTest::newRow("glob that ends with *, no extension") << "README" << "text/x-readme";
@@ -328,6 +340,7 @@ void KMimeTypeTest::testFindByContent_data()
     QTest::newRow("simple text") << QByteArray("Hello world") << "text/plain";
     QTest::newRow("html: <html>") << QByteArray("<html>foo</html>") << "text/html";
 #if 0 // currently buggy, xml and html have conflicting magic rules, discussed on xdg list.
+      // Solution: making the magic rules for xml priority 40
     QTest::newRow("html: comment+<html>") << QByteArray("<!--foo--><html>foo</html>") << "text/html";
 #endif
 #if 0 // https://bugs.freedesktop.org/show_bug.cgi?id=11259
@@ -356,10 +369,20 @@ void KMimeTypeTest::testFindByContent()
 void KMimeTypeTest::testFindByFileContent()
 {
     KMimeType::Ptr mime;
+    int accuracy = 0;
+
     // Calling findByContent on a directory
-    mime = KMimeType::findByFileContent("/");
-    QVERIFY( mime );
-    QCOMPARE( mime->name(), QString::fromLatin1("inode/directory") );
+    mime = KMimeType::findByFileContent("/", &accuracy);
+    QVERIFY(mime);
+    QCOMPARE(mime->name(), QString::fromLatin1("inode/directory"));
+    QCOMPARE(accuracy, 100);
+
+    // Albert calls findByFileContent with a URL instead of a path and gets 11021 as accuracy :)
+    // It was not set inside findByFileContent -> fixed.
+    mime = KMimeType::findByFileContent("file:///etc/passwd" /*bad example code, use a path instead*/, &accuracy);
+    QVERIFY(mime);
+    QCOMPARE(mime->name(), QString::fromLatin1("application/octet-stream"));
+    QCOMPARE(accuracy, 0);
 }
 
 void KMimeTypeTest::testAllMimeTypes()
@@ -413,6 +436,8 @@ void KMimeTypeTest::testMimeTypeParent()
     QCOMPARE(wordperfect->parentMimeType(), QString("application/octet-stream"));
     QVERIFY(wordperfect->is("application/octet-stream"));
 
+    QVERIFY(KMimeType::mimeType("image/svg+xml-compressed")->is("application/x-gzip"));
+
     // Check that msword derives from ole-storage [it didn't in 0.20, but we added it to kde.xml]
     const KMimeType::Ptr msword = KMimeType::mimeType("application/msword");
     QVERIFY(msword);
@@ -434,6 +459,14 @@ void KMimeTypeTest::testMimeTypeParent()
     QVERIFY( derived->is("text/plain") );
     QVERIFY( derived->is("application/octet-stream") );
 
+    // Check that application/x-shellscript inherits from application/x-executable
+    // (Otherwise KRun cannot start shellscripts...)
+    // This is a test for multiple inheritance...
+    const KMimeType::Ptr shellscript = KMimeType::mimeType("application/x-shellscript");
+    QVERIFY(shellscript);
+    QVERIFY(shellscript->is("text/plain"));
+    QVERIFY(shellscript->is("application/x-executable"));
+
     // Check that text/mrml knows that it inherits from text/plain (implicitly)
     const KMimeType::Ptr mrml = KMimeType::mimeType("text/mrml");
     if (!mrml)
@@ -704,6 +737,3 @@ void KMimeTypeTest::testParseMagicFile()
     QCOMPARE(found, expected);
     testBuffer.close();
 }
-
-
-// TODO tests that involve writing a profilerc and checking that the trader is obeying it
