@@ -38,6 +38,13 @@ QTEST_KDEMAIN_CORE( KUrlTest )
 #include <stdio.h>
 #include <stdlib.h>
 
+// needed to build a valid path on windows
+#ifdef Q_OS_WIN
+#define LOCAL_FILE_PREFIX "C:"
+#else
+#define LOCAL_FILE_PREFIX
+#endif
+
 void KUrlTest::testEmptyURL()
 {
   KUrl emptyURL;
@@ -296,8 +303,11 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
   carsten.setPath("/home/gis/src/kde/kdelibs/kfile/.#kfiledetailview.cpp.1.18");
   QCOMPARE( carsten.path(), QString("/home/gis/src/kde/kdelibs/kfile/.#kfiledetailview.cpp.1.18") );
 
+  KUrl longUserName("http://thisisaverylongusername@foobar.com/");
+  QCOMPARE(longUserName.prettyUrl(), QString("http://thisisaverylongusername@foobar.com/"));
+  QCOMPARE(KUrl(longUserName.prettyUrl()).url(), QString("http://thisisaverylongusername@foobar.com/"));
   KUrl whitespaceInUser("http://www.google.com%20%20%20%20%20@foobar.com/");
-  QCOMPARE(whitespaceInUser.prettyUrl(), QString("http://www.go...m%20%20%20%20%20@foobar.com/"));
+  QCOMPARE(whitespaceInUser.prettyUrl(), QString("http://www.google.com%20%20%20%20%20@foobar.com/"));
   KUrl whitespaceInPath("http://www.google.com/foo%20bar/");
   QCOMPARE(whitespaceInPath.prettyUrl(), QString("http://www.google.com/foo bar/"));
   KUrl whitespaceInPath2("http://www.google.com/foo%20%20%20%20%20%20%20bar/");
@@ -634,7 +644,7 @@ void KUrlTest::testPrettyURL()
   KUrl urlWithPass("ftp://user:password@ftp.kde.org/path");
   QCOMPARE( urlWithPass.pass(), QString::fromLatin1( "password" ) );
   QCOMPARE( urlWithPass.prettyUrl(), QString::fromLatin1( "ftp://user@ftp.kde.org/path" ) );
-  
+
   KUrl xmppUri("xmpp:ogoffart@kde.org");
   QCOMPARE( xmppUri.prettyUrl(), QString::fromLatin1( "xmpp:ogoffart@kde.org" ) );
 }
@@ -735,6 +745,21 @@ void KUrlTest::testAdjustPath()
     KUrl u5 = u4;
     u5.adjustPath(KUrl::RemoveTrailingSlash);
     QCOMPARE( u5.url(), u4.url() );
+
+    {
+    KUrl remote("remote:/");
+    QCOMPARE( remote.url(KUrl::RemoveTrailingSlash ), QString("remote:/") );
+    remote.adjustPath( KUrl::RemoveTrailingSlash );
+    QCOMPARE( remote.url(), QString("remote:/") );
+    remote.adjustPath( KUrl::RemoveTrailingSlash );
+    QCOMPARE( remote.url(), QString("remote:/") );
+    }
+
+    {
+    KUrl remote2("remote://");
+    QCOMPARE( remote2.url(), QString("remote:") );
+    QCOMPARE( remote2.url(KUrl::RemoveTrailingSlash ), QString("remote:") );
+    }
 }
 
 void KUrlTest::testIPV6()
@@ -961,6 +986,19 @@ void KUrlTest::testBaseURL() // those are tests for the KUrl(base,relative) cons
   KUrl sadEagleCombined( sadEagleBase, "/personal/diary/rpc.php?C=jsrs1&F=getDiaryDay&P0=[2006-3-8]&U=1141858921458" );
   QCOMPARE( sadEagleCombined.url(), sadEagleExpectedResult.url() );
 
+  KUrl dxOffEagle( KUrl("http://something/other.html"), "newpage.html?[{\"foo: bar\"}]" );
+  //QEXPECT_FAIL("","Issue N183630, task ID 183874", Continue); // Fixed by _setEncodedUrl
+  QVERIFY(dxOffEagle.isValid());
+  //QEXPECT_FAIL("","Issue N183630, task ID 183874", Continue); // Fixed by _setEncodedUrl
+  QCOMPARE(dxOffEagle.url(), QString("http://something/newpage.html?%5B%7B%22foo:%20bar%22%7D%5D") );
+
+  // Shows up in nspluginviewer/flash
+  QString flashRel = "javascript:window.location+\"__flashplugin_unique__\"";
+  KUrl flashUrl(flashRel);
+  QVERIFY(flashUrl.isValid());
+  KUrl flashBase("http://www.youtube.com/?v=JvOSnRD5aNk");
+  KUrl flashComposed(flashBase, flashRel);
+  QCOMPARE(flashComposed.url(), QString("javascript:window.location+%22__flashplugin_unique__%22"));
 }
 
 void KUrlTest::testSubURL()
@@ -1056,6 +1094,11 @@ void KUrlTest::testSetUser()
   QCOMPARE( emptyUserTest2.user(), QString::fromLatin1( "foo" ) );
   emptyUserTest2.setUser( QString() );
   QCOMPARE( emptyUserTest1==emptyUserTest2, true );
+
+  KUrl uga("ftp://ftp.kde.org");
+  uga.setUser("foo@bar");
+  QCOMPARE(uga.user(), QString::fromLatin1("foo@bar"));
+  QCOMPARE(uga.url(), QString::fromLatin1("ftp://foo%40bar@ftp.kde.org"));
 }
 
 void KUrlTest::testComparisons()
@@ -1229,8 +1272,8 @@ void KUrlTest::testBrokenStuff()
   QVERIFY( !weird.isValid() );
 
   weird = "http://strange<username>@ok_hostname/";
-  QVERIFY( !weird.isValid() ); // KDE3: was valid. bah...
-  //QCOMPARE( weird.host(), QString("ok_hostname") );
+  QVERIFY( weird.isValid() ); // KDE3: was valid. Fixed by _setEncodedUrl.
+  QCOMPARE( weird.host(), QString("ok_hostname") );
 
   weird = "http://strange;hostname/";
   QVERIFY( weird.isValid() ); // KDE3: was invalid. bah*2.
@@ -1274,13 +1317,37 @@ void KUrlTest::testBrokenStuff()
 
   broken = "ptal://mlc:usb:PC_970";
   QVERIFY( !broken.isValid() );
-  QSKIP( "QUrl doesn't provide the initial string if it's an invalid url; asked for 4.2...", SkipSingle );
+  QEXPECT_FAIL( "", "QUrl doesn't provide the initial string if it's an invalid url...", Continue );
   QCOMPARE( broken.url(), QString("ptal://mlc:usb:PC_970") ); // FAILS - but we need it...
 
   QUrl brokenUrl( "ptal://mlc:usb:PC_970" );
   QVERIFY( !brokenUrl.isValid() );
+  QEXPECT_FAIL( "", "QUrl doesn't provide the initial string if it's an invalid url...", Continue );
   QCOMPARE( brokenUrl.toString(), QString("ptal://mlc:usb:PC_970") );
+  QEXPECT_FAIL( "", "QUrl doesn't provide the initial string if it's an invalid url...", Continue );
   QCOMPARE( brokenUrl.toEncoded(), QByteArray("ptal://mlc:usb:PC_970") );
+
+  QUrl dxOffEagle( "http://something/newpage.html?[{\"foo: bar\"}]", QUrl::TolerantMode);
+  QVERIFY(dxOffEagle.isValid());
+  QCOMPARE(QString(dxOffEagle.toEncoded()), QString("http://something/newpage.html?%5B%7B%22foo:%20bar%22%7D%5D"));
+  QUrl dxOffEagle2;
+  dxOffEagle2.setUrl( "http://something/newpage.html?[{\"foo: bar\"}]", QUrl::TolerantMode);
+  QVERIFY(dxOffEagle2.isValid());
+  QCOMPARE(dxOffEagle.toEncoded(), dxOffEagle2.toEncoded());
+
+  QUrl dxOffEagle3;
+  dxOffEagle3.setEncodedUrl( "http://something/newpage.html?[{\"foo: bar\"}]", QUrl::TolerantMode);
+  QEXPECT_FAIL("","Issue N183630, task ID 183874; works with setUrl so we do that in _setEncodedUrl now", Continue);
+  QVERIFY(dxOffEagle3.isValid());
+  //QEXPECT_FAIL("","Issue N183630, task ID 183874", Continue);
+  //QCOMPARE(dxOffEagle.toEncoded(), dxOffEagle3.toEncoded());
+
+  QUrl javascript;
+  javascript.setUrl("javascript:window.location+\"__flashplugin_unique__\"", QUrl::TolerantMode);
+  QVERIFY(javascript.isValid());
+  javascript.setEncodedUrl("javascript:window.location+\"__flashplugin_unique__\"", QUrl::TolerantMode);
+  QEXPECT_FAIL("","Issue N183630, task ID 183874", Continue);
+  QVERIFY(javascript.isValid());
 }
 
 void KUrlTest::testMailto()
@@ -1401,8 +1468,8 @@ void KUrlTest::testOtherEncodings()
 void KUrlTest::testPathOrURL()
 {
   // passing path or url to the constructor: both work
-  KUrl uloc( "/home/dfaure/konqtests/Mat%C3%A9riel" );
-  QCOMPARE( uloc.path(), QString("/home/dfaure/konqtests/Mat%C3%A9riel") );
+  KUrl uloc( LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel" );
+  QCOMPARE( uloc.path(), QString(LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel") );
   uloc = KUrl( "http://www.kde.org" );
   QCOMPARE( uloc.pathOrUrl(), uloc.url() );
   uloc = KUrl( QString("www.kde.org" ) );
@@ -1426,7 +1493,7 @@ void KUrlTest::testPathOrURL()
 #endif
 
   // pathOrUrl tests
-  uloc = KUrl( "/home/dfaure/konqtests/Mat%C3%A9riel" );
+  uloc = KUrl( LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel" );
   QCOMPARE( uloc.pathOrUrl(), uloc.path() );
   uloc = "http://www.kde.org";
   QCOMPARE( uloc.url(), QString("http://www.kde.org") );
@@ -1434,19 +1501,19 @@ void KUrlTest::testPathOrURL()
   QCOMPARE( uloc.pathOrUrl(), QString::fromUtf8("file:///home/dfaure/konq tests/Matériel#ref" ) );
   uloc = "file:///home/dfaure/konq%20tests/Mat%C3%A9riel?query";
   QCOMPARE( uloc.pathOrUrl(), QString::fromUtf8("file:///home/dfaure/konq tests/Matériel?query" ) );
-  uloc = KUrl( "/home/dfaure/file#with#hash" );
-  QCOMPARE( uloc.pathOrUrl(), QString("/home/dfaure/file#with#hash" ) );
+  uloc = KUrl( LOCAL_FILE_PREFIX"/home/dfaure/file#with#hash" );
+  QCOMPARE( uloc.pathOrUrl(), QString(LOCAL_FILE_PREFIX"/home/dfaure/file#with#hash" ) );
 }
 
 void KUrlTest::testAssignment()
 {
   // passing path or url to the constructor: both work
   KUrl uloc;
-  uloc = "/home/dfaure/konqtests/Mat%C3%A9riel";
-  QCOMPARE( uloc.path(), QString("/home/dfaure/konqtests/Mat%C3%A9riel") );
+  uloc = LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel";
+  QCOMPARE( uloc.path(), QString(LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel") );
   KUrl u2;
   u2 = uloc;
-  QCOMPARE( u2.path(), QString("/home/dfaure/konqtests/Mat%C3%A9riel") );
+  QCOMPARE( u2.path(), QString(LOCAL_FILE_PREFIX"/home/dfaure/konqtests/Mat%C3%A9riel") );
   uloc = "http://www.kde.org";
   QCOMPARE( uloc.pathOrUrl(), uloc.url() );
   uloc = QString("www.kde.org" );
