@@ -28,6 +28,8 @@
 #include <QtCore/QVariant>
 #include <QtCore/QStringList>
 #include <kcomponentdata.h>
+#include <kexportplugin.h>
+#include <kglobal.h>
 
 class KPluginFactoryPrivate;
 namespace KParts { class Part; }
@@ -106,7 +108,7 @@ KComponentData name::componentData() \
  *     KAboutData aboutData("myplugin", "myplugin", ki18n("MyPlugin"), "0.1",
  *             ki18n("a description of the plugin"), KAboutData::License_LGPL,
  *             ki18n("Copyright (C) 2007 Your Name"));
- *     aboutData->addAuthor(ki18n("Your Name"), ...);
+ *     aboutData.addAuthor(ki18n("Your Name"), ...);
  *     return aboutData;
  * }
  * K_EXPORT_PLUGIN(MyPluginFactory(createAboutData()))
@@ -211,7 +213,7 @@ KComponentData name::componentData() \
  * \endcode
  *
  * If you want to load a library use KPluginLoader.
- * The application that wants to instantiate plugin classes later on can do the following:
+ * The application that wants to instantiate plugin classes can do the following:
  * \code
  * KPluginFactory *factory = KPluginLoader("libraryname").factory();
  * if (factory) {
@@ -415,7 +417,7 @@ protected:
      */
     virtual KDE_DEPRECATED KParts::Part *createPartObject(QWidget *parentWidget, QObject *parent, const char *classname, const QStringList &args);
 
-    
+
     /**
      * This method sets the component data of the plugin. You can access the component data object
      * later with componentData().
