@@ -287,12 +287,14 @@ KDEDModule *Kded::loadModule(const KService::Ptr& s, bool onDemand)
             factoryName = "create_" + factoryName;
             KLibrary* lib = KLibLoader::self()->library(libname);
             KDEDModule* (*create)();
-            create = (KDEDModule* (*)())lib->resolveFunction(QFile::encodeName(factoryName));
-            if (create)
-                module = create();
+            if (lib) {
+                create = (KDEDModule* (*)())lib->resolveFunction(QFile::encodeName(factoryName));
+                if (create)
+                    module = create();
+            }
             if (!module) {
-                kWarning() << "Could not load library. [ "
-                           << loader.errorString() << " ]";
+                kWarning() << "Could not load library" << libname << ". ["
+                           << loader.errorString() << "]";
             }
         } else {
             // create the module
@@ -745,7 +747,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char *argv[])
 {
      KAboutData aboutData( "kded" /*don't change this one to kded4! dbus registration should be org.kde.kded etc.*/,
         "kdelibs4", ki18n("KDE Daemon"),
-        "$Id: kded.cpp 736326 2007-11-13 23:24:13Z lunakl $",
+        "$Id: kded.cpp 770025 2008-02-02 16:20:39Z dfaure $",
         ki18n("KDE Daemon - triggers Sycoca database updates when needed"));
 
      KCmdLineOptions options;
