@@ -97,6 +97,7 @@ const ColorCollectionNameType colorCollectionName[]=
     { "Recent_Colors", I18N_NOOP2( "palette name", "* Recent Colors *" ) },
     { "Custom_Colors", I18N_NOOP2( "palette name", "* Custom Colors *" ) },
     { "40.colors",     I18N_NOOP2( "palette name", "Forty Colors" ) },
+    { "Oxygen.colors", I18N_NOOP2( "palette name", "Oxygen Colors" ) },
     { "Rainbow.colors",I18N_NOOP2( "palette name", "Rainbow Colors" ) },
     { "Royal.colors",  I18N_NOOP2( "palette name", "Royal Colors" ) },
     { "Web.colors",    I18N_NOOP2( "palette name", "Web Colors" ) },
@@ -687,7 +688,7 @@ void
 KColorTable::KColorTablePrivate::slotSetColors( const QString &_collectionName )
 {
   q->setColors( _collectionName );
-  if( mNamedColorList->isVisible() )
+  if( mNamedColorList->count() && mNamedColorList->isVisible() )
   {
     int item = mNamedColorList->currentRow();
     mNamedColorList->setCurrentRow( item < 0 ? 0 : item );
@@ -791,7 +792,7 @@ KColorTable::setColors( const QString &_collectionName )
 void
 KColorTable::KColorTablePrivate::slotColorCellSelected( int index , const QColor& /*color*/ )
 {
-    if (!mPalette || (index >= mPalette->count()))
+  if (!mPalette || (index >= mPalette->count()))
      return;
   emit q->colorSelected( mPalette->color(index), mPalette->name(index) );
 }
@@ -1153,9 +1154,7 @@ KColorDialog::KColorDialog( QWidget *parent, bool modal )
   QPushButton* button = new QPushButton( page );
   button->setIcon( KIcon("color-picker"));
   int commonHeight = addButton->sizeHint().height();
-  button->setMinimumHeight( commonHeight );
-  kDebug() << commonHeight;
-  button->setIconSize(QSize(commonHeight, commonHeight));
+  button->setFixedSize( commonHeight, commonHeight );
   l_hbox->addWidget(button, 0, Qt::AlignHCenter );
   connect( button, SIGNAL( clicked()), SLOT( slotColorPicker()));
 
