@@ -144,6 +144,7 @@ class KDEUI_EXPORT KDialog : public QDialog
       Close   = 0x00000040, ///< Show Close-button. (this button closes the dialog)
       No      = 0x00000080, ///< Show No button. (this button closes the dialog and sets the result to KDialog::No)
       Yes     = 0x00000100, ///< Show Yes button. (this button closes the fialog and sets the result to KDialog::Yes)
+      Reset   = 0x00000200, ///< Show Reset button
       Details = 0x00000400, ///< Show Details button. (this button will show the detail widget set with setDetailsWidget)
       User1   = 0x00001000, ///< Show User defined button 1.
       User2   = 0x00002000, ///< Show User defined button 2.
@@ -356,7 +357,7 @@ class KDEUI_EXPORT KDialog : public QDialog
      * @param config The config group to read from.
      * @param options passed to KConfigGroup::writeEntry()
      */
-    void saveDialogSize( KConfigGroup& config, KConfigGroup::WriteConfigFlags options = 0 ) const;
+    void saveDialogSize( KConfigGroup& config, KConfigGroup::WriteConfigFlags options = KConfigGroup::Normal ) const;
 
     /**
      * Returns the help link text.
@@ -655,6 +656,12 @@ class KDEUI_EXPORT KDialog : public QDialog
      */
     void defaultClicked();
 
+    /**
+     * The Reset button was pressed. This signal is only emitted if
+     * slotButtonClicked() is not replaced
+     */
+    void resetClicked();
+
     /**
      * The User3 button was pressed. This signal is only emitted if
      * slotButtonClicked() is not replaced
@@ -776,7 +783,22 @@ class KDEUI_EXPORT KDialog : public QDialog
   protected Q_SLOTS:
     /**
      * Activated when the button @p button is clicked
-     * @param button is the type ButtonCode
+     *
+     * Sample that shows how to catch and handle button clicks within
+     * an own dialog;
+     * @code
+     * class MyDialog : public KDialog {
+     *     protected Q_SLOTS:
+     *         virtual void slotButtonClicked(int button) {
+     *             if (button == KDialog::Ok)
+     *                 accept();
+     *             else
+     *                 KDialog::slotButtonClicked(button);
+     *         }
+     * }
+     * @endcode
+     *
+     * @param button is the type @a KDialog::ButtonCode
      */
     virtual void slotButtonClicked(int button);
 
