@@ -43,6 +43,38 @@
 #include <X11/Xlib.h>
 #endif
 
+// Some i18n filters, that standard button texts are piped through
+// (the new KGuiItem object with filtered text is created from the old one).
+
+// i18n: Filter for the Yes-button text in standard message dialogs,
+// after the message caption/text have been translated.
+#define I18N_FILTER_BUTTON_YES(src, dst) \
+    KGuiItem dst(src); \
+    dst.setText( i18nc( "@action:button filter-yes", "%1", src.text() ) );
+
+// i18n: Filter for the No-button text in standard message dialogs,
+// after the message caption/text have been translated.
+#define I18N_FILTER_BUTTON_NO(src, dst) \
+    KGuiItem dst(src); \
+    dst.setText( i18nc( "@action:button filter-no", "%1", src.text() ) );
+
+// i18n: Filter for the Continue-button text in standard message dialogs,
+// after the message caption/text have been translated.
+#define I18N_FILTER_BUTTON_CONTINUE(src, dst) \
+    KGuiItem dst(src); \
+    dst.setText( i18nc( "@action:button filter-continue", "%1", src.text() ) );
+
+// i18n: Filter for the Cancel-button text in standard message dialogs,
+// after the message caption/text have been translated.
+#define I18N_FILTER_BUTTON_CANCEL(src, dst) \
+    KGuiItem dst(src); \
+    dst.setText( i18nc( "@action:button filter-cancel", "%1", src.text() ) );
+
+// i18n: Called after the button texts in standard message dialogs
+// have been filtered by the messages above. Not visible to user.
+#define I18N_POST_BUTTON_FILTER \
+    i18nc( "@action:button post-filter", "." );
+
  /**
   * Easy MessageBox Dialog.
   *
@@ -159,19 +191,29 @@ int KMessageBox::createKMessageBox(KDialog *dialog, const QIcon &icon,
     QLabel *label2 = new QLabel( text, 0 );
     label2->setOpenExternalLinks(options & KMessageBox::AllowLink);
     label2->setTextInteractionFlags(Qt::TextSelectableByMouse | Qt::TextSelectableByKeyboard);
-    label2->setWordWrap(true);
-    QRect d = KGlobalSettings::desktopGeometry(dialog);
+    QPalette messagePal(label2->palette());
+    messagePal.setColor(QPalette::Window, Qt::transparent);
+    label2->setPalette(messagePal);
 
+    QRect desktop = KGlobalSettings::desktopGeometry(dialog);
+    if (desktop.width() / 3 < label2->fontMetrics().width(text)) {
+        // do only enable automatic wrapping for messages which are longer than one third of the current screen width
+        label2->setWordWrap(true);
+    }
     QScrollArea* textArea = new QScrollArea( contents );
     textArea->setWidget( label2 );
     textArea->setFrameShape( QFrame::NoFrame );
     textArea->setWidgetResizable(true);
-    textArea->setStyleSheet("QAbstractScrollArea { background-color: none; }");
+    QPalette scrollPal(textArea->palette());
+    scrollPal.setColor(QPalette::Window, Qt::transparent);
+    textArea->setPalette(scrollPal);
 
     lay->addWidget( textArea );
 
     QListWidget *listwidget = 0;
     if (!strlist.isEmpty()) {
+        // enable automatic wrapping since the listwidget has already a good initial width
+        label2->setWordWrap(true);
         listwidget = new QListWidget(topcontents);
         toplayout->addWidget(listwidget);
         listwidget->addItems(strlist);
@@ -352,8 +394,8 @@ int KMessageBox::questionYesNoList(QWidget *parent, const QString &text,
 int KMessageBox::questionYesNoListWId(WId parent_id, const QString &text,
                            const QStringList &strlist,
                            const QString &caption,
-                           const KGuiItem &buttonYes,
-                           const KGuiItem &buttonNo,
+                           const KGuiItem &buttonYes_,
+                           const KGuiItem &buttonNo_,
                            const QString &dontAskAgainName,
                            Options options)
 {
@@ -362,6 +404,10 @@ int KMessageBox::questionYesNoListWId(WId parent_id, const QString &text,
         return res;
     }
 
+    I18N_FILTER_BUTTON_YES(buttonYes_, buttonYes)
+    I18N_FILTER_BUTTON_NO(buttonNo_, buttonNo)
+    I18N_POST_BUTTON_FILTER
+
     QWidget* parent = QWidget::find( parent_id );
     KDialog *dialog = new KDialog(parent, Qt::Dialog);
     dialog->setCaption( caption.isEmpty() ? i18n("Question") : caption );
@@ -410,9 +456,9 @@ int KMessageBox::questionYesNoCancel(QWidget *parent,
 int KMessageBox::questionYesNoCancelWId(WId parent_id,
                           const QString &text,
                           const QString &caption,
-                          const KGuiItem &buttonYes,
-                          const KGuiItem &buttonNo,
-                          const KGuiItem &buttonCancel,
+                          const KGuiItem &buttonYes_,
+                          const KGuiItem &buttonNo_,
+                          const KGuiItem &buttonCancel_,
                           const QString &dontAskAgainName,
                           Options options)
 {
@@ -421,6 +467,11 @@ int KMessageBox::questionYesNoCancelWId(WId parent_id,
         return res;
     }
 
+    I18N_FILTER_BUTTON_YES(buttonYes_, buttonYes)
+    I18N_FILTER_BUTTON_NO(buttonNo_, buttonNo)
+    I18N_FILTER_BUTTON_CANCEL(buttonCancel_, buttonCancel)
+    I18N_POST_BUTTON_FILTER
+
     QWidget* parent = QWidget::find( parent_id );
     KDialog *dialog= new KDialog(parent, Qt::Dialog);
     dialog->setCaption( caption.isEmpty() ? i18n("Question") : caption );
@@ -498,8 +549,8 @@ int KMessageBox::warningYesNoList(QWidget *parent, const QString &text,
 int KMessageBox::warningYesNoListWId(WId parent_id, const QString &text,
                               const QStringList &strlist,
                               const QString &caption,
-                              const KGuiItem &buttonYes,
-                              const KGuiItem &buttonNo,
+                              const KGuiItem &buttonYes_,
+                              const KGuiItem &buttonNo_,
                               const QString &dontAskAgainName,
                               Options options)
 {
@@ -513,6 +564,10 @@ int KMessageBox::warningYesNoListWId(WId parent_id, const QString &text,
         return res;
     }
 
+    I18N_FILTER_BUTTON_YES(buttonYes_, buttonYes)
+    I18N_FILTER_BUTTON_NO(buttonNo_, buttonNo)
+    I18N_POST_BUTTON_FILTER
+
     QWidget* parent = QWidget::find( parent_id );
     KDialog *dialog = new KDialog(parent, Qt::Dialog);
     dialog->setCaption( caption.isEmpty() ? i18n("Warning") : caption );
@@ -584,14 +639,18 @@ int KMessageBox::warningContinueCancelList(QWidget *parent, const QString &text,
 int KMessageBox::warningContinueCancelListWId(WId parent_id, const QString &text,
                              const QStringList &strlist,
                              const QString &caption,
-                             const KGuiItem &buttonContinue,
-                             const KGuiItem &buttonCancel,
+                             const KGuiItem &buttonContinue_,
+                             const KGuiItem &buttonCancel_,
                              const QString &dontAskAgainName,
                              Options options)
 {
     if ( !shouldBeShownContinue(dontAskAgainName) )
         return Continue;
 
+    I18N_FILTER_BUTTON_CONTINUE(buttonContinue_, buttonContinue)
+    I18N_FILTER_BUTTON_CANCEL(buttonCancel_, buttonCancel)
+    I18N_POST_BUTTON_FILTER
+
     QWidget* parent = QWidget::find( parent_id );
     KDialog *dialog = new KDialog(parent, Qt::Dialog);
     dialog->setCaption( caption.isEmpty() ? i18n("Warning") : caption );
@@ -666,9 +725,9 @@ int KMessageBox::warningYesNoCancelList(QWidget *parent, const QString &text,
 int KMessageBox::warningYesNoCancelListWId(WId parent_id, const QString &text,
                                     const QStringList &strlist,
                                     const QString &caption,
-                                    const KGuiItem &buttonYes,
-                                    const KGuiItem &buttonNo,
-                                    const KGuiItem &buttonCancel,
+                                    const KGuiItem &buttonYes_,
+                                    const KGuiItem &buttonNo_,
+                                    const KGuiItem &buttonCancel_,
                                     const QString &dontAskAgainName,
                                     Options options)
 {
@@ -677,6 +736,11 @@ int KMessageBox::warningYesNoCancelListWId(WId parent_id, const QString &text,
         return res;
     }
 
+    I18N_FILTER_BUTTON_YES(buttonYes_, buttonYes)
+    I18N_FILTER_BUTTON_NO(buttonNo_, buttonNo)
+    I18N_FILTER_BUTTON_CANCEL(buttonCancel_, buttonCancel)
+    I18N_POST_BUTTON_FILTER
+
     QWidget* parent = QWidget::find( parent_id );
     KDialog *dialog = new KDialog(parent, Qt::Dialog);
     dialog->setCaption( caption.isEmpty() ? i18n("Warning") : caption );
