@@ -203,6 +203,7 @@ public:
     // lazy loading: initIconThemes() is only needed when the "links" list is needed
     // mIconThemeInited is used inside initIconThemes() to init only once
     bool mIconThemeInited :1;
+    bool lastWasUnknown :1; // last loaded image was the unknown image
     QString appname;
 
     KIconLoader *q;
@@ -217,15 +218,18 @@ void KIconLoaderPrivate::drawOverlays(const KIconLoader *iconLoader, KIconLoader
     }
 
     int iconSize = pix.size().width();
-    int overlaySize = 22;
-    if (iconSize >= 256) {
-        overlaySize = 64;
-    } else if (iconSize >= 128) {
-        overlaySize = 32;
-    } else if (iconSize <= 22) {
+    int overlaySize;
+
+    if (iconSize < 32) {
         overlaySize = 8;
-    } else if (iconSize < 32) {
-        overlaySize = 12;
+    } else if (iconSize <= 48) {
+        overlaySize = 16;
+    } else if (iconSize <= 96) {
+        overlaySize = 22;
+    } else if (iconSize < 256) {
+        overlaySize = 32;
+    } else {
+        overlaySize = 64;
     }
 
     QPainter painter(&pix);
@@ -411,7 +415,7 @@ bool KIconLoaderPrivate::initIconThemes()
         // If mpThemeRoot isn't 0 then initing has succeeded
         return (mpThemeRoot != 0);
     }
-    kDebug(264);
+    //kDebug(264);
     mIconThemeInited = true;
 
     // Add the default theme and its base themes to the theme tree
@@ -436,7 +440,7 @@ bool KIconLoaderPrivate::initIconThemes()
 
     // Insert application specific themes at the top.
     mpDirs->addResourceType("appicon", "data", appname + "/pics/");
-    // ################## KDE4: consider removing the toolbar directory
+    // ################## KDE5: consider removing the toolbar directory
     mpDirs->addResourceType("appicon", "data", appname + "/toolbar/");
 
     // Add legacy icon dirs.
@@ -893,16 +897,18 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
         if (path.isEmpty())
         {
             if (!canReturnNull) {
+#ifndef NDEBUG
                 kWarning(264) << "No such icon" << _name;
+#endif
                 unknownIcon = true;
+            } else {
+                return pix; // null pixmap
             }
-            if (canReturnNull)
-                return pix;
             // We don't know the desired size: use small
             path = iconPath(str_unknown, KIconLoader::Small, true);
             if (path.isEmpty())
             {
-                kDebug(264) << "Warning: Cannot find \"unknown\" icon.";
+                kWarning(264) << "Warning: Cannot find \"unknown\" icon.";
                 return pix;
             }
         }
@@ -917,12 +923,8 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
 
         pix = QPixmap::fromImage(img);
         d->drawOverlays(this, KIconLoader::Desktop, state, pix, overlays);
-#ifndef NDEBUG
-        // do not insert 'unknown' icons into cache in debug mode;
-        // thus warnings are displayed every time a broken icon has been requested
         if (!unknownIcon)
-#endif
-        d->mIconCache->insert(key, pix, path);
+            d->mIconCache->insert(key, pix, path);
         return pix;
     }
 
@@ -985,7 +987,8 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
     if (d->mIconCache->find(key, pix, path_store)) {
         //kDebug() << "KIL: " << "found icon from KIC";
         return pix;
-    } else if (!d->initIconThemes()) {
+    }
+    if (!d->initIconThemes()) {
         return pix; // null pixmap
     }
 
@@ -993,10 +996,11 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
     int iconType;
     int iconThreshold;
 
-    if ( ( path_store != 0L ) ||
-         noEffectKey != d->lastImageKey )
+    if ( ( path_store != 0 ) ||
+         ( noEffectKey != d->lastImageKey ) ||
+         ( d->lastWasUnknown && canReturnNull ) )
     {
-        // No? load it.
+        // Not in cache and not the same as the last requested icon -> load it.
         K3Icon icon;
         if (absolutePath && !favIconOverlay)
         {
@@ -1015,12 +1019,17 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
                 if (!name.isEmpty()) {
                     pix = loadIcon(name, KIconLoader::User, size, state, overlays, path_store, true);
                 }
-                if (!pix.isNull() || canReturnNull) {
+                if (!pix.isNull()) {
                     d->mIconCache->insert(key, pix, path);
                     return pix;
                 }
+                if (canReturnNull) {
+                    return pix; // null pixmap
+                }
 
+#ifndef NDEBUG
                 kWarning(264) << "No such icon" << _name;
+#endif
                 unknownIcon = true;
                 icon = d->findMatchingIcon(str_unknown, size);
                 if (!icon.isValid())
@@ -1033,10 +1042,8 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
             }
         }
 
-        if (path_store != 0L)
+        if (path_store != 0)
             *path_store = icon.path;
-        //if (inCache)
-        //    return pix;
 
         // Use the extension as the format. Works for XPM and PNG, but not for SVG
         QString ext = icon.path.right(3).toUpper();
@@ -1085,12 +1092,14 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
         d->lastImageKey = noEffectKey;
         d->lastIconType = iconType;
         d->lastIconThreshold = iconThreshold;
+        d->lastWasUnknown = unknownIcon;
     }
     else
     {
         img = new QImage( d->lastImage.copy() );
         iconType = d->lastIconType;
         iconThreshold = d->lastIconThreshold;
+        unknownIcon = d->lastWasUnknown;
     }
 
     // Scale the icon and apply effects if necessary
@@ -1159,12 +1168,10 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
 
     delete img;
 
-#ifndef NDEBUG
     if (unknownIcon)
     {
         addToCache = false;
     }
-#endif
     if (addToCache)
     {
         d->mIconCache->insert(key, pix, path);
