@@ -30,6 +30,10 @@
 #include <kcomponentdata.h>
 #include <kcolorscheme.h>
 
+#ifdef Q_WS_X11
+#include <kstyle.h>
+#endif
+
 #include <QtGui/QColor>
 #include <QtGui/QCursor>
 #include <QtGui/QDesktopWidget>
@@ -45,6 +49,10 @@
 #include <QtDBus/QtDBus>
 #include <QtGui/QStyleFactory>
 
+// next two needed so we can set their palettes
+#include <QtGui/QToolTip>
+#include <QtGui/QWhatsThis>
+
 #ifdef Q_WS_WIN
 #include <windows.h>
 #include <kkernel_win.h>
@@ -77,9 +85,6 @@ static QFont *_windowTitleFont = 0;
 static QFont *_taskbarFont = 0;
 static QFont *_largeFont = 0;
 static QFont *_smallestReadableFont = 0;
-static QColor *_inactiveBackground = 0;
-static QColor *_inactiveForeground = 0;
-static QColor *_activeBackground = 0;
 //static QColor *_buttonBackground = 0;
 
 static KGlobalSettings::KMouseSettings *s_mouseSettings = 0;
@@ -100,6 +105,9 @@ class KGlobalSettings::Private
         void kdisplaySetFont();
         void applyGUIStyle();
 
+        // TODO (4.1?) - make not-private, if used in KStyle
+        static QPalette createTooltipPalette(const KSharedConfigPtr &config = KSharedConfigPtr());
+
         /**
          * @internal
          *
@@ -236,49 +244,47 @@ int KGlobalSettings::contextMenuKey ()
     return QKeySequence::fromString(s)[0];
 }
 
+// NOTE: keep this in sync with kdebase/workspace/kcontrol/colors/colorscm.cpp
 QColor KGlobalSettings::inactiveTitleColor()
 {
 #ifdef Q_WS_WIN
     return qt_colorref2qrgb(GetSysColor(COLOR_INACTIVECAPTION));
 #else
-    if (!_inactiveBackground)
-        _inactiveBackground = new QColor(157, 170, 186);
     KConfigGroup g( KGlobal::config(), "WM" );
-    return g.readEntry( "inactiveBackground", *_inactiveBackground );
+    return g.readEntry( "inactiveBackground", QColor(224, 223, 222) );
 #endif
 }
 
+// NOTE: keep this in sync with kdebase/workspace/kcontrol/colors/colorscm.cpp
 QColor KGlobalSettings::inactiveTextColor()
 {
 #ifdef Q_WS_WIN
     return qt_colorref2qrgb(GetSysColor(COLOR_INACTIVECAPTIONTEXT));
 #else
-    if (!_inactiveForeground)
-       _inactiveForeground = new QColor(221,221,221);
     KConfigGroup g( KGlobal::config(), "WM" );
-    return g.readEntry( "inactiveForeground", *_inactiveForeground );
+    return g.readEntry( "inactiveForeground", QColor(20, 19, 18) );
 #endif
 }
 
+// NOTE: keep this in sync with kdebase/workspace/kcontrol/colors/colorscm.cpp
 QColor KGlobalSettings::activeTitleColor()
 {
 #ifdef Q_WS_WIN
     return qt_colorref2qrgb(GetSysColor(COLOR_ACTIVECAPTION));
 #else
-    if (!_activeBackground)
-      _activeBackground = new QColor(65,142,220);
     KConfigGroup g( KGlobal::config(), "WM" );
-    return g.readEntry( "activeBackground", *_activeBackground);
+    return g.readEntry( "activeBackground", QColor(96, 148, 207));
 #endif
 }
 
+// NOTE: keep this in sync with kdebase/workspace/kcontrol/colors/colorscm.cpp
 QColor KGlobalSettings::activeTextColor()
 {
 #ifdef Q_WS_WIN
     return qt_colorref2qrgb(GetSysColor(COLOR_CAPTIONTEXT));
 #else
     KConfigGroup g( KGlobal::config(), "WM" );
-    return g.readEntry( "activeForeground", QColor(Qt::white) );
+    return g.readEntry( "activeForeground", QColor(255, 255, 255) );
 #endif
 }
 
@@ -361,8 +367,8 @@ QFont KGlobalSettings::toolBarFont()
     _toolBarFont->setPointSize(11);
 #else
     // Sync default with kdebase/kcontrol/fonts/fonts.cpp
-    _toolBarFont = new QFont("Sans Serif", 10);
-    _toolBarFont->setPointSize(10);
+    _toolBarFont = new QFont("Sans Serif", 8);
+    _toolBarFont->setPointSize(8);
 #endif
     _toolBarFont->setStyleHint(QFont::SansSerif);
 
@@ -823,11 +829,16 @@ QString kde_overrideStyle;
 void KGlobalSettings::Private::applyGUIStyle()
 {
 #ifdef Q_WS_X11
+    QString defaultStyle = KStyle::defaultStyle();
+#else
+    QString defaultStyle; // Mac, Windows: no change for style by default
+#endif
     KConfigGroup pConfig (KGlobal::config(), "General");
-    QString defaultStyle = QLatin1String("plastique");// = KStyle::defaultStyle(); ### wait for KStyle4
     QString styleStr = pConfig.readEntry("widgetStyle", defaultStyle);
 
     if (kde_overrideStyle.isEmpty()) {
+        if (styleStr.isEmpty())
+            return;
         // ### add check whether we already use the correct style to return then
         // (workaround for Qt misbehavior to avoid double style initialization)
 
@@ -844,7 +855,6 @@ void KGlobalSettings::Private::applyGUIStyle()
         qApp->setStyle(kde_overrideStyle);
     // Reread palette from config file.
     kdisplaySetPalette();
-#endif
 }
 
 QPalette KGlobalSettings::createApplicationPalette(const KSharedConfigPtr &config)
@@ -885,6 +895,47 @@ QPalette KGlobalSettings::createApplicationPalette(const KSharedConfigPtr &confi
 }
 
 
+QPalette KGlobalSettings::Private::createTooltipPalette(const KSharedConfigPtr &config)
+{
+    QPalette palette = QToolTip::palette();
+
+    QPalette::ColorGroup setStates[3] = { QPalette::Active, QPalette::Inactive,
+                                          QPalette::Disabled };
+    QPalette::ColorGroup useStates[3] = { QPalette::Active, QPalette::Active,
+                                          QPalette::Disabled };
+
+    for ( int i = 0; i < 3 ; i++ ) {
+        // tooltips are never active (arguably a bug?), but should be drawn as if they were
+        QPalette::ColorGroup state = setStates[i];
+        QPalette::ColorGroup stateUsed = useStates[i];
+        KColorScheme schemeButton(stateUsed, KColorScheme::Button, config);
+        KColorScheme schemeSelection(stateUsed, KColorScheme::Selection, config);
+        KColorScheme schemeTooltip(stateUsed, KColorScheme::Tooltip, config);
+
+        palette.setBrush( state, QPalette::WindowText, schemeTooltip.foreground() );
+        palette.setBrush( state, QPalette::Window, schemeTooltip.background() );
+        palette.setBrush( state, QPalette::Base, schemeTooltip.background() );
+        palette.setBrush( state, QPalette::Text, schemeTooltip.foreground() );
+        palette.setBrush( state, QPalette::Button, schemeButton.background() );
+        palette.setBrush( state, QPalette::ButtonText, schemeButton.foreground() );
+        palette.setBrush( state, QPalette::Highlight, schemeSelection.background() );
+        palette.setBrush( state, QPalette::HighlightedText, schemeSelection.foreground() );
+
+        palette.setColor( state, QPalette::Light, schemeTooltip.shade( KColorScheme::LightShade ) );
+        palette.setColor( state, QPalette::Midlight, schemeTooltip.shade( KColorScheme::MidlightShade ) );
+        palette.setColor( state, QPalette::Mid, schemeTooltip.shade( KColorScheme::MidShade ) );
+        palette.setColor( state, QPalette::Dark, schemeTooltip.shade( KColorScheme::DarkShade ) );
+        palette.setColor( state, QPalette::Shadow, schemeTooltip.shade( KColorScheme::ShadowShade ) );
+
+        palette.setBrush( state, QPalette::AlternateBase, schemeTooltip.background( KColorScheme::AlternateBackground) );
+        palette.setBrush( state, QPalette::Link, schemeTooltip.foreground( KColorScheme::LinkText ) );
+        palette.setBrush( state, QPalette::LinkVisited, schemeTooltip.foreground( KColorScheme::VisitedText ) );
+    }
+
+    return palette;
+}
+
+
 void KGlobalSettings::Private::kdisplaySetPalette()
 {
     // Added by Sam/Harald (TT) for Mac OS X initially, but why?
@@ -894,6 +945,9 @@ void KGlobalSettings::Private::kdisplaySetPalette()
 
     if (qApp && qApp->type() == QApplication::GuiClient) {
         QApplication::setPalette( q->createApplicationPalette() );
+        QPalette ttp = createTooltipPalette();
+        QToolTip::setPalette( ttp );
+        // QWhatsThis::setPalette( ttp ); // TODO (when Qt supports it)
         emit q->kdisplayPaletteChanged();
         emit q->appearanceChanged();
     }
@@ -907,6 +961,7 @@ void KGlobalSettings::Private::kdisplaySetFont()
         QApplication::setFont(KGlobalSettings::menuFont(), "QMenuBar");
         QApplication::setFont(KGlobalSettings::menuFont(), "QPopupMenu");
         QApplication::setFont(KGlobalSettings::menuFont(), "KPopupTitle");
+        QApplication::setFont(KGlobalSettings::toolBarFont(), "QToolBar");
 
 #if 0
         // "patch" standard QStyleSheet to follow our fonts
