@@ -61,19 +61,22 @@
 
 //TODO what was the problem that got fixed recently in the old version? - forward port if necessary
 
-KGlobalAccelPrivate::KGlobalAccelPrivate()
+KGlobalAccelPrivate::KGlobalAccelPrivate(KGlobalAccel* q)
      : isUsingForeignComponentName(false),
        enabled(true),
        iface("org.kde.kded", "/modules/kdedglobalaccel", QDBusConnection::sessionBus())
 {
     // Make sure kded is running
-    if (!QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.kded")) {
+    QDBusConnectionInterface* bus = QDBusConnection::sessionBus().interface();
+    if (!bus->isServiceRegistered("org.kde.kded")) {
         KToolInvocation::klauncher(); // this calls startKdeinit
     }
+    QObject::connect(bus, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
+            q, SLOT(_k_serviceOwnerChanged(QString,QString,QString)));
 }
 
 KGlobalAccel::KGlobalAccel()
-    : d(new KGlobalAccelPrivate)
+    : d(new KGlobalAccelPrivate(this))
 {
     qDBusRegisterMetaType<QList<int> >();
 
@@ -152,10 +155,14 @@ void KGlobalAccelPrivate::updateGlobalShortcutAllowed(KAction *action, uint flag
 
     if (!oldEnabled && newEnabled) {
         uint setterFlags = KdedGlobalAccel::SetPresent;
+        KShortcut defaultShortcut = action->globalShortcut(KAction::DefaultShortcut);
+        KShortcut activeShortcut = action->globalShortcut();
         if (flags & KAction::NoAutoloading)
             setterFlags |= KdedGlobalAccel::NoAutoloading;
-        if (action->globalShortcut(KAction::DefaultShortcut).isEmpty())
+        if (defaultShortcut.isEmpty())
             setterFlags |= KdedGlobalAccel::IsDefaultEmpty;
+        if (defaultShortcut == activeShortcut)
+            setterFlags |= KdedGlobalAccel::IsDefault;
 
         nameToAction.insert(actionId.at(1), action);
         actionToName.insert(action, actionId.at(1));
@@ -187,10 +194,14 @@ void KGlobalAccelPrivate::updateGlobalShortcut(KAction *action, uint flags)
     //TODO: what about i18ned names?
 
     uint setterFlags = 0;
+    KShortcut defaultShortcut = action->globalShortcut(KAction::DefaultShortcut);
+    KShortcut activeShortcut = action->globalShortcut();
     if (flags & KAction::NoAutoloading)
         setterFlags |= KdedGlobalAccel::NoAutoloading;
-    if (action->globalShortcut(KAction::DefaultShortcut).isEmpty())
+    if (defaultShortcut.isEmpty())
         setterFlags |= KdedGlobalAccel::IsDefaultEmpty;
+    if (defaultShortcut == activeShortcut)
+        setterFlags |= KdedGlobalAccel::IsDefault;
 
     QList<int> result = iface.setShortcut(actionId,
                                           intListFromShortcut(action->globalShortcut()),
@@ -240,6 +251,8 @@ void KGlobalAccelPrivate::_k_invokeAction(const QStringList &actionId)
     KAction *action = nameToAction.value(actionId.at(1));
     if (!action)
         return;
+    if (!action->isEnabled())
+        return;
 
 #ifdef Q_WS_X11
     // Update this application's X timestamp if needed.
@@ -266,8 +279,16 @@ void KGlobalAccelPrivate::_k_shortcutGotChanged(const QStringList &actionId,
     action->d->setActiveGlobalShortcutNoEnable(shortcutFromIntList(keys));
 }
 
+void KGlobalAccelPrivate::_k_serviceOwnerChanged(const QString& name, const QString& oldOwner, const QString& newOwner)
+{
+    Q_UNUSED(oldOwner);
+    if (name == QLatin1String("org.kde.kded") && !newOwner.isEmpty()) {
+        // kded was restarted (what? you mean it crashes sometimes?)
+        reRegisterAll();
+    }
+}
 
-void KGlobalAccelPrivate::_k_reRegisterAll()
+void KGlobalAccelPrivate::reRegisterAll()
 {
     //### Special case for isUsingForeignComponentName?
 
@@ -311,8 +332,10 @@ bool KGlobalAccel::promptStealShortcutSystemwide(QWidget *parent, const QStringL
 //static
 void KGlobalAccel::stealShortcutSystemwide(const QKeySequence &seq)
 {
-    //get the shortcut, remove &seq, and set the new shorctut
-    QStringList actionId = self()->d->iface.action(seq[0]);
+    //get the shortcut, remove seq, and set the new shorctut
+    const QStringList actionId = self()->d->iface.action(seq[0]);
+    if (actionId.size() < 2) // not a global shortcut
+        return;
     QList<int> sc = self()->d->iface.shortcut(actionId);
 
     for (int i = 0; i < sc.count(); i++)
