@@ -54,8 +54,8 @@ public:
     QString                myCurrentMatch;
     KCompTreeNode *        myTreeRoot;
     //QStringList            myRotations;
-    bool                   myBeep;
-    bool                   myIgnoreCase;
+    bool                   myBeep : 1;
+    bool                   myIgnoreCase : 1;
     bool                   myHasMultipleMatches;
     int                    myRotationIndex;
 };
@@ -830,6 +830,23 @@ public:
     bool sorting;
 };
 
+KCompletionMatches::KCompletionMatches( const KCompletionMatches &o )
+ : KSortableList<QString, int>(),
+   d( new KCompletionMatchesPrivate( o.d->sorting ) )
+{
+    *this = KCompletionMatches::operator=( o );
+}
+
+KCompletionMatches &KCompletionMatches::operator=( const KCompletionMatches &o )
+{
+    if( *this == o )
+        return *this;
+    KCompletionMatchesList::operator=( o );
+    d->sorting = o.d->sorting;
+
+    return *this;
+}
+
 KCompletionMatches::KCompletionMatches( bool sort_P )
     : d( new KCompletionMatchesPrivate( sort_P ) )
 {
