@@ -29,6 +29,8 @@
 #include <kdeui_export.h>
 
 
+class QDBusError;
+
 namespace KWallet {
 
 /**
@@ -481,6 +483,12 @@ class KDEUI_EXPORT Wallet : public QObject
 		 */
 		void walletOpenResult(int rc);
 
+		/**
+		 *  @internal
+		 *  DBUS error slot.
+		 */
+		void walletOpenError(const QDBusError& error);
+
 		/**
 		 *  @internal
 		 *  Used to detect when the wallet service dies.
