@@ -67,12 +67,6 @@ class KDEUI_EXPORT KXErrorHandler
          * any X error.
          */
         explicit KXErrorHandler( Display* dpy = QX11Info::display());
-        /**
-         * This constructor takes pointer to a function that will get request number,
-         * error code number and resource id of the failed request, as provided
-         * by XErrorEvent. If the function returns true, the error flag will be set.
-         */
-        explicit KXErrorHandler( bool (*handler)( int request, int error_code, unsigned long resource_id ), Display* dpy = QX11Info::display());
         /**
          * This constructor takes pointer to a function whose prototype matches
          * the one that's used with the XSetErrorHandler() Xlib function.
@@ -80,6 +74,13 @@ class KDEUI_EXPORT KXErrorHandler
          * value.
          */
         explicit KXErrorHandler( int (*handler)( Display*, XErrorEvent* ), Display* dpy = QX11Info::display());
+        /**
+         * This constructor takes pointer to a function that will get request number,
+         * error code number and resource id of the failed request, as provided
+         * by XErrorEvent. If the function returns true, the error flag will be set.
+         * @deprecated Use the variant with XErrorEvent.
+         */
+        explicit KXErrorHandler( bool (*handler)( int request, int error_code, unsigned long resource_id ), Display* dpy = QX11Info::display()) KDE_DEPRECATED;
         /**
          * This function returns true if the error flag is set (i.e. no custom handler
          * function was used and there was any error, or the custom handler indicated
@@ -89,6 +90,18 @@ class KDEUI_EXPORT KXErrorHandler
          *             when the last X request required a roundtrip.
          */
         bool error( bool sync ) const;
+        /**
+         * This function returns the error event for the first X error that occured.
+         * The return value is useful only if error() returned true.
+         * @since 4.0.1
+         */
+        XErrorEvent errorEvent() const;
+        /**
+         * Returns error message for the given error. The error message is not translated,
+         * as it is meant for debugging.
+         * @since 4.0.1
+         */
+        static QByteArray errorMessage( const XErrorEvent& e, Display* dpy = QX11Info::display());
         ~KXErrorHandler();
     private:
         void addHandler();
