@@ -7,12 +7,12 @@
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
     version 2 of the License, or (at your option) any later version.
- 
+
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Library General Public License for more details.
- 
+
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
     the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
@@ -35,6 +35,7 @@
 #include "kmenu.h"
 #include "kactioncollection.h"
 #include "kaction.h"
+#include <kdeversion.h>
 
 #include <QtCore/QDate>
 #include <QtCore/QCharRef>
@@ -106,8 +107,8 @@ public:
      */
     int numDayColumns;
 
-    bool popupMenuEnabled;
-    bool useCustomColors;
+    bool popupMenuEnabled : 1;
+    bool useCustomColors : 1;
 
     struct DatePaintingMode
     {
@@ -231,33 +232,39 @@ KDateTable::~KDateTable()
 void KDateTable::initAccels()
 {
     KActionCollection * localCollection = new KActionCollection( this );
-    localCollection->setAssociatedWidget( this );
 
-    QAction* next = localCollection->addAction( QLatin1String( "next" ) );
+    KAction* next = localCollection->addAction( QLatin1String( "next" ) );
     next->setShortcuts( KStandardShortcut::next() );
     connect( next, SIGNAL( triggered( bool ) ), SLOT( nextMonth() ) );
 
-    QAction* prior = localCollection->addAction( QLatin1String( "prior" ) );
+    KAction* prior = localCollection->addAction( QLatin1String( "prior" ) );
     prior->setShortcuts( KStandardShortcut::prior() );
     connect( prior, SIGNAL( triggered( bool ) ), SLOT( previousMonth() ) );
 
-    QAction* beginMonth = localCollection->addAction( QLatin1String( "beginMonth" ) );
-    beginMonth->setShortcuts( KStandardShortcut::home() );
+    KAction* beginMonth = localCollection->addAction( QLatin1String( "beginMonth" ) );
+    beginMonth->setShortcuts( KStandardShortcut::begin() );
     connect( beginMonth, SIGNAL( triggered( bool ) ), SLOT( beginningOfMonth() ) );
 
-    QAction* endMonth = localCollection->addAction( QLatin1String( "endMonth" ) );
+    KAction* endMonth = localCollection->addAction( QLatin1String( "endMonth" ) );
     endMonth->setShortcuts( KStandardShortcut::end() );
     connect( endMonth, SIGNAL( triggered( bool ) ), SLOT( endOfMonth() ) );
 
-    QAction* beginWeek = localCollection->addAction( QLatin1String( "beginWeek" ) );
+    KAction* beginWeek = localCollection->addAction( QLatin1String( "beginWeek" ) );
     beginWeek->setShortcuts( KStandardShortcut::beginningOfLine() );
     connect( beginWeek, SIGNAL( triggered( bool ) ), SLOT( beginningOfWeek() ) );
 
-    QAction* endWeek = localCollection->addAction( "endWeek" );
+    KAction* endWeek = localCollection->addAction( "endWeek" );
     endWeek->setShortcuts( KStandardShortcut::endOfLine() );
     connect( endWeek, SIGNAL( triggered( bool ) ), SLOT( endOfWeek() ) );
 
     localCollection->readSettings();
+    localCollection->addAssociatedWidget( this );
+    foreach (QAction* action, localCollection->actions())
+#if QT_VERSION < KDE_MAKE_VERSION(4,4,0)
+        action->setShortcutContext(Qt::WidgetShortcut); // remove after Qt4.4 becomes mandatory
+#else
+        action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
+#endif
 }
 
 int KDateTable::posFromDate( const QDate &date_ )
@@ -392,7 +399,7 @@ void KDateTable::paintCell( QPainter *painter, int row, int col )
         } else {
             // paint a day of the current month
             if ( d->useCustomColors ) {
-                KDateTablePrivate::DatePaintingMode * mode = d->customPaintingModes[cellDate.toString()];
+                KDateTablePrivate::DatePaintingMode * mode = d->customPaintingModes[calendar()->formatDate(cellDate)];
                 if ( mode ) {
                     if ( mode->bgMode != NoBgMode ) {
                         QBrush oldbrush = painter->brush();
@@ -489,7 +496,7 @@ void KDateTable::KDateTablePrivate::beginningOfWeek()
 void KDateTable::KDateTablePrivate::endOfWeek()
 {
     // setDate does validity checking for us
-    q->setDate( q->calendar()->addDays( mDate, 
+    q->setDate( q->calendar()->addDays( mDate,
                 q->calendar()->daysInWeek( mDate ) - q->calendar()->dayOfWeek( mDate ) ) );
 }
 
@@ -638,7 +645,7 @@ bool KDateTable::setDate( const QDate& date_ )
     // set weekday number of first day of this month, but this may not be a valid date so fake
     // it if needed e.g. in QDate Mon 1 Jan -4713 is not valid when it should be, so fake as day 1
     QDate firstDayOfMonth;
-    if ( calendar()->setYMD( firstDayOfMonth, 
+    if ( calendar()->setYMD( firstDayOfMonth,
                              calendar()->year( d->mDate ), calendar()->month( d->mDate ), 1 ) ) {
         d->weekDayFirstOfMonth = calendar()->dayOfWeek( firstDayOfMonth );
     } else {
@@ -817,28 +824,29 @@ void KPopupFrame::resizeEvent( QResizeEvent *e )
 void KPopupFrame::popup( const QPoint &pos )
 {
     // Make sure the whole popup is visible.
-    QRect d = KGlobalSettings::desktopGeometry( pos );
+    QRect desktopGeometry = KGlobalSettings::desktopGeometry( pos );
 
     int x = pos.x();
     int y = pos.y();
     int w = width();
     int h = height();
-    if ( x + w > d.x() + d.width() ) {
-        x = d.width() - w;
+    if ( x + w > desktopGeometry.x() + desktopGeometry.width() ) {
+        x = desktopGeometry.width() - w;
     }
-    if ( y + h > d.y() + d.height() ) {
-        y = d.height() - h;
+    if ( y + h > desktopGeometry.y() + desktopGeometry.height() ) {
+        y = desktopGeometry.height() - h;
     }
-    if ( x < d.x() ) {
+    if ( x < desktopGeometry.x() ) {
         x = 0;
     }
-    if ( y < d.y() ) {
+    if ( y < desktopGeometry.y() ) {
         y = 0;
     }
 
     // Pop the thingy up.
     move( x, y );
     show();
+    d->main->setFocus();
 }
 
 int KPopupFrame::exec( const QPoint &pos )
