@@ -149,6 +149,7 @@ class KUrl;
 class KDEUI_EXPORT KLineEdit : public QLineEdit, public KCompletionBase
 {
     friend class KComboBox;
+    friend class KLineEditStyle;
 
     Q_OBJECT
     Q_PROPERTY( bool contextMenuEnabled READ isContextMenuEnabled WRITE setContextMenuEnabled )
@@ -318,7 +319,7 @@ public:
     */
     void setCompletionBox( KCompletionBox *box );
 
-    /*
+    /**
      * This makes the line edit display a grayed-out hinting text as long as
      * the user didn't enter any text. It is often used as indication about
      * the purpose of the line edit.
@@ -552,7 +553,7 @@ protected:
     */
     virtual void dropEvent( QDropEvent * );
 
-    /*
+    /**
     * This function simply sets the lineedit text and
     * highlights the text appropriately if the boolean
     * value is set to true.
