@@ -426,6 +426,12 @@ public:
      */
     void ignoreInitialGeometry();
 
+    /**
+     * Returns the path under which this window's D-Bus object is exported.
+     * @since 4.0.1
+     */
+    QString dbusName() const;
+
 public Q_SLOTS:
     /**
      * Makes a KDE compliant caption.
