@@ -275,7 +275,9 @@ QString KIntSpinBox::textFromValue(int v) const
 int KIntSpinBox::valueFromText(const QString &text) const
 {
     bool ok;
-    return text.toInt(&ok, d->val_base);
+    QString theText = text;
+    if (theText.endsWith(suffix())) theText.chop(suffix().length());
+    return theText.toInt(&ok, d->val_base);
 }
 
 void KIntSpinBox::setEditFocus(bool mark)
@@ -412,7 +414,6 @@ void KIntNumInput::setRange(int lower, int upper, int step)
     d->m_spin->setSingleStep(step);
 
     step = d->m_spin->singleStep(); // maybe QRangeControl didn't like out lineStep?
-    setSliderEnabled(true);
 
     // check that reference point is still inside valid range:
     setReferencePoint( referencePoint() );
