@@ -26,6 +26,7 @@
 #include <kuniqueapplication.h>
 #include <kdebug.h>
 #include <kxerrorhandler.h>
+#include <kxutils.h>
 #include <netwm.h>
 #include <QtGui/QBitmap>
 #include <QDesktopWidget>
@@ -53,10 +54,12 @@ static unsigned long windows_properties[ 2 ] = { NET::ClientList | NET::ClientLi
 				     NET::CurrentDesktop |
 				     NET::DesktopNames |
 				     NET::ActiveWindow |
-				     NET::WorkArea |
+				     NET::WorkArea, /**/
                                      NET::WM2ShowingDesktop };
 
-static unsigned long desktop_properties[ 2 ] = { NET::ClientList |
+// ClientList and ClientListStacking is not per-window information, but a desktop information,
+// so track it even with only INFO_BASIC
+static unsigned long desktop_properties[ 2 ] = { NET::ClientList | NET::ClientListStacking |
                                      NET::Supported |
 				     NET::NumberOfDesktops |
 				     NET::DesktopGeometry |
@@ -64,7 +67,7 @@ static unsigned long desktop_properties[ 2 ] = { NET::ClientList |
 				     NET::CurrentDesktop |
 				     NET::DesktopNames |
 				     NET::ActiveWindow |
-				     NET::WorkArea |
+				     NET::WorkArea, /**/
                                      NET::WM2ShowingDesktop };
 
 class KWindowSystemPrivate
@@ -72,6 +75,7 @@ class KWindowSystemPrivate
 {
 public:
     KWindowSystemPrivate(int _what);
+    void activate();
     QList<WId> windows;
     QList<WId> stackingOrder;
 
@@ -107,9 +111,15 @@ KWindowSystemPrivate::KWindowSystemPrivate(int _what)
       strutSignalConnected( false ),
       what( _what )
 {
-    kapp->installX11EventFilter( this );
+    if (kapp)
+        kapp->installX11EventFilter( this );
     (void ) qApp->desktop(); //trigger desktop widget creation to select root window events
-    activate();
+}
+
+// not virtual, but it's called directly only from init()
+void KWindowSystemPrivate::activate()
+{
+    NETRootInfo::activate();
     updateStackingOrder();
 }
 
@@ -127,12 +137,16 @@ bool KWindowSystemPrivate::x11Event( XEvent * ev )
 
 	if (( m[ PROTOCOLS ] & CurrentDesktop ) && currentDesktop() != old_current_desktop )
 	    emit s_q->currentDesktopChanged( currentDesktop() );
+	if (( m[ PROTOCOLS ] & DesktopViewport ) && mapViewport() && currentDesktop() != old_current_desktop )
+	    emit s_q->currentDesktopChanged( currentDesktop() );
 	if (( m[ PROTOCOLS ] & ActiveWindow ) && activeWindow() != old_active_window )
 	    emit s_q->activeWindowChanged( activeWindow() );
 	if ( m[ PROTOCOLS ] & DesktopNames )
 	    emit s_q->desktopNamesChanged();
 	if (( m[ PROTOCOLS ] & NumberOfDesktops ) && numberOfDesktops() != old_number_of_desktops )
 	    emit s_q->numberOfDesktopsChanged( numberOfDesktops() );
+	if (( m[ PROTOCOLS ] & DesktopGeometry ) && mapViewport() && numberOfDesktops() != old_number_of_desktops )
+	    emit s_q->numberOfDesktopsChanged( numberOfDesktops() );
 	if ( m[ PROTOCOLS ] & WorkArea )
 	    emit s_q->workAreaChanged();
 	if ( m[ PROTOCOLS ] & ClientListStacking ) {
@@ -154,6 +168,8 @@ bool KWindowSystemPrivate::x11Event( XEvent * ev )
             else if( ev->xproperty.atom == XA_WM_ICON_NAME )
                 dirty[ NETWinInfo::PROTOCOLS ] |= NET::WMIconName; // support for old iconic name
         }
+        if( mapViewport() && ( dirty[ NETWinInfo::PROTOCOLS ] & NET::WMState ))
+            dirty[ NETWinInfo::PROTOCOLS ] |= NET::WMDesktop; // possible NET::Sticky change
 	if ( (dirty[ NETWinInfo::PROTOCOLS ] & NET::WMStrut) != 0 ) {
             removeStrutWindow( ev->xany.window );
             if ( !possibleStrutWindows.contains( ev->xany.window ) )
@@ -239,7 +255,7 @@ bool KWindowSystemPrivate::mapViewport()
 {
 // compiz claims support even though it doesn't use virtual desktops :(
 //    if( isSupported( NET::DesktopViewport ) && !isSupported( NET::NumberOfDesktops ))
-    
+
     if( isSupported( NET::DesktopViewport ) && numberOfDesktops( true ) <= 1
         && ( desktopGeometry( currentDesktop( true )).width > QApplication::desktop()->width()
             || desktopGeometry( currentDesktop( true )).height > QApplication::desktop()->height()))
@@ -267,7 +283,7 @@ static void create_atoms( Display* dpy = QX11Info::display()) {
 
         atoms[n] = &_wm_protocols;
         names[n++] = "WM_PROTOCOLS";
-        
+
         atoms[n] = &kwm_utf8_string;
         names[n++] = "UTF8_STRING";
 
@@ -321,8 +337,6 @@ void KWindowSystem::connectNotify( const char* signal )
         what = INFO_WINDOWS;
     else if( QLatin1String( signal ) == SIGNAL(strutChanged()))
         what = INFO_WINDOWS;
-    else if( QLatin1String( signal ) == SIGNAL(stackingOrderChanged()))
-        what = INFO_WINDOWS;
     else if( QLatin1String( signal ) == QMetaObject::normalizedSignature(SIGNAL(windowChanged(WId,const unsigned long*))).data())
         what = INFO_WINDOWS;
     else if( QLatin1String( signal ) ==  QMetaObject::normalizedSignature(SIGNAL(windowChanged(WId,unsigned int))).data())
@@ -350,11 +364,15 @@ void KWindowSystem::init(int what)
        what = INFO_BASIC;
 
     if ( !s_d )
+    {
         g_kwmInstanceContainer->d = new KWindowSystemPrivate(what); // invalidates s_d
+        g_kwmInstanceContainer->d->activate();
+    }
     else if (s_d->what < what)
     {
         delete s_d;
         g_kwmInstanceContainer->d = new KWindowSystemPrivate(what); // invalidates s_d
+        g_kwmInstanceContainer->d->activate();
     }
 }
 
@@ -387,6 +405,7 @@ int KWindowSystem::currentDesktop()
       return 1;
 
     if( mapViewport()) {
+        init( INFO_BASIC );
         KWindowSystemPrivate* const s_d = s_d_func();
         NETPoint p = s_d->desktopViewport( s_d->currentDesktop( true ));
         return viewportToDesktop( QPoint( p.x, p.y ));
@@ -405,6 +424,7 @@ int KWindowSystem::numberOfDesktops()
       return 1;
 
     if( mapViewport()) {
+        init( INFO_BASIC );
         KWindowSystemPrivate* const s_d = s_d_func();
         NETSize s = s_d->desktopGeometry( s_d->currentDesktop( true ));
         return s.width / qApp->desktop()->width() * s.height / qApp->desktop()->height();
@@ -420,6 +440,7 @@ int KWindowSystem::numberOfDesktops()
 void KWindowSystem::setCurrentDesktop( int desktop )
 {
     if( mapViewport()) {
+        init( INFO_BASIC );
         KWindowSystemPrivate* const s_d = s_d_func();
         NETRootInfo info( QX11Info::display(), 0 );
         QPoint pos = desktopToViewport( desktop, true );
@@ -436,7 +457,10 @@ void KWindowSystem::setCurrentDesktop( int desktop )
 void KWindowSystem::setOnAllDesktops( WId win, bool b )
 {
     if( mapViewport()) {
-        setState( b ? NET::Sticky : 0, NET::Sticky );
+        if( b )
+            setState( win, NET::Sticky );
+        else
+            clearState( win, NET::Sticky );
         return;
     }
     NETWinInfo info( QX11Info::display(), win, QX11Info::appRootWindow(), NET::WMDesktop );
@@ -451,11 +475,18 @@ void KWindowSystem::setOnAllDesktops( WId win, bool b )
 void KWindowSystem::setOnDesktop( WId win, int desktop )
 {
     if( mapViewport()) {
+        if( desktop == NET::OnAllDesktops )
+            return setOnAllDesktops( win, true );
+        else
+            clearState( win, NET::Sticky );
+        init( INFO_BASIC );
         QPoint p = desktopToViewport( desktop, false );
-        Window r;
+        Window dummy;
         int x, y;
-        unsigned int w, h, b, d;
-        XGetGeometry( QX11Info::display(), win, &r, &x, &y, &w, &h, &b, &d );
+        unsigned int w, h, b, dp;
+        XGetGeometry( QX11Info::display(), win, &dummy, &x, &y, &w, &h, &b, &dp );
+        // get global position
+        XTranslateCoordinates( QX11Info::display(), win, QX11Info::appRootWindow(), 0, 0, &x, &y, &dummy );
         x += w / 2; // center
         y += h / 2;
         // transform to coordinates on the current "desktop"
@@ -469,11 +500,10 @@ void KWindowSystem::setOnDesktop( WId win, int desktop )
         y += p.y();
         x -= w / 2; // from center back to topleft
         y -= h / 2;
+        p = constrainViewportRelativePosition( QPoint( x, y ));
+        int flags = ( 0x20 << 12 ) | ( 0x03 << 8 ) | 10; // from tool(?), x/y, static gravity
         KWindowSystemPrivate* const s_d = s_d_func();
-        int flags = 0x20 | 0x03 | 10; // from tool(?), x/y, static gravity
-        // This actually doesn't work with Compiz, because it restricts all window movement
-        // to the currently visible area ... *shrug* .
-        s_d->moveResizeWindowRequest( win, flags, x, y, w, h );
+        s_d->moveResizeWindowRequest( win, flags, p.x(), p.y(), w, h );
         return;
     }
     NETWinInfo info( QX11Info::display(), win, QX11Info::appRootWindow(), NET::WMDesktop );
@@ -510,7 +540,7 @@ void KWindowSystem::forceActiveWindow( WId win, long time )
 
 void KWindowSystem::demandAttention( WId win, bool set )
 {
-    NETWinInfo info( QX11Info::display(), win, QX11Info::appRootWindow(), 0 );
+    NETWinInfo info( QX11Info::display(), win, QX11Info::appRootWindow(), NET::WMState );
     info.setState( set ? NET::DemandsAttention : 0, NET::DemandsAttention );
 }
 
@@ -587,35 +617,12 @@ QPixmap KWindowSystem::icon( WId win, int width, int height, bool scale, int fla
 	    XFree((char*)hints);
 
         if (p != None){
-	    Window root;
-	    int x, y;
-	    unsigned int w = 0;
-	    unsigned int h = 0;
-            unsigned int border_w, depth;
-	    XGetGeometry(QX11Info::display(), p, &root,
-		         &x, &y, &w, &h, &border_w, &depth);
-	    if (w > 0 && h > 0){
-	        QPixmap pm(w, h);
-	        // Always detach before doing something behind QPixmap's back.
-	        pm.detach();
-                GC gc = XCreateGC( QX11Info::display(), p, 0, NULL );
-	        XCopyArea(QX11Info::display(), p, pm.handle(), gc,
-		          0, 0, w, h, 0, 0);
-                XFreeGC( QX11Info::display(), gc );
-	        if (p_mask != None){
-	    	    QBitmap bm(w, h);
-                    GC gc = XCreateGC( QX11Info::display(), p_mask, 0, NULL );
-		    XCopyArea(QX11Info::display(), p_mask, bm.handle(), gc,
-			      0, 0, w, h, 0, 0);
-		    pm.setMask(bm);
-                    XFreeGC( QX11Info::display(), gc );
-	        }
-	        if ( scale && width > 0 && height > 0 && !pm.isNull() &&
-		     ( (int) w != width || (int) h != height) ){
-		    result = QPixmap::fromImage( pm.toImage().scaled( width, height, Qt::IgnoreAspectRatio, Qt::SmoothTransformation ) );
-	        } else {
-		    result = pm;
-	        }
+            QPixmap pm = KXUtils::createPixmapFromHandle( p, p_mask );
+            if ( scale && width > 0 && height > 0 && !pm.isNull()
+                 && ( pm.width() != width || pm.height() != height) ){
+                result = QPixmap::fromImage( pm.toImage().scaled( width, height, Qt::IgnoreAspectRatio, Qt::SmoothTransformation ) );
+	    } else {
+                result = pm;
 	    }
         }
     }
@@ -696,13 +703,13 @@ void KWindowSystem::setType( WId win, NET::WindowType windowType )
 
 void KWindowSystem::setState( WId win, unsigned long state )
 {
-    NETWinInfo info( QX11Info::display(), win, QX11Info::appRootWindow(), 0 );
+    NETWinInfo info( QX11Info::display(), win, QX11Info::appRootWindow(), NET::WMState );
     info.setState( state, state );
 }
 
 void KWindowSystem::clearState( WId win, unsigned long state )
 {
-    NETWinInfo info( QX11Info::display(), win, QX11Info::appRootWindow(), 0 );
+    NETWinInfo info( QX11Info::display(), win, QX11Info::appRootWindow(), NET::WMState );
     info.setState( 0, state );
 }
 
@@ -1020,4 +1027,19 @@ QPoint KWindowSystem::desktopToViewport( int desktop, bool absolute )
     return ret;
 }
 
+QPoint KWindowSystem::constrainViewportRelativePosition( const QPoint& pos )
+{
+    init( INFO_BASIC );
+    KWindowSystemPrivate* const s_d = s_d_func();
+    NETSize s = s_d->desktopGeometry( s_d->currentDesktop( true ));
+    NETPoint c = s_d->desktopViewport( s_d->currentDesktop( true ));
+    int x = ( pos.x() + c.x ) % s.width;
+    int y = ( pos.y() + c.y ) % s.height;
+    if( x < 0 )
+        x += s.width;
+    if( y < 0 )
+        y += s.height;
+    return QPoint( x - c.x, y - c.y );
+}
+
 #include "kwindowsystem.moc"
