@@ -392,7 +392,7 @@ KFileWidget::KFileWidget( const KUrl& startDir, QWidget *parent )
     // the Location label/edit
     d->locationLabel = new QLabel(i18n("&Location:"), this);
     d->locationEdit = new KUrlComboBox(KUrlComboBox::Files, true, this);
-    connect( d->locationEdit, SIGNAL( textChanged( const QString& ) ),
+    connect( d->locationEdit, SIGNAL( editTextChanged( const QString& ) ),
              SLOT( _k_slotLocationChanged( const QString& )) );
 
     d->updateLocationWhatsThis();
@@ -989,10 +989,10 @@ void KFileWidgetPrivate::setLocationText( const QString& text )
     // setCurrentItem() will cause textChanged() being emitted,
     // so slotLocationChanged() will be called. Make sure we don't clear
     // the KDirOperator's view-selection in there
-    QObject::disconnect( locationEdit, SIGNAL( textChanged( const QString& ) ),
+    QObject::disconnect( locationEdit, SIGNAL( editTextChanged( const QString& ) ),
                          q, SLOT( _k_slotLocationChanged( const QString& ) ) );
     locationEdit->setCurrentIndex( 0 );
-    QObject::connect( locationEdit, SIGNAL( textChanged( const QString& ) ),
+    QObject::connect( locationEdit, SIGNAL( editTextChanged ( const QString& ) ),
                       q, SLOT( _k_slotLocationChanged( const QString& )) );
     locationEdit->setEditText( text );
 
@@ -1148,7 +1148,7 @@ void KFileWidgetPrivate::_k_urlEntered(const KUrl& url)
 
     locationEdit->blockSignals( blocked );
 
-    urlNavigator->setUrl( directoryUrl( url ) );
+    urlNavigator->setUrl(  url );
 
     QString dir = url.url(KUrl::AddTrailingSlash);
     // is trigged in ctor before completion object is set
@@ -1256,6 +1256,8 @@ void KFileWidgetPrivate::_k_fileCompletion( const QString& match )
 
 void KFileWidgetPrivate::_k_slotLocationChanged( const QString& text )
 {
+    locationEdit->lineEdit()->setModified( true );
+
     if ( text.isEmpty() && ops->view() )
         ops->view()->clearSelection();
 
@@ -1828,6 +1830,8 @@ void KFileWidgetPrivate::updateLocationEditExtension (const QString &lastExtensi
         // catch "double extensions" like ".tar.gz"
         if (lastExtension.length() && fileName.endsWith (lastExtension))
             fileName.truncate (len - lastExtension.length());
+        else if (extension.length() && fileName.endsWith (extension))
+            fileName.truncate (len - extension.length());
         // can only handle "single extensions"
         else
             fileName.truncate (dot);
