@@ -71,6 +71,9 @@ namespace KJS {
 
 static QString cssPropertyName( const Identifier &p, bool& hadPixelPrefix )
 {
+    // The point here is to provide compatibility with IE 
+    // syntax for accessing properties, which camel-cases them
+    // and can add prefixes to produce things like pixelFoo
     QString prop = p.qstring();
     int i = prop.length();
     while ( --i ) {
@@ -149,12 +152,12 @@ DOMCSSStyleDeclaration::~DOMCSSStyleDeclaration()
   ScriptInterpreter::forgetDOMObject(m_impl.get());
 }
 
-ValueImp* DOMCSSStyleDeclaration::getValueProperty(ExecState *exec, int token)
+JSValue* DOMCSSStyleDeclaration::getValueProperty(ExecState *exec, int token)
 {
     //### null decls?
     switch (token) {
     case CssText:
-        return String(m_impl->cssText());
+        return jsString(m_impl->cssText());
     case Length:
         return jsNumber(m_impl->length());
     case ParentRule:
@@ -162,42 +165,14 @@ ValueImp* DOMCSSStyleDeclaration::getValueProperty(ExecState *exec, int token)
     }
 
     assert(0);
-    return Undefined();
+    return jsUndefined();
 }
 
-ValueImp *DOMCSSStyleDeclaration::indexGetter(ExecState* , unsigned index)
+JSValue *DOMCSSStyleDeclaration::indexGetter(ExecState* , unsigned index)
 {
-  return String(m_impl->item(index));
+  return jsString(m_impl->item(index));
 }
 
-ValueImp *DOMCSSStyleDeclaration::cssPropertyGetter(ExecState*, JSObject*, const Identifier& propertyName, const PropertySlot& slot)
-{
-  DOMCSSStyleDeclaration *thisObj = static_cast<DOMCSSStyleDeclaration *>(slot.slotBase());
-
-  // Set up pixelOrPos boolean to handle the fact that
-  // pixelTop returns "CSS Top" as number value in unit pixels
-  // posTop returns "CSS top" as number value in unit pixels _if_ its a
-  // positioned element. if it is not a positioned element, return 0
-  // from MSIE documentation ### IMPLEMENT THAT (Dirk)
-  bool asNumber;
-  DOMString p   = cssPropertyName(propertyName, asNumber);
-  
-  if (asNumber) {
-    CSSValueImpl *v = thisObj->m_impl->getPropertyCSSValue(p);
-    if (v->cssValueType() == DOM::CSSValue::CSS_PRIMITIVE_VALUE)
-      //### FIXME: should this not set exception when type is wrong, or convert?
-      return Number(static_cast<CSSPrimitiveValueImpl*>(v)->floatValue(DOM::CSSPrimitiveValue::CSS_PX));
-  }
-
-  DOM::DOMString str = thisObj->m_impl->getPropertyValue(p);
-  if (!str.isNull())
-    return String(str);
-
-  //we know this css property, but nothing set, return empty then
-  return String("");
-}
-
-
 bool DOMCSSStyleDeclaration::getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot)
 {
 #ifdef KJS_VERBOSE
@@ -212,15 +187,33 @@ bool DOMCSSStyleDeclaration::getOwnPropertySlot(ExecState *exec, const Identifie
     return true;
 
   if (isCSSPropertyName(propertyName)) {
-    slot.setCustom(this, cssPropertyGetter);
-    return true;
+    // Set up pixelOrPos boolean to handle the fact that
+    // pixelTop returns "CSS Top" as number value in unit pixels
+    // posTop returns "CSS top" as number value in unit pixels _if_ its a
+    // positioned element. if it is not a positioned element, return 0
+    // from MSIE documentation ### IMPLEMENT THAT (Dirk)
+    bool asNumber;
+    DOMString p   = cssPropertyName(propertyName, asNumber);
+
+    if (asNumber) {
+      CSSValueImpl *v = m_impl->getPropertyCSSValue(p);
+      if (v && v->cssValueType() == DOM::CSSValue::CSS_PRIMITIVE_VALUE)
+         //### FIXME: should this not set exception when type is wrong, or convert?
+        return getImmediateValueSlot(this, 
+                  jsNumber(static_cast<CSSPrimitiveValueImpl*>(v)->floatValue(DOM::CSSPrimitiveValue::CSS_PX)), slot);
+    }
+
+    DOM::DOMString str = m_impl->getPropertyValue(p);
+    
+    // We want to return at least an empty string here --- see #152791
+    return getImmediateValueSlot(this, jsString(str), slot);
   }
 
   return DOMObject::getOwnPropertySlot(exec, propertyName, slot);
 }
 
 
-void DOMCSSStyleDeclaration::put(ExecState *exec, const Identifier &propertyName, ValueImp *value, int attr )
+void DOMCSSStyleDeclaration::put(ExecState *exec, const Identifier &propertyName, JSValue *value, int attr )
 {
 #ifdef KJS_VERBOSE
   kDebug(6070) << "DOMCSSStyleDeclaration::put " << propertyName.qstring();
@@ -241,17 +234,16 @@ void DOMCSSStyleDeclaration::put(ExecState *exec, const Identifier &propertyName
 #ifdef KJS_VERBOSE
     kDebug(6070) << "DOMCSSStyleDeclaration: prop=" << prop << " propvalue=" << propvalue;
 #endif
-    // Look whether the property is known.d In that case add it as a CSS property.
+    // Look whether the property is known. In that case add it as a CSS property.
     if (int pId = cssPropertyId(prop)) {
       if (propvalue.isEmpty())
         styleDecl.removeProperty(pId);
       else {
         int important = propvalue.indexOf("!important", 0, Qt::CaseInsensitive);
-	int exception_value = exception;
-        if (important == -1)
-            styleDecl.setProperty(pId, DOM::DOMString(propvalue), "", exception_value);
-        else
-            styleDecl.setProperty(pId, DOM::DOMString(propvalue.left(important - 1)), "important", exception_value);
+        if (important == -1) {
+            styleDecl.setProperty(pId, DOM::DOMString(propvalue), false /*important*/, exception);
+        } else
+            styleDecl.setProperty(pId, DOM::DOMString(propvalue.left(important - 1)), true /*important*/, exception);
       }
     }
     else
@@ -260,7 +252,7 @@ void DOMCSSStyleDeclaration::put(ExecState *exec, const Identifier &propertyName
   }
 }
 
-ValueImp *DOMCSSStyleDeclarationProtoFunc::callAsFunction(ExecState *exec, ObjectImp *thisObj, const List &args)
+JSValue *DOMCSSStyleDeclarationProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
   KJS_CHECK_THIS( KJS::DOMCSSStyleDeclaration, thisObj );
   CSSStyleDeclarationImpl& styleDecl = *static_cast<DOMCSSStyleDeclaration*>(thisObj)->impl();
@@ -269,26 +261,26 @@ ValueImp *DOMCSSStyleDeclarationProtoFunc::callAsFunction(ExecState *exec, Objec
 
   switch (id) {
     case DOMCSSStyleDeclaration::GetPropertyValue:
-      return String(styleDecl.getPropertyValue(s));
+      return jsString(styleDecl.getPropertyValue(s));
     case DOMCSSStyleDeclaration::GetPropertyCSSValue:
       return getDOMCSSValue(exec,styleDecl.getPropertyCSSValue(s));
     case DOMCSSStyleDeclaration::RemoveProperty:
-      return String(styleDecl.removeProperty(s));
+      return jsString(styleDecl.removeProperty(s));
     case DOMCSSStyleDeclaration::GetPropertyPriority:
-      return String(styleDecl.getPropertyPriority(s));
+      return jsString(styleDecl.getPropertyPriority(s));
     case DOMCSSStyleDeclaration::SetProperty:
       styleDecl.setProperty(args[0]->toString(exec).domString(),
                             args[1]->toString(exec).domString(),
                             args[2]->toString(exec).domString());
-      return Undefined();
+      return jsUndefined();
     case DOMCSSStyleDeclaration::Item:
-      return String(styleDecl.item(args[0]->toInteger(exec)));
+      return jsString(styleDecl.item(args[0]->toInteger(exec)));
     default:
-      return Undefined();
+      return jsUndefined();
   }
 }
 
-ValueImp *getDOMCSSStyleDeclaration(ExecState *exec, CSSStyleDeclarationImpl *s)
+JSValue *getDOMCSSStyleDeclaration(ExecState *exec, CSSStyleDeclarationImpl *s)
 {
   return cacheDOMObject<CSSStyleDeclarationImpl, DOMCSSStyleDeclaration>(exec, s);
 }
@@ -324,29 +316,29 @@ bool DOMStyleSheet::getOwnPropertySlot(ExecState *exec, const Identifier& proper
   return getStaticValueSlot<DOMStyleSheet, DOMObject>(exec, &DOMStyleSheetTable, this, propertyName, slot);
 }
 
-ValueImp *DOMStyleSheet::getValueProperty(ExecState *exec, int token) const
+JSValue *DOMStyleSheet::getValueProperty(ExecState *exec, int token) const
 {
   StyleSheetImpl &styleSheet = *m_impl;
   switch (token) {
   case Type:
-    return String(styleSheet.type());
+    return jsString(styleSheet.type());
   case Disabled:
-    return Boolean(styleSheet.disabled());
+    return jsBoolean(styleSheet.disabled());
   case OwnerNode:
     return getDOMNode(exec,styleSheet.ownerNode());
   case ParentStyleSheet:
     return getDOMStyleSheet(exec,styleSheet.parentStyleSheet());
   case Href:
-    return String(styleSheet.href());
+    return jsString(styleSheet.href());
   case Title:
-    return String(styleSheet.title());
+    return jsString(styleSheet.title());
   case Media:
     return getDOMMediaList(exec, styleSheet.media());
   }
   return 0;
 }
 
-void DOMStyleSheet::put(ExecState *exec, const Identifier &propertyName, ValueImp *value, int attr)
+void DOMStyleSheet::put(ExecState *exec, const Identifier &propertyName, JSValue *value, int attr)
 {
   StyleSheetImpl &styleSheet = *m_impl;
   if (propertyName == "disabled") {
@@ -356,10 +348,10 @@ void DOMStyleSheet::put(ExecState *exec, const Identifier &propertyName, ValueIm
     DOMObject::put(exec, propertyName, value, attr);
 }
 
-ValueImp *getDOMStyleSheet(ExecState *exec, DOM::StyleSheetImpl* ss)
+JSValue *getDOMStyleSheet(ExecState *exec, DOM::StyleSheetImpl* ss)
 {
   if (!ss)
-    return Null();
+    return jsNull();
 
   DOMObject *ret;
   ScriptInterpreter* interp = static_cast<ScriptInterpreter *>(exec->dynamicInterpreter());
@@ -400,23 +392,23 @@ DOMStyleSheetList::~DOMStyleSheetList()
   ScriptInterpreter::forgetDOMObject(m_impl.get());
 }
 
-ValueImp *DOMStyleSheetList::getValueProperty(ExecState*, int token) const
+JSValue *DOMStyleSheetList::getValueProperty(ExecState*, int token) const
 {
     switch(token) {
     case Length:
-      return Number(m_impl->length());
+      return jsNumber(m_impl->length());
     default:
       assert(0);
-      return Undefined();
+      return jsUndefined();
     }
 }
 
-ValueImp *DOMStyleSheetList::indexGetter(ExecState *exec, unsigned index)
+JSValue *DOMStyleSheetList::indexGetter(ExecState *exec, unsigned index)
 {
   return getDOMStyleSheet(exec, m_impl->item(index));
 }
 
-ValueImp *DOMStyleSheetList::nameGetter(ExecState *exec, JSObject*, const Identifier& propertyName, const PropertySlot& slot)
+JSValue *DOMStyleSheetList::nameGetter(ExecState *exec, JSObject*, const Identifier& propertyName, const PropertySlot& slot)
 {
   DOMStyleSheetList *thisObj = static_cast<DOMStyleSheetList *>(slot.slotBase());
   ElementImpl *element = thisObj->m_doc->getElementById(propertyName.domString());
@@ -452,21 +444,21 @@ bool DOMStyleSheetList::getOwnPropertySlot(ExecState *exec, const Identifier& pr
   return DOMObject::getOwnPropertySlot(exec, propertyName, slot);
 }
 
-ValueImp *DOMStyleSheetList::callAsFunction(ExecState *exec, ObjectImp * /*thisObj*/, const List &args)
+JSValue *DOMStyleSheetList::callAsFunction(ExecState *exec, JSObject * /*thisObj*/, const List &args)
 {
   if (args.size() == 1) {
     // support for styleSheets(<index>) and styleSheets(<name>)
     return get( exec, Identifier(args[0]->toString(exec)) );
   }
-  return Undefined();
+  return jsUndefined();
 }
 
-ValueImp *getDOMStyleSheetList(ExecState *exec, DOM::StyleSheetListImpl* ssl, DOM::DocumentImpl* doc)
+JSValue *getDOMStyleSheetList(ExecState *exec, DOM::StyleSheetListImpl* ssl, DOM::DocumentImpl* doc)
 {
   // Can't use the cacheDOMObject macro because of the doc argument
   DOMObject *ret;
   if (!ssl)
-    return Null();
+    return jsNull();
   ScriptInterpreter* interp = static_cast<ScriptInterpreter *>(exec->dynamicInterpreter());
   if ((ret = interp->getDOMObject(ssl)))
     return ret;
@@ -477,13 +469,13 @@ ValueImp *getDOMStyleSheetList(ExecState *exec, DOM::StyleSheetListImpl* ssl, DO
   }
 }
 
-ValueImp *DOMStyleSheetListFunc::callAsFunction(ExecState *exec, ObjectImp *thisObj, const List &args)
+JSValue *DOMStyleSheetListFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
   KJS_CHECK_THIS( KJS::DOMStyleSheetList, thisObj );
   DOM::StyleSheetListImpl* styleSheetList = static_cast<DOMStyleSheetList *>(thisObj)->impl();
   if (id == DOMStyleSheetList::Item)
     return getDOMStyleSheet(exec, styleSheetList->item(args[0]->toInteger(exec)));
-  return Undefined();
+  return jsUndefined();
 }
 
 // -------------------------------------------------------------------------
@@ -516,24 +508,24 @@ DOMMediaList::~DOMMediaList()
   ScriptInterpreter::forgetDOMObject(m_impl.get());
 }
 
-ValueImp* DOMMediaList::getValueProperty(ExecState*, int token) const
+JSValue* DOMMediaList::getValueProperty(ExecState*, int token) const
 {
   const MediaListImpl& mediaList = *m_impl;
   switch (token)
   {
   case MediaText:
-    return String(mediaList.mediaText());
+    return jsString(mediaList.mediaText());
   case Length:
-    return Number(mediaList.length());
+    return jsNumber(mediaList.length());
   default:
     assert(0);
-    return Undefined();
+    return jsUndefined();
   }
 }
 
-ValueImp *DOMMediaList::indexGetter(ExecState*, unsigned index)
+JSValue *DOMMediaList::indexGetter(ExecState*, unsigned index)
 {
-  return String(m_impl->item(index));
+  return jsString(m_impl->item(index));
 }
 
 bool DOMMediaList::getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot)
@@ -547,7 +539,7 @@ bool DOMMediaList::getOwnPropertySlot(ExecState *exec, const Identifier& propert
   return DOMObject::getOwnPropertySlot(exec, propertyName, slot);
 }
 
-void DOMMediaList::put(ExecState *exec, const Identifier &propertyName, ValueImp *value, int attr)
+void DOMMediaList::put(ExecState *exec, const Identifier &propertyName, JSValue *value, int attr)
 {
   if (propertyName == "mediaText")
     m_impl->setMediaText(value->toString(exec).domString());
@@ -555,26 +547,26 @@ void DOMMediaList::put(ExecState *exec, const Identifier &propertyName, ValueImp
     DOMObject::put(exec, propertyName, value, attr);
 }
 
-ValueImp *getDOMMediaList(ExecState *exec, DOM::MediaListImpl* ml)
+JSValue *getDOMMediaList(ExecState *exec, DOM::MediaListImpl* ml)
 {
   return cacheDOMObject<DOM::MediaListImpl, KJS::DOMMediaList>(exec, ml);
 }
 
-ValueImp *DOMMediaListProtoFunc::callAsFunction(ExecState *exec, ObjectImp *thisObj, const List &args)
+JSValue *DOMMediaListProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
   KJS_CHECK_THIS( KJS::DOMMediaList, thisObj );
   DOM::MediaListImpl& mediaList = *static_cast<DOMMediaList *>(thisObj)->impl();
   switch (id) {
     case DOMMediaList::Item:
-      return String(mediaList.item(args[0]->toInteger(exec)));
+      return jsString(mediaList.item(args[0]->toInteger(exec)));
     case DOMMediaList::DeleteMedium:
       mediaList.deleteMedium(args[0]->toString(exec).domString());
-      return Undefined();
+      return jsUndefined();
     case DOMMediaList::AppendMedium:
       mediaList.appendMedium(args[0]->toString(exec).domString());
-      return Undefined();
+      return jsUndefined();
     default:
-      return Undefined();
+      return jsUndefined();
   }
 }
 
@@ -609,22 +601,23 @@ DOMCSSStyleSheet::DOMCSSStyleSheet(ExecState *exec, DOM::CSSStyleSheetImpl* ss):
 DOMCSSStyleSheet::~DOMCSSStyleSheet()
 {}
 
-ValueImp* DOMCSSStyleSheet::getValueProperty(ExecState *exec, int token)
+JSValue* DOMCSSStyleSheet::getValueProperty(ExecState *exec, int token)
 {
   CSSStyleSheetImpl& cssStyleSheet = *impl();
+  // MSIE does not list the charset rules in its proprietary extension
+  bool omitCharsetRules = true;
   switch (token) {
     case OwnerRule:
       return getDOMCSSRule(exec,cssStyleSheet.ownerRule());
     case CssRules:
+        omitCharsetRules = false;
+        // nobreak
     case Rules: {
-      //### this is a bit odd -- why is the impl returning the wrapper?
-      DOM::CSSRuleList rules = cssStyleSheet.cssRules();
-      SharedPtr<CSSRuleListImpl> ri = static_cast<CSSRuleListImpl*>(rules.handle());
-      return getDOMCSSRuleList(exec, ri.get());
+        return getDOMCSSRuleList(exec, cssStyleSheet.cssRules(omitCharsetRules));
     }
     default:
       assert(0);
-      return Undefined();
+      return jsUndefined();
   }
 }
 
@@ -634,7 +627,7 @@ bool DOMCSSStyleSheet::getOwnPropertySlot(ExecState *exec, const Identifier& pro
   return getStaticValueSlot<DOMCSSStyleSheet, DOMStyleSheet>(exec, &DOMCSSStyleSheetTable, this, propertyName, slot);
 }
 
-ValueImp *DOMCSSStyleSheetProtoFunc::callAsFunction(ExecState *exec, ObjectImp *thisObj, const List &args)
+JSValue *DOMCSSStyleSheetProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
   KJS_CHECK_THIS( KJS::DOMCSSStyleSheet, thisObj );
   DOM::CSSStyleSheetImpl& styleSheet = *static_cast<DOMCSSStyleSheet *>(thisObj)->impl();
@@ -642,28 +635,28 @@ ValueImp *DOMCSSStyleSheetProtoFunc::callAsFunction(ExecState *exec, ObjectImp *
 
   switch (id) {
     case DOMCSSStyleSheet::InsertRule:
-      return Number(styleSheet.insertRule(args[0]->toString(exec).domString(),(long unsigned int)args[1]->toInteger(exec), exception));
+      return jsNumber(styleSheet.insertRule(args[0]->toString(exec).domString(),(long unsigned int)args[1]->toInteger(exec), exception));
     case DOMCSSStyleSheet::DeleteRule:
       styleSheet.deleteRule(args[0]->toInteger(exec), exception);
-      return Undefined();
+      return jsUndefined();
     // IE extensions
     case DOMCSSStyleSheet::AddRule: {
       //Unpassed/-1 means append. Since insertRule is picky (throws exceptions)
       //we adjust it to the desired length
       unsigned long index  = args[2]->toInteger(exec);
-      unsigned long length = styleSheet.cssRules().length();
+      unsigned long length = styleSheet.length();
       if (args[2]->type() == UndefinedType) index = length;
       if (index > length)                   index = length;
       DOM::DOMString str = args[0]->toString(exec).domString() + " { " + args[1]->toString(exec).domString() + " } ";
-      return Number(styleSheet.insertRule(str, index, exception));
+      return jsNumber(styleSheet.insertRule(str, index, exception));
     }
     case DOMCSSStyleSheet::RemoveRule: {
       int index = args.size() > 0 ? args[0]->toInteger(exec) : 0 /*first one*/;
       styleSheet.deleteRule(index, exception);
-      return Undefined();
+      return jsUndefined();
     }
     default:
-      return Undefined();
+      return jsUndefined();
   }
 }
 
@@ -689,18 +682,18 @@ DOMCSSRuleList::~DOMCSSRuleList()
   ScriptInterpreter::forgetDOMObject(m_impl.get());
 }
 
-ValueImp *DOMCSSRuleList::getValueProperty(ExecState*, int token) const
+JSValue *DOMCSSRuleList::getValueProperty(ExecState*, int token) const
 {
   switch (token) {
   case Length:
-    return Number(m_impl->length());
+    return jsNumber(m_impl->length());
   default:
     assert(0);
-    return Undefined();
+    return jsUndefined();
   }
 }
 
-ValueImp *DOMCSSRuleList::indexGetter(ExecState* exec, unsigned index)
+JSValue *DOMCSSRuleList::indexGetter(ExecState* exec, unsigned index)
 {
   return getDOMCSSRule(exec, m_impl->item(index));
 }
@@ -719,7 +712,7 @@ bool DOMCSSRuleList::getOwnPropertySlot(ExecState *exec, const Identifier& prope
   return DOMObject::getOwnPropertySlot(exec, propertyName, slot);
 }
 
-ValueImp *DOMCSSRuleListFunc::callAsFunction(ExecState *exec, ObjectImp *thisObj, const List &args)
+JSValue *DOMCSSRuleListFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
   KJS_CHECK_THIS( KJS::DOMCSSRuleList, thisObj );
   DOM::CSSRuleListImpl& cssRuleList = *static_cast<DOMCSSRuleList *>(thisObj)->impl();
@@ -727,11 +720,11 @@ ValueImp *DOMCSSRuleListFunc::callAsFunction(ExecState *exec, ObjectImp *thisObj
     case DOMCSSRuleList::Item:
       return getDOMCSSRule(exec,cssRuleList.item(args[0]->toInteger(exec)));
     default:
-      return Undefined();
+      return jsUndefined();
   }
 }
 
-ValueImp *getDOMCSSRuleList(ExecState *exec, DOM::CSSRuleListImpl* rl)
+JSValue *getDOMCSSRuleList(ExecState *exec, DOM::CSSRuleListImpl* rl)
 {
   return cacheDOMObject<DOM::CSSRuleListImpl, KJS::DOMCSSRuleList>(exec, rl);
 }
@@ -826,14 +819,14 @@ bool DOMCSSRule::getOwnPropertySlot(ExecState *exec, const Identifier& propertyN
   return getStaticPropertySlot<DOMCSSRuleFunc, DOMCSSRule, DOMObject>(exec, &DOMCSSRuleTable, this, propertyName, slot);
 }
 
-ValueImp *DOMCSSRule::getValueProperty(ExecState *exec, int token) const
+JSValue *DOMCSSRule::getValueProperty(ExecState *exec, int token) const
 {
   CSSRuleImpl &cssRule = *m_impl;
   switch (token) {
   case Type:
-    return Number(cssRule.type());
+    return jsNumber(cssRule.type());
   case CssText:
-    return String(cssRule.cssText());
+    return jsString(cssRule.cssText());
   case ParentStyleSheet:
     return getDOMStyleSheet(exec,cssRule.parentStyleSheet());
   case ParentRule:
@@ -841,7 +834,7 @@ ValueImp *DOMCSSRule::getValueProperty(ExecState *exec, int token) const
 
   // for DOM::CSSRule::STYLE_RULE:
   case Style_SelectorText:
-    return String(static_cast<CSSStyleRuleImpl *>(m_impl.get())->selectorText());
+    return jsString(static_cast<CSSStyleRuleImpl *>(m_impl.get())->selectorText());
   case Style_Style:
     return getDOMCSSStyleDeclaration(exec, static_cast<CSSStyleRuleImpl *>(m_impl.get())->style());
 
@@ -857,13 +850,13 @@ ValueImp *DOMCSSRule::getValueProperty(ExecState *exec, int token) const
 
   // for DOM::CSSRule::PAGE_RULE:
   case Page_SelectorText:
-    return String(static_cast<CSSPageRuleImpl *>(m_impl.get())->selectorText());
+    return jsString(static_cast<CSSPageRuleImpl *>(m_impl.get())->selectorText());
   case Page_Style:
     return getDOMCSSStyleDeclaration(exec, static_cast<CSSPageRuleImpl *>(m_impl.get())->style());
 
   // for DOM::CSSRule::IMPORT_RULE:
   case Import_Href:
-    return String(static_cast<CSSImportRuleImpl *>(m_impl.get())->href());
+    return jsString(static_cast<CSSImportRuleImpl *>(m_impl.get())->href());
   case Import_Media:
     return getDOMMediaList(exec, static_cast<CSSImportRuleImpl *>(m_impl.get())->media());
   case Import_StyleSheet:
@@ -871,22 +864,22 @@ ValueImp *DOMCSSRule::getValueProperty(ExecState *exec, int token) const
 
   // for DOM::CSSRule::CHARSET_RULE:
   case Charset_Encoding:
-    return String(static_cast<CSSCharsetRuleImpl *>(m_impl.get())->encoding());
+    return jsString(static_cast<CSSCharsetRuleImpl *>(m_impl.get())->encoding());
 
   default:
     assert(0);
   }
-  return Undefined();
+  return jsUndefined();
 }
 
-void DOMCSSRule::put(ExecState *exec, const Identifier &propertyName, ValueImp *value, int attr)
+void DOMCSSRule::put(ExecState *exec, const Identifier &propertyName, JSValue *value, int attr)
 {
   const HashTable* table = classInfo()->propHashTable; // get the right hashtable
   const HashEntry* entry = Lookup::findEntry(table, propertyName);
   if (entry) {
     if (entry->attr & Function) // function: put as override property
     {
-      ObjectImp::put(exec, propertyName, value, attr);
+      JSObject::put(exec, propertyName, value, attr);
       return;
     }
     else if ((entry->attr & ReadOnly) == 0) // let lookupPut print the warning if not
@@ -898,7 +891,7 @@ void DOMCSSRule::put(ExecState *exec, const Identifier &propertyName, ValueImp *
   lookupPut<DOMCSSRule, DOMObject>(exec, propertyName, value, attr, &DOMCSSRuleTable, this);
 }
 
-void DOMCSSRule::putValueProperty(ExecState *exec, int token, ValueImp *value, int)
+void DOMCSSRule::putValueProperty(ExecState *exec, int token, JSValue *value, int)
 {
   switch (token) {
   // for DOM::CSSRule::STYLE_RULE:
@@ -921,7 +914,7 @@ void DOMCSSRule::putValueProperty(ExecState *exec, int token, ValueImp *value, i
   }
 }
 
-ValueImp *DOMCSSRuleFunc::callAsFunction(ExecState *exec, ObjectImp *thisObj, const List &args)
+JSValue *DOMCSSRuleFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
   KJS_CHECK_THIS( KJS::DOMCSSRule, thisObj );
   DOM::CSSRuleImpl& cssRule = *static_cast<DOMCSSRule *>(thisObj)->impl();
@@ -929,15 +922,15 @@ ValueImp *DOMCSSRuleFunc::callAsFunction(ExecState *exec, ObjectImp *thisObj, co
   if (cssRule.type() == DOM::CSSRule::MEDIA_RULE) {
     DOM::CSSMediaRuleImpl& rule = static_cast<DOM::CSSMediaRuleImpl&>(cssRule);
     if (id == DOMCSSRule::Media_InsertRule)
-      return Number(rule.insertRule(args[0]->toString(exec).domString(),args[1]->toInteger(exec)));
+      return jsNumber(rule.insertRule(args[0]->toString(exec).domString(),args[1]->toInteger(exec)));
     else if (id == DOMCSSRule::Media_DeleteRule)
       rule.deleteRule(args[0]->toInteger(exec));
   }
 
-  return Undefined();
+  return jsUndefined();
 }
 
-ValueImp *getDOMCSSRule(ExecState *exec, DOM::CSSRuleImpl* r)
+JSValue *getDOMCSSRule(ExecState *exec, DOM::CSSRuleImpl* r)
 {
   return cacheDOMObject<DOM::CSSRuleImpl, KJS::DOMCSSRule>(exec, r);
 }
@@ -967,28 +960,28 @@ bool CSSRuleConstructor::getOwnPropertySlot(ExecState *exec, const Identifier& p
   return getStaticValueSlot<CSSRuleConstructor, DOMObject>(exec, &CSSRuleConstructorTable, this, propertyName, slot);
 }
 
-ValueImp *CSSRuleConstructor::getValueProperty(ExecState *, int token) const
+JSValue *CSSRuleConstructor::getValueProperty(ExecState *, int token) const
 {
   switch (token) {
   case UNKNOWN_RULE:
-    return Number(DOM::CSSRule::UNKNOWN_RULE);
+    return jsNumber(DOM::CSSRule::UNKNOWN_RULE);
   case STYLE_RULE:
-    return Number(DOM::CSSRule::STYLE_RULE);
+    return jsNumber(DOM::CSSRule::STYLE_RULE);
   case CHARSET_RULE:
-    return Number(DOM::CSSRule::CHARSET_RULE);
+    return jsNumber(DOM::CSSRule::CHARSET_RULE);
   case IMPORT_RULE:
-    return Number(DOM::CSSRule::IMPORT_RULE);
+    return jsNumber(DOM::CSSRule::IMPORT_RULE);
   case MEDIA_RULE:
-    return Number(DOM::CSSRule::MEDIA_RULE);
+    return jsNumber(DOM::CSSRule::MEDIA_RULE);
   case FONT_FACE_RULE:
-    return Number(DOM::CSSRule::FONT_FACE_RULE);
+    return jsNumber(DOM::CSSRule::FONT_FACE_RULE);
   case PAGE_RULE:
-    return Number(DOM::CSSRule::PAGE_RULE);
+    return jsNumber(DOM::CSSRule::PAGE_RULE);
   }
   return 0;
 }
 
-ValueImp *getCSSRuleConstructor(ExecState *exec)
+JSValue *getCSSRuleConstructor(ExecState *exec)
 {
   return cacheGlobalObject<CSSRuleConstructor>( exec, "[[cssRule.constructor]]" );
 }
@@ -1015,17 +1008,17 @@ DOMCSSValue::~DOMCSSValue()
   ScriptInterpreter::forgetDOMObject(m_impl.get());
 }
 
-ValueImp *DOMCSSValue::getValueProperty(ExecState*, int token) const
+JSValue *DOMCSSValue::getValueProperty(ExecState*, int token) const
 {
   CSSValueImpl &cssValue = *m_impl;
   switch (token) {
   case CssText:
-    return String(cssValue.cssText());
+    return jsString(cssValue.cssText());
   case CssValueType:
-    return Number(cssValue.cssValueType());
+    return jsNumber(cssValue.cssValueType());
   default:
     assert(0);
-    return Undefined();
+    return jsUndefined();
   }
 }
 
@@ -1034,7 +1027,7 @@ bool DOMCSSValue::getOwnPropertySlot(ExecState *exec, const Identifier& property
   return getStaticValueSlot<DOMCSSValue, DOMObject>(exec, &DOMCSSValueTable, this, propertyName, slot);
 }
 
-void DOMCSSValue::put(ExecState *exec, const Identifier &propertyName, ValueImp *value, int attr)
+void DOMCSSValue::put(ExecState *exec, const Identifier &propertyName, JSValue *value, int attr)
 {
   CSSValueImpl &cssValue = *m_impl;
   if (propertyName == "cssText") {
@@ -1046,11 +1039,11 @@ void DOMCSSValue::put(ExecState *exec, const Identifier &propertyName, ValueImp
     DOMObject::put(exec, propertyName, value, attr);
 }
 
-ValueImp *getDOMCSSValue(ExecState *exec, DOM::CSSValueImpl* v)
+JSValue *getDOMCSSValue(ExecState *exec, DOM::CSSValueImpl* v)
 {
   DOMObject *ret;
   if (!v)
-    return Null();
+    return jsNull();
   ScriptInterpreter* interp = static_cast<ScriptInterpreter *>(exec->dynamicInterpreter());
   if ((ret = interp->getDOMObject(v)))
     return ret;
@@ -1088,22 +1081,22 @@ bool CSSValueConstructor::getOwnPropertySlot(ExecState *exec, const Identifier&
   return getStaticValueSlot<CSSValueConstructor, DOMObject>(exec, &CSSValueConstructorTable, this, propertyName, slot);
 }
 
-ValueImp *CSSValueConstructor::getValueProperty(ExecState *, int token) const
+JSValue *CSSValueConstructor::getValueProperty(ExecState *, int token) const
 {
   switch (token) {
   case CSS_INHERIT:
-    return Number(DOM::CSSValue::CSS_INHERIT);
+    return jsNumber(DOM::CSSValue::CSS_INHERIT);
   case CSS_PRIMITIVE_VALUE:
-    return Number(DOM::CSSValue::CSS_PRIMITIVE_VALUE);
+    return jsNumber(DOM::CSSValue::CSS_PRIMITIVE_VALUE);
   case CSS_VALUE_LIST:
-    return Number(DOM::CSSValue::CSS_VALUE_LIST);
+    return jsNumber(DOM::CSSValue::CSS_VALUE_LIST);
   case CSS_CUSTOM:
-    return Number(DOM::CSSValue::CSS_CUSTOM);
+    return jsNumber(DOM::CSSValue::CSS_CUSTOM);
   }
   return 0;
 }
 
-ValueImp *getCSSValueConstructor(ExecState *exec)
+JSValue *getCSSValueConstructor(ExecState *exec)
 {
   return cacheGlobalObject<CSSValueConstructor>( exec, "[[cssValue.constructor]]" );
 }
@@ -1134,10 +1127,10 @@ DOMCSSPrimitiveValue::DOMCSSPrimitiveValue(ExecState *exec, DOM::CSSPrimitiveVal
   setPrototype(DOMCSSPrimitiveValueProto::self(exec));
 }
 
-ValueImp *DOMCSSPrimitiveValue::getValueProperty(ExecState*, int token)
+JSValue *DOMCSSPrimitiveValue::getValueProperty(ExecState*, int token)
 {
   assert(token == PrimitiveType);
-  return Number(static_cast<CSSPrimitiveValueImpl *>(impl())->primitiveType());
+  return jsNumber(static_cast<CSSPrimitiveValueImpl *>(impl())->primitiveType());
 }
 
 bool DOMCSSPrimitiveValue::getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot)
@@ -1145,7 +1138,7 @@ bool DOMCSSPrimitiveValue::getOwnPropertySlot(ExecState *exec, const Identifier&
   return getStaticValueSlot<DOMCSSPrimitiveValue, DOMCSSValue>(exec, &DOMCSSPrimitiveValueTable, this, propertyName, slot);
 }
 
-ValueImp *DOMCSSPrimitiveValueProtoFunc::callAsFunction(ExecState *exec, ObjectImp *thisObj, const List &args)
+JSValue *DOMCSSPrimitiveValueProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
   KJS_CHECK_THIS( KJS::DOMCSSPrimitiveValue, thisObj );
   CSSPrimitiveValueImpl &val = *static_cast<CSSPrimitiveValueImpl *>(static_cast<DOMCSSPrimitiveValue *>(thisObj)->impl());
@@ -1153,15 +1146,15 @@ ValueImp *DOMCSSPrimitiveValueProtoFunc::callAsFunction(ExecState *exec, ObjectI
   switch (id) {
     case DOMCSSPrimitiveValue::SetFloatValue:
       val.setFloatValue(args[0]->toInteger(exec),args[1]->toNumber(exec), exception);
-      return Undefined();
+      return jsUndefined();
     case DOMCSSPrimitiveValue::GetFloatValue:
       //### FIXME: exception?
-      return Number(val.floatValue(args[0]->toInteger(exec)));
+      return jsNumber(val.floatValue(args[0]->toInteger(exec)));
     case DOMCSSPrimitiveValue::SetStringValue:
       val.setStringValue(args[0]->toInteger(exec),args[1]->toString(exec).domString(), exception);
-      return Undefined();
+      return jsUndefined();
     case DOMCSSPrimitiveValue::GetStringValue:
-      return String(DOM::DOMString(val.getStringValue()));
+      return jsString(DOM::DOMString(val.getStringValue()));
     case DOMCSSPrimitiveValue::GetCounterValue:
       return getDOMCounter(exec,val.getCounterValue());
     case DOMCSSPrimitiveValue::GetRectValue:
@@ -1169,7 +1162,7 @@ ValueImp *DOMCSSPrimitiveValueProtoFunc::callAsFunction(ExecState *exec, ObjectI
     case DOMCSSPrimitiveValue::GetRGBColorValue:
       return getDOMRGBColor(exec,val.getRGBColorValue());
     default:
-      return Undefined();
+      return jsUndefined();
   }
 }
 
@@ -1212,13 +1205,13 @@ bool CSSPrimitiveValueConstructor::getOwnPropertySlot(ExecState *exec, const Ide
   return getStaticValueSlot<CSSPrimitiveValueConstructor, DOMObject>(exec, &CSSPrimitiveValueConstructorTable, this, propertyName, slot);
 }
 
-ValueImp *CSSPrimitiveValueConstructor::getValueProperty(ExecState *, int token) const
+JSValue *CSSPrimitiveValueConstructor::getValueProperty(ExecState *, int token) const
 {
   // We use the token as the value to return directly
-  return Number(token);
+  return jsNumber(token);
 }
 
-ValueImp *getCSSPrimitiveValueConstructor(ExecState *exec)
+JSValue *getCSSPrimitiveValueConstructor(ExecState *exec)
 {
   return cacheGlobalObject<CSSPrimitiveValueConstructor>( exec, "[[cssPrimitiveValue.constructor]]" );
 }
@@ -1239,7 +1232,7 @@ DOMCSSValueList::DOMCSSValueList(ExecState *exec, DOM::CSSValueListImpl* v)
   : DOMCSSValue(exec, v) { }
 
 
-ValueImp *DOMCSSValueList::indexGetter(ExecState *exec, unsigned index)
+JSValue *DOMCSSValueList::indexGetter(ExecState *exec, unsigned index)
 {
   return getDOMCSSValue(exec, impl()->item(index));
 }
@@ -1257,7 +1250,7 @@ bool DOMCSSValueList::getOwnPropertySlot(ExecState *exec, const Identifier& prop
   return DOMCSSValue::getOwnPropertySlot(exec, propertyName, slot);
 }
 
-ValueImp *DOMCSSValueListFunc::callAsFunction(ExecState *exec, ObjectImp *thisObj, const List &args)
+JSValue *DOMCSSValueListFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
   KJS_CHECK_THIS( KJS::DOMCSSValueList, thisObj );
   CSSValueListImpl &valueList = *static_cast<CSSValueListImpl *>(static_cast<DOMCSSValueList *>(thisObj)->impl());
@@ -1265,7 +1258,7 @@ ValueImp *DOMCSSValueListFunc::callAsFunction(ExecState *exec, ObjectImp *thisOb
     case DOMCSSValueList::Item:
       return getDOMCSSValue(exec,valueList.item(args[0]->toInteger(exec)));
     default:
-      return Undefined();
+      return jsUndefined();
   }
 }
 
@@ -1291,7 +1284,7 @@ bool DOMRGBColor::getOwnPropertySlot(ExecState *exec, const Identifier& property
   return getStaticValueSlot<DOMRGBColor, DOMObject>(exec, &DOMRGBColorTable, this, propertyName, slot);
 }
 
-ValueImp *DOMRGBColor::getValueProperty(ExecState *exec, int token) const
+JSValue *DOMRGBColor::getValueProperty(ExecState *exec, int token) const
 {
   int color;
   switch (token) {
@@ -1303,13 +1296,13 @@ ValueImp *DOMRGBColor::getValueProperty(ExecState *exec, int token) const
     color = qBlue(m_color); break;
   default:
     assert(0);
-    return Undefined();
+    return jsUndefined();
   }
 
   return new DOMCSSPrimitiveValue(exec, new CSSPrimitiveValueImpl(color, CSSPrimitiveValue::CSS_NUMBER));
 }
 
-ValueImp *getDOMRGBColor(ExecState *exec, unsigned color)
+JSValue *getDOMRGBColor(ExecState *exec, unsigned color)
 {
   // ### implement equals for RGBColor since they're not refcounted objects
   return new DOMRGBColor(exec, color);
@@ -1343,7 +1336,7 @@ bool DOMRect::getOwnPropertySlot(ExecState *exec, const Identifier& propertyName
   return getStaticValueSlot<DOMRect, DOMObject>(exec, &DOMRectTable, this, propertyName, slot);
 }
 
-ValueImp *DOMRect::getValueProperty(ExecState *exec, int token) const
+JSValue *DOMRect::getValueProperty(ExecState *exec, int token) const
 {
   DOM::RectImpl& rect = *m_impl;
   switch (token) {
@@ -1360,7 +1353,7 @@ ValueImp *DOMRect::getValueProperty(ExecState *exec, int token) const
   }
 }
 
-ValueImp *getDOMRect(ExecState *exec, DOM::RectImpl* r)
+JSValue *getDOMRect(ExecState *exec, DOM::RectImpl* r)
 {
   return cacheDOMObject<DOM::RectImpl, KJS::DOMRect>(exec, r);
 }
@@ -1391,22 +1384,22 @@ bool DOMCounter::getOwnPropertySlot(ExecState *exec, const Identifier& propertyN
   return getStaticValueSlot<DOMCounter, DOMObject>(exec, &DOMCounterTable, this, propertyName, slot);
 }
 
-ValueImp *DOMCounter::getValueProperty(ExecState *, int token) const
+JSValue *DOMCounter::getValueProperty(ExecState *, int token) const
 {
   CounterImpl &counter = *m_impl;
   switch (token) {
   case identifier:
-    return String(counter.identifier());
+    return jsString(counter.identifier());
   case listStyle:
-    return String(khtml::stringForListStyleType((khtml::EListStyleType)counter.listStyle()));
+    return jsString(khtml::stringForListStyleType((khtml::EListStyleType)counter.listStyle()));
   case separator:
-    return String(counter.separator());
+    return jsString(counter.separator());
   default:
     return 0;
   }
 }
 
-ValueImp *getDOMCounter(ExecState *exec, DOM::CounterImpl* c)
+JSValue *getDOMCounter(ExecState *exec, DOM::CounterImpl* c)
 {
   return cacheDOMObject<DOM::CounterImpl, KJS::DOMCounter>(exec, c);
 }
