@@ -546,7 +546,7 @@ UString DOMNode::toString(ExecState *) const
 {
   if (m_impl.isNull())
     return "null";
-  return "[object " + className() + "]";	//krazy:exclude=doublequote_chars DOM demands chars
+  return "[object " + className() + "]";
 }
 
 void DOMNode::setListener(ExecState *exec, int eventId, JSValue* func) const
@@ -623,11 +623,9 @@ JSValue* DOMNodeProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, co
     case DOMNode::Contains:
     {
       DOM::NodeImpl* other = toNode(args[0]);
-      if (other && node.isElementNode())
-      {
-          bool retval = other->isAncestor(&node);
-          return jsBoolean(retval);
-      }
+      if (other && other->isElementNode())
+          return jsBoolean(other->isAncestor(&node));
+      setDOMException(exec, DOMException::TYPE_MISMATCH_ERR); // ### a guess
       return jsUndefined();
     }
     case DOMNode::InsertAdjacentHTML:
@@ -681,6 +679,8 @@ KJS_DEFINE_PROTOTYPE(DOMNodeListProto)
 KJS_IMPLEMENT_PROTOFUNC(DOMNodeListProtoFunc)
 KJS_IMPLEMENT_PROTOTYPE("DOMNodeList",DOMNodeListProto,DOMNodeListProtoFunc)
 
+IMPLEMENT_PSEUDO_CONSTRUCTOR(NodeListPseudoCtor, "NodeList", DOMNodeListProto)
+
 const ClassInfo DOMNodeList::info = { "NodeList", 0, 0, 0 };
 
 DOMNodeList::DOMNodeList(ExecState *exec, DOM::NodeListImpl* l)
@@ -892,6 +892,7 @@ AttrImpl *toAttr(JSValue *val)
   abort              DOMDocument::Abort                        DontDelete|Function 0
   load               DOMDocument::Load                         DontDelete|Function 1
   loadXML            DOMDocument::LoadXML                      DontDelete|Function 2
+  getElementsByClassName DOMDocument::GetElementsByClassName   DontDelete|Function 1
 @end
 */
 
@@ -1046,9 +1047,9 @@ JSValue* DOMDocumentProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj
   case DOMDocument::ImportNode: // DOM2
     return getDOMNode(exec,doc.importNode(toNode(args[0]), args[1]->toBoolean(exec), exception));
   case DOMDocument::CreateElementNS: // DOM2
-    return getDOMNode(exec,doc.createElementNS(args[0]->toString(exec).domString(), args[1]->toString(exec).domString(), exception));
+    return getDOMNode(exec,doc.createElementNS(valueToStringWithNullCheck(exec, args[0]), args[1]->toString(exec).domString(), exception));
   case DOMDocument::CreateAttributeNS: // DOM2
-    return getDOMNode(exec,doc.createAttributeNS(args[0]->toString(exec).domString(),args[1]->toString(exec).domString(), exception));
+    return getDOMNode(exec,doc.createAttributeNS(valueToStringWithNullCheck(exec, args[0]), args[1]->toString(exec).domString(), exception));
   case DOMDocument::GetElementsByTagNameNS: // DOM2
     return getDOMNodeList(exec,doc.getElementsByTagNameNS(args[0]->toString(exec).domString(),
                                                           args[1]->toString(exec).domString()));
@@ -1060,24 +1061,10 @@ JSValue* DOMDocumentProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj
   case DOMDocument::CreateRange:
     return getDOMRange(exec,doc.createRange());
   case DOMDocument::CreateNodeIterator:
-    if (args[2]->type() == NullType) {
         return getDOMNodeIterator(exec,
                                   doc.createNodeIterator(toNode(args[0]),
                                                          (long unsigned int)(args[1]->toNumber(exec)),
-                                                         0,args[3]->toBoolean(exec), exception));
-    }
-    else {
-      JSObject *obj = args[2]->getObject();
-      if (obj)
-      {
-        DOM::CustomNodeFilter *customFilter = new JSNodeFilter(obj);
-        DOM::NodeFilter filter = DOM::NodeFilter::createCustom(customFilter);
-        return getDOMNodeIterator(exec,
-          doc.createNodeIterator(
-            toNode(args[0]),(long unsigned int)(args[1]->toNumber(exec)),
-            filter.handle(),args[3]->toBoolean(exec), exception));
-      }// else?
-    }
+                                                         toNodeFilter(args[2]),args[3]->toBoolean(exec), exception));
   case DOMDocument::CreateTreeWalker:
     return getDOMTreeWalker(exec,doc.createTreeWalker(toNode(args[0]),(long unsigned int)(args[1]->toNumber(exec)),
              toNodeFilter(args[2]),args[3]->toBoolean(exec), exception));
@@ -1115,6 +1102,8 @@ JSValue* DOMDocumentProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj
   case DOMDocument::LoadXML:
     doc.loadXML(s);
     break;
+  case DOMDocument::GetElementsByClassName:
+    return getDOMNodeList(exec, doc.getElementsByClassName(s));
   default:
     break;
   }
@@ -1141,6 +1130,7 @@ JSValue* DOMDocumentProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj
   setAttributeNodeNS	DOMElement::SetAttributeNodeNS	DontDelete|Function 1
   getElementsByTagNameNS DOMElement::GetElementsByTagNameNS	DontDelete|Function 2
   hasAttributeNS	DOMElement::HasAttributeNS	DontDelete|Function 2
+  getElementsByClassName DOMElement::GetElementsByClassName   DontDelete|Function 1
 @end
 */
 KJS_IMPLEMENT_PROTOFUNC(DOMElementProtoFunc)
@@ -1200,16 +1190,19 @@ bool DOMElement::getOwnPropertySlot(ExecState *exec, const Identifier& propertyN
   if (proto->isObject() && static_cast<JSObject *>(proto)->getOwnPropertySlot(exec, propertyName, slot))
     return true;
 
+#if 0 // IE-only feature
   // Give access to attributes
   ElementImpl &element = *static_cast<ElementImpl *>(impl());
   if (element.hasAttribute(propertyName.domString())) {
     slot.setCustom(this, attributeGetter);
     return true;
   }
+#endif
 
   return false;
 }
 
+#if 0
 JSValue *DOMElement::attributeGetter(ExecState*, JSObject*, const Identifier& propertyName, const PropertySlot& slot)
 {
   DOMElement *thisObj = static_cast<DOMElement *>(slot.slotBase());
@@ -1218,6 +1211,7 @@ JSValue *DOMElement::attributeGetter(ExecState*, JSObject*, const Identifier& pr
   DOM::DOMString attr = element->getAttribute(propertyName.domString());
   return getStringOrNull(attr);
 }
+#endif
 
 JSValue* DOMElementProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
@@ -1269,6 +1263,8 @@ JSValue* DOMElementProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj,
       return getDOMNodeList(exec,element.getElementsByTagNameNS(args[0]->toString(exec).domString(),args[1]->toString(exec).domString()));
     case DOMElement::HasAttributeNS: // DOM2
       return jsBoolean(element.hasAttributeNS(args[0]->toString(exec).domString(),args[1]->toString(exec).domString()));
+  case DOMElement::GetElementsByClassName: // HTML 5
+      return getDOMNodeList(exec, element.getElementsByClassName(args[0]->toString(exec).domString()));
   default:
     return jsUndefined();
   }
@@ -1325,7 +1321,8 @@ JSValue* DOMDOMImplementationProtoFunc::callAsFunction(ExecState *exec, JSObject
 
   switch(id) {
   case DOMDOMImplementation::HasFeature:
-    return jsBoolean(implementation.hasFeature(args[0]->toString(exec).domString(),args[1]->toString(exec).domString()));
+    return jsBoolean(implementation.hasFeature(args[0]->toString(exec).domString(),
+                                               valueToStringWithNullCheck(exec, args[1])));
   case DOMDOMImplementation::CreateDocumentType: // DOM2
     return getDOMNode(exec,implementation.createDocumentType(args[0]->toString(exec).domString(),args[1]->toString(exec).domString(),args[2]->toString(exec).domString(),exception));
   case DOMDOMImplementation::CreateDocument: { // DOM2
@@ -1342,7 +1339,9 @@ JSValue* DOMDOMImplementationProtoFunc::callAsFunction(ExecState *exec, JSObject
 
       DOM::DocumentTypeImpl* docType = static_cast<DOM::DocumentTypeImpl*>(supposedDocType);
 
-      DOM::DocumentImpl* doc = implementation.createDocument(args[0]->toString(exec).domString(),args[1]->toString(exec).domString(),docType,exception);
+      DOM::DocumentImpl* doc = implementation.createDocument(valueToStringWithNullCheck(exec, args[0]),
+                                                             valueToStringWithNullCheck(exec, args[1]),
+                                                             docType, exception);
       if (!doc)
         return jsNull();
       KUrl url = static_cast<DocumentImpl*>(part->document().handle())->URL();
@@ -1700,10 +1699,8 @@ JSValue* KJS::getDOMDOMImplementation(ExecState *exec, DOM::DOMImplementationImp
 IMPLEMENT_PSEUDO_CONSTRUCTOR_WITH_PARENT(NodeConstructor, "NodeConstructor", DOMNodeProto, DOMNodeConstants)
 // -------------------------------------------------------------------------
 
-const ClassInfo DOMExceptionConstructor::info = { "DOMExceptionConstructor", 0, 0, 0 };
-
-/* Source for DOMExceptionConstructorTable.
-@begin DOMExceptionConstructorTable 15
+/* Source for DOMExceptionProtoTable.
+@begin DOMExceptionProtoTable 15
   INDEX_SIZE_ERR		DOM::DOMException::INDEX_SIZE_ERR		DontDelete|ReadOnly
   DOMSTRING_SIZE_ERR		DOM::DOMException::DOMSTRING_SIZE_ERR	DontDelete|ReadOnly
   HIERARCHY_REQUEST_ERR		DOM::DOMException::HIERARCHY_REQUEST_ERR	DontDelete|ReadOnly
@@ -1724,62 +1721,23 @@ const ClassInfo DOMExceptionConstructor::info = { "DOMExceptionConstructor", 0,
 @end
 */
 
-DOMExceptionConstructor::DOMExceptionConstructor(ExecState* exec)
-  : DOMObject(exec->lexicalInterpreter()->builtinObjectPrototype())
+DEFINE_CONSTANT_TABLE(DOMExceptionProto)
+IMPLEMENT_CONSTANT_TABLE(DOMExceptionProto, "DOMException")
+
+IMPLEMENT_PSEUDO_CONSTRUCTOR_WITH_PARENT(DOMExceptionPseudoCtor,
+                             "DOMException",
+                             DOMExceptionProto, DOMExceptionProto)
+
+JSDOMException::JSDOMException(ExecState* exec)
+  : DOMObject(DOMExceptionProto::self(exec))
 {
 }
 
-bool DOMExceptionConstructor::getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot)
-{
-  return getStaticValueSlot<DOMExceptionConstructor, DOMObject>(exec, &DOMExceptionConstructorTable, this, propertyName, slot);
-}
-
-JSValue* DOMExceptionConstructor::getValueProperty(ExecState *, int token) const
-{
-  // We use the token as the value to return directly
-  return jsNumber((unsigned int)token);
-#if 0
-  switch (token) {
-  case INDEX_SIZE_ERR:
-    return jsNumber((unsigned int)DOM::DOMException::INDEX_SIZE_ERR);
-  case DOMSTRING_SIZE_ERR:
-    return jsNumber((unsigned int)DOM::DOMException::DOMSTRING_SIZE_ERR);
-  case HIERARCHY_REQUEST_ERR:
-    return jsNumber((unsigned int)DOM::DOMException::HIERARCHY_REQUEST_ERR);
-  case WRONG_DOCUMENT_ERR:
-    return jsNumber((unsigned int)DOM::DOMException::WRONG_DOCUMENT_ERR);
-  case INVALID_CHARACTER_ERR:
-    return jsNumber((unsigned int)DOM::DOMException::INVALID_CHARACTER_ERR);
-  case NO_DATA_ALLOWED_ERR:
-    return jsNumber((unsigned int)DOM::DOMException::NO_DATA_ALLOWED_ERR);
-  case NO_MODIFICATION_ALLOWED_ERR:
-    return jsNumber((unsigned int)DOM::DOMException::NO_MODIFICATION_ALLOWED_ERR);
-  case NOT_FOUND_ERR:
-    return jsNumber((unsigned int)DOM::DOMException::NOT_FOUND_ERR);
-  case NOT_SUPPORTED_ERR:
-    return jsNumber((unsigned int)DOM::DOMException::NOT_SUPPORTED_ERR);
-  case INUSE_ATTRIBUTE_ERR:
-    return jsNumber((unsigned int)DOM::DOMException::INUSE_ATTRIBUTE_ERR);
-  case INVALID_STATE_ERR:
-    return jsNumber((unsigned int)DOM::DOMException::INVALID_STATE_ERR);
-  case SYNTAX_ERR:
-    return jsNumber((unsigned int)DOM::DOMException::SYNTAX_ERR);
-  case INVALID_MODIFICATION_ERR:
-    return jsNumber((unsigned int)DOM::DOMException::INVALID_MODIFICATION_ERR);
-  case NAMESPACE_ERR:
-    return jsNumber((unsigned int)DOM::DOMException::NAMESPACE_ERR);
-  case INVALID_ACCESS_ERR:
-    return jsNumber((unsigned int)DOM::DOMException::INVALID_ACCESS_ERR);
-  default:
-    kDebug(6070) << "WARNING: DOMExceptionConstructor::getValueProperty unhandled token " << token;
-    return jsNull();
-  }
-#endif
-}
+const ClassInfo JSDOMException::info = { "DOMException", 0, 0, 0 };
 
 JSObject *KJS::getDOMExceptionConstructor(ExecState *exec)
 {
-  return cacheGlobalObject<DOMExceptionConstructor>(exec, "[[DOMException.constructor]]");
+  return DOMExceptionPseudoCtor::self(exec);
 }
 
 // -------------------------------------------------------------------------
