@@ -105,8 +105,8 @@ namespace KJS {
            TextAreaAccessKey, TextAreaName, TextAreaDefaultValue, TextAreaSelect,
            TextAreaCols, TextAreaDisabled, TextAreaForm, TextAreaType,
            TextAreaTabIndex, TextAreaReadOnly, TextAreaRows, TextAreaValue,
-           TextAreaBlur, TextAreaFocus, TextAreaSelectionStart, TextAreaSelectionEnd, TextAreaSetSelectionRange, 
-           TextAreaTextLength, ButtonBlur, ButtonFocus, ButtonForm, ButtonTabIndex, ButtonName,
+           TextAreaBlur, TextAreaFocus, TextAreaSelectionStart, TextAreaSelectionEnd, TextAreaSetSelectionRange,
+           TextAreaTextLength, ButtonBlur, ButtonFocus, ButtonClick, ButtonForm, ButtonTabIndex, ButtonName,
            ButtonDisabled, ButtonAccessKey, ButtonType, ButtonValue, LabelHtmlFor,
            LabelForm, LabelAccessKey, FieldSetForm, LegendForm, LegendAccessKey,
            LegendAlign, UListType, UListCompact, OListStart, OListCompact,
@@ -118,7 +118,7 @@ namespace KJS {
            AnchorAccessKey, AnchorCoords, AnchorHref, AnchorProtocol, AnchorHost,
            AnchorCharset, AnchorHrefLang, AnchorHostname, AnchorType, AnchorFocus,
            AnchorPort, AnchorPathName, AnchorHash, AnchorSearch, AnchorName,
-           AnchorRev, AnchorTabIndex, AnchorTarget, AnchorText, AnchorBlur, AnchorClick, 
+           AnchorRev, AnchorTabIndex, AnchorTarget, AnchorText, AnchorBlur, AnchorClick,
            ImageName, ImageAlign, ImageHspace, ImageVspace, ImageUseMap, ImageAlt,
            ImageLowSrc, ImageWidth, ImageIsMap, ImageBorder, ImageHeight,
            ImageLongDesc, ImageSrc, ImageX, ImageY, ImageComplete, ObjectHspace, ObjectHeight, ObjectAlign,
@@ -156,7 +156,7 @@ namespace KJS {
            IFrameMarginHeight, IFrameMarginWidth, IFrameScrolling, IFrameWidth,
            IFrameContentDocument, IFrameContentWindow,
            MarqueeStart, MarqueeStop,
-           GetContext, CanvasWidth, CanvasHeight,
+           CanvasGetContext, CanvasWidth, CanvasHeight, CanvasToDataURL,
            LayerTop, LayerLeft, LayerVisibility, LayerBgColor, LayerClip, LayerDocument, LayerLayers,
            ElementInnerHTML, ElementTitle, ElementId, ElementDir, ElementLang,
            ElementClassName, ElementInnerText, ElementDocument,
@@ -216,6 +216,7 @@ namespace KJS {
 
     virtual JSValue* callAsFunction(ExecState *exec, JSObject* thisObj, const List& args);
     virtual bool implementsCall() const { return true; }
+    virtual bool masqueradeAsUndefined() const;
     virtual bool toBoolean(ExecState *) const;
     virtual void getPropertyNames(ExecState*, PropertyNameArray&);
     enum { Item, NamedItem, Tags };
@@ -239,10 +240,10 @@ namespace KJS {
     HTMLSelectCollection(ExecState *exec, DOM::HTMLCollectionImpl* c, DOM::HTMLSelectElementImpl* e);
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
     virtual void put(ExecState *exec, const Identifier &propertyName, JSValue* value, int attr = None);
-    
+
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
-    
+
     DOM::HTMLSelectElementImpl* toElement() const { return element.get(); }
   private:
     SharedPtr<DOM::HTMLSelectElementImpl> element;
@@ -332,6 +333,7 @@ namespace KJS {
   DEFINE_PSEUDO_CONSTRUCTOR(HTMLLayerElementPseudoCtor)
   DEFINE_PSEUDO_CONSTRUCTOR(HTMLFrameElementPseudoCtor)
   DEFINE_PSEUDO_CONSTRUCTOR(HTMLIFrameElementPseudoCtor)
+  DEFINE_PSEUDO_CONSTRUCTOR(HTMLCollectionPseudoCtor)
   DEFINE_PSEUDO_CONSTRUCTOR(HTMLMarqueeElementPseudoCtor)
   DEFINE_PSEUDO_CONSTRUCTOR(HTMLCanvasElementPseudoCtor)
 } // namespace
