@@ -30,6 +30,19 @@
 
 using namespace KJS;
 
+namespace KJS {
+
+class TraversalExceptionForwarder {
+public:
+  explicit TraversalExceptionForwarder(ExecState *exec) : m_exec(exec), m_code(0) { }
+  ~TraversalExceptionForwarder() { if (m_code) m_exec->setException(static_cast<JSValue*>(m_code)); }
+  operator void* &() { return m_code; }
+private:
+  ExecState *m_exec;
+  void* m_code;
+};
+
+}
 // -------------------------------------------------------------------------
 
 const ClassInfo DOMNodeIterator::info = { "NodeIterator", 0, &DOMNodeIteratorTable, 0 };
@@ -84,13 +97,22 @@ JSValue *DOMNodeIterator::getValueProperty(ExecState *exec, int token) const
 JSValue* DOMNodeIteratorProtoFunc::callAsFunction(ExecState *exec, JSObject* thisObj, const List &)
 {
   KJS_CHECK_THIS( KJS::DOMNodeIterator, thisObj );
-  DOMExceptionTranslator exception(exec);
+  DOMExceptionTranslator      exception(exec);
+  TraversalExceptionForwarder filterException(exec);
   DOM::NodeIteratorImpl& nodeIterator = *static_cast<DOMNodeIterator *>(thisObj)->impl();
   switch (id) {
-  case DOMNodeIterator::PreviousNode:
-    return getDOMNode(exec,nodeIterator.previousNode(exception));
-  case DOMNodeIterator::NextNode:
-    return getDOMNode(exec,nodeIterator.nextNode(exception));
+  case DOMNodeIterator::PreviousNode: {
+    SharedPtr<DOM::NodeImpl> node = nodeIterator.previousNode(exception, filterException);
+    if (!filterException)
+      return getDOMNode(exec, node.get());
+    break;
+  }
+  case DOMNodeIterator::NextNode: {
+    SharedPtr<DOM::NodeImpl> node = nodeIterator.nextNode(exception, filterException);
+    if (!filterException)
+      return getDOMNode(exec, node.get());
+    break;
+  }
   case DOMNodeIterator::Detach:
     nodeIterator.detach(exception);
     return jsUndefined();
@@ -151,44 +173,6 @@ JSValue *KJS::getNodeFilterConstructor(ExecState *exec)
 
 // -------------------------------------------------------------------------
 
-const ClassInfo DOMNodeFilter::info = { "NodeFilter", 0, 0, 0 };
-/*
-@begin DOMNodeFilterProtoTable 1
-  acceptNode	DOMNodeFilter::AcceptNode	DontDelete|Function 0
-@end
-*/
-KJS_DEFINE_PROTOTYPE(DOMNodeFilterProto)
-KJS_IMPLEMENT_PROTOFUNC(DOMNodeFilterProtoFunc)
-KJS_IMPLEMENT_PROTOTYPE("DOMNodeFilter",DOMNodeFilterProto,DOMNodeFilterProtoFunc)
-
-DOMNodeFilter::DOMNodeFilter(ExecState *exec, DOM::NodeFilterImpl* nf)
-  : m_impl(nf) {
-  setPrototype(DOMNodeFilterProto::self(exec));
-}
-
-DOMNodeFilter::~DOMNodeFilter()
-{
-  ScriptInterpreter::forgetDOMObject(m_impl.get());
-}
-
-JSValue *DOMNodeFilterProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
-{
-  KJS_CHECK_THIS( KJS::DOMNodeFilter, thisObj );
-  DOM::NodeFilterImpl& nodeFilter = *static_cast<DOMNodeFilter *>(thisObj)->impl();
-  switch (id) {
-    case DOMNodeFilter::AcceptNode:
-      return jsNumber(nodeFilter.acceptNode(toNode(args[0])));
-  }
-  return jsUndefined();
-}
-
-JSValue *KJS::getDOMNodeFilter(ExecState *exec, DOM::NodeFilterImpl* nf)
-{
-  return cacheDOMObject<DOM::NodeFilterImpl, DOMNodeFilter>(exec, nf);
-}
-
-// -------------------------------------------------------------------------
-
 const ClassInfo DOMTreeWalker::info = { "TreeWalker", 0, &DOMTreeWalkerTable, 0 };
 /*
 @begin DOMTreeWalkerTable 5
@@ -222,6 +206,13 @@ DOMTreeWalker::~DOMTreeWalker()
   ScriptInterpreter::forgetDOMObject(m_impl.get());
 }
 
+void DOMTreeWalker::mark()
+{
+  JSNodeFilter* filt = JSNodeFilter::fromDOMFilter(impl()->getFilter());
+  if (filt)
+    filt->mark();
+}
+
 bool DOMTreeWalker::getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot)
 {
   return getStaticValueSlot<DOMTreeWalker,DOMObject>(exec, &DOMTreeWalkerTable, this, propertyName, slot);
@@ -250,8 +241,9 @@ JSValue *DOMTreeWalker::getValueProperty(ExecState *exec, int token) const
 void DOMTreeWalker::put(ExecState *exec, const Identifier &propertyName,
                            JSValue *value, int attr)
 {
+  DOMExceptionTranslator exception(exec);
   if (propertyName == "currentNode") {
-    m_impl->setCurrentNode(toNode(value));
+    m_impl->setCurrentNode(toNode(value), exception);
   }
   else
     JSObject::put(exec, propertyName, value, attr);
@@ -261,21 +253,22 @@ JSValue* DOMTreeWalkerProtoFunc::callAsFunction(ExecState *exec, JSObject *thisO
 {
   KJS_CHECK_THIS( KJS::DOMTreeWalker, thisObj );
   DOM::TreeWalkerImpl& treeWalker = *static_cast<DOMTreeWalker *>(thisObj)->impl();
+  TraversalExceptionForwarder   filterException(exec);
   switch (id) {
     case DOMTreeWalker::ParentNode:
-      return getDOMNode(exec,treeWalker.parentNode());
+      return getDOMNode(exec,treeWalker.parentNode(filterException));
     case DOMTreeWalker::FirstChild:
-      return getDOMNode(exec,treeWalker.firstChild());
+      return getDOMNode(exec,treeWalker.firstChild(filterException));
     case DOMTreeWalker::LastChild:
-      return getDOMNode(exec,treeWalker.lastChild());
+      return getDOMNode(exec,treeWalker.lastChild(filterException));
     case DOMTreeWalker::PreviousSibling:
-      return getDOMNode(exec,treeWalker.previousSibling());
+      return getDOMNode(exec,treeWalker.previousSibling(filterException));
     case DOMTreeWalker::NextSibling:
-      return getDOMNode(exec,treeWalker.nextSibling());
+      return getDOMNode(exec,treeWalker.nextSibling(filterException));
     case DOMTreeWalker::PreviousNode:
-      return getDOMNode(exec,treeWalker.previousSibling());
+      return getDOMNode(exec,treeWalker.previousNode(filterException));
     case DOMTreeWalker::NextNode:
-      return getDOMNode(exec,treeWalker.nextNode());
+      return getDOMNode(exec,treeWalker.nextNode(filterException));
   }
   return jsUndefined();
 }
@@ -285,27 +278,53 @@ JSValue *KJS::getDOMTreeWalker(ExecState *exec, DOM::TreeWalkerImpl* tw)
   return cacheDOMObject<DOM::TreeWalkerImpl, DOMTreeWalker>(exec, tw);
 }
 
+// -------------------------------------------------------------------------
+
+
 DOM::NodeFilterImpl* KJS::toNodeFilter(JSValue *val)
 {
   JSObject *obj = val->getObject();
-  if (!obj || !obj->inherits(&DOMNodeFilter::info))
+  if (!obj)
     return 0;
 
-  const DOMNodeFilter *dobj = static_cast<const DOMNodeFilter*>(obj);
-  return dobj->impl();
+  return new JSNodeFilter(obj);
 }
 
-// -------------------------------------------------------------------------
-
-JSNodeFilter::JSNodeFilter(JSObject *_filter) : DOM::CustomNodeFilter(), filter( _filter )
+JSValue *KJS::getDOMNodeFilter(ExecState *exec, DOM::NodeFilterImpl* nf)
 {
+  if (nf->isJSFilter()) {
+    return static_cast<JSNodeFilter*>(nf)->filter();
+  }
+
+  return jsNull();
 }
 
+JSNodeFilter::JSNodeFilter(JSObject *_filter) : m_filter( _filter )
+{}
+
 JSNodeFilter::~JSNodeFilter()
+{}
+
+void JSNodeFilter::mark()
 {
+  if (!m_filter->marked())
+    m_filter->mark();
 }
 
-short JSNodeFilter::acceptNode(const DOM::Node &n)
+bool JSNodeFilter::isJSFilter() const
+{
+  return true;
+}
+
+JSNodeFilter* JSNodeFilter::fromDOMFilter(DOM::NodeFilterImpl* nf)
+{
+  if (!nf || !nf->isJSFilter())
+    return 0;
+
+  return static_cast<JSNodeFilter*>(nf);
+}
+
+short JSNodeFilter::acceptNode(const DOM::Node &n, void*& bindingsException)
 {
   KHTMLView *view = static_cast<DOM::DocumentImpl *>( n.handle()->docPtr() )->view();
   if (!view)
@@ -315,13 +334,26 @@ short JSNodeFilter::acceptNode(const DOM::Node &n)
   KJSProxy *proxy = part->jScript();
   if (proxy) {
     ExecState *exec = proxy->interpreter()->globalExec();
-    JSObject *acceptNodeFunc = filter->get(exec, "acceptNode")->getObject();
-    if (acceptNodeFunc && acceptNodeFunc->implementsCall()) {
+    JSObject* fn = 0;
+
+    // Use a function given directly, or extract it from the acceptNode field
+    if (m_filter->implementsCall()) {
+      fn = m_filter;
+    } else {
+      JSObject *acceptNodeFunc = m_filter->get(exec, "acceptNode")->getObject();
+      if (acceptNodeFunc && acceptNodeFunc->implementsCall())
+        fn = acceptNodeFunc;
+    }
+
+    if (fn) {
       List args;
       args.append(getDOMNode(exec,n.handle()));
-      JSValue *result = acceptNodeFunc->call(exec,filter,args);
-      if (exec->hadException())
-	exec->clearException();
+      JSValue *result = fn->call(exec, m_filter, args);
+      if (exec->hadException()) {
+        bindingsException = exec->exception();
+        exec->clearException();
+        return DOM::NodeFilter::FILTER_REJECT; // throw '1' isn't accept :-)
+      }
       return result->toInteger(exec);
     }
   }
