@@ -348,7 +348,7 @@ QByteArray HTMLFormElementImpl::formData(bool& ok)
                     form_data.append(hstr);
                     form_data.append(*it);
                     form_data.append("\r\n");
-                    
+
 		    // reset unsubmittedFormChange flag
                     if (current->id() == ID_INPUT &&
                         static_cast<HTMLInputElementImpl*>(current)->inputType() == HTMLInputElementImpl::TEXT)
@@ -796,7 +796,7 @@ void HTMLFormElementImpl::registerFormElement(HTMLGenericFormElementImpl *e)
 
 void HTMLFormElementImpl::removeFormElement(HTMLGenericFormElementImpl *e)
 {
-    int i = formElements.indexOf(e); 
+    int i = formElements.indexOf(e);
     if (i != -1)
         formElements.removeAt(i);
 }
@@ -808,7 +808,7 @@ void HTMLFormElementImpl::registerImgElement(HTMLImageElementImpl *e)
 
 void HTMLFormElementImpl::removeImgElement(HTMLImageElementImpl *e)
 {
-    int i = imgElements.indexOf(e); 
+    int i = imgElements.indexOf(e);
     if (i != -1)
         imgElements.removeAt(i);
 }
@@ -1116,7 +1116,14 @@ NodeImpl::Id HTMLButtonElementImpl::id() const
 
 DOMString HTMLButtonElementImpl::type() const
 {
-    return getAttribute(ATTR_TYPE);
+    switch (m_type) {
+        case SUBMIT:
+            return "submit";
+        case RESET:
+            return "reset";
+        case BUTTON:
+            return "button";
+    }
 }
 
 void HTMLButtonElementImpl::blur()
@@ -1135,9 +1142,10 @@ void HTMLButtonElementImpl::parseAttribute(AttributeImpl *attr)
     switch(attr->id())
     {
     case ATTR_TYPE:
-        if ( strcasecmp( attr->value(), "submit" ) == 0 )
-            m_type = SUBMIT;
-        else if ( strcasecmp( attr->value(), "reset" ) == 0 )
+        // Per WF2.0, any invalid values are to be ignored -- and hence
+        // handled as the default, SUBMIT.
+        m_type = SUBMIT;
+        if ( strcasecmp( attr->value(), "reset" ) == 0 )
             m_type = RESET;
         else if ( strcasecmp( attr->value(), "button" ) == 0 )
             m_type = BUTTON;
@@ -1382,6 +1390,9 @@ void HTMLInputElementImpl::restoreState(const QString &state)
         m_value = DOMString(state.left(state.length()-1));
         setChanged();
         break;
+    case HIDDEN:
+        // Don't mess with those...
+        break;
     default:
         setValue(DOMString(state.left(state.length()-1)));
         m_unsubmittedFormChange = state.endsWith('M');
@@ -1479,6 +1490,18 @@ void HTMLInputElementImpl::parseAttribute(AttributeImpl *attr)
     }
 }
 
+void HTMLInputElementImpl::copyNonAttributeProperties(const ElementImpl* source)
+{
+    const HTMLInputElementImpl *e =
+        static_cast<const HTMLInputElementImpl*>(source);
+
+    m_value = e->m_value;
+    m_checked = e->m_checked;
+    // ### copy more?
+
+    HTMLGenericFormElementImpl::copyNonAttributeProperties(source);
+}
+
 void HTMLInputElementImpl::attach()
 {
     assert(!attached());
@@ -2682,10 +2705,10 @@ HTMLSelectElementImpl *HTMLOptionElementImpl::getSelect() const
 /*
  The rules for storing the value are simple:
 
- If there is no renderer, either m_value or defaultValue() is definitive, 
+ If there is no renderer, either m_value or defaultValue() is definitive,
     depending on whether m_initialized is true or not.
  If there is a renderer, m_render->text() is definitive. During its construction,
-    m_value is initialized if needed,  so there is no longer any need to worry 
+    m_value is initialized if needed,  so there is no longer any need to worry
     about default values or not.
 */
 
