@@ -350,8 +350,7 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
                 n->attach();
             if (n->maintainsState()) {
                 document->registerMaintainsState(n);
-                QString state(document->nextState());
-                if (!state.isNull()) n->restoreState(state);
+                document->attemptRestoreState(n);
             }
             n->close();
 #endif
@@ -1660,8 +1659,7 @@ void KHTMLParser::popOneBlock(bool delBlock)
     if((Elem->node != current)) {
         if (current->maintainsState() && document){
             document->registerMaintainsState(current);
-            QString state(document->nextState());
-            if (!state.isNull()) current->restoreState(state);
+            document->attemptRestoreState(current);
         }
         current->close();
     }
@@ -1716,6 +1714,12 @@ void KHTMLParser::createHead()
         delete head;
         head = 0;
     }
+        
+    // If the body does not exist yet, then the <head> should be pushed as the current block.
+    if (head && !body) {
+        pushBlock(head->id(), tagPriority(head->id()));
+        setCurrent(head);
+    }
 }
 
 NodeImpl *KHTMLParser::handleIsindex( Token *t )
