@@ -59,7 +59,7 @@
 #include <kdesktopfile.h>
 #include <kinputdialog.h>
 #include <ktemporaryfile.h>
-#include "khtml_factory.h"
+#include "khtml_global.h"
 #include <kstandardaction.h>
 #include <kactioncollection.h>
 #include <kactionmenu.h>
@@ -560,7 +560,7 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
     // frameset? -> add "Reload Frame" etc.
     if (!hasSelection) {
         if ( khtml->parentPart() ) {
-            KActionMenu* menu = new KActionMenu(this);
+            KActionMenu* menu = new KActionMenu( i18nc("@title:menu HTML frame/iframe", "Frame"), this);
             KAction *action = new KAction( i18n( "Open in New &Window" ), this );
             d->m_actionCollection->addAction( "frameinwindow", action );
             action->setIcon( KIcon( "window-new" ) );
@@ -589,7 +589,7 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
 
             action = new KAction( i18n( "Print Frame..." ), this );
             d->m_actionCollection->addAction( "printFrame", action );
-            action->setIcon( KIcon( "print-frame" ) );
+            action->setIcon( KIcon( "document-print-frame" ) );
             connect( action, SIGNAL(triggered(bool)), d->m_khtml->browserExtension(), SLOT( print() ) );
             menu->addAction(action);
 
@@ -611,7 +611,7 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
             action->setSeparator(true);
             menu->addAction(action);
 
-            if ( KHTMLFactory::defaultHTMLSettings()->isAdFilterEnabled() ) {
+            if ( KHTMLGlobal::defaultHTMLSettings()->isAdFilterEnabled() ) {
                 if ( khtml->d->m_frame->m_type == khtml::ChildFrame::IFrame ) {
                     action = new KAction( i18n( "Block IFrame..." ), this );
                     d->m_actionCollection->addAction( "blockiframe", action );
@@ -671,7 +671,7 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
         connect( action, SIGNAL(triggered(bool)), this, SLOT( slotViewImage() ) );
         partActions.append(action);
 
-        if (KHTMLFactory::defaultHTMLSettings()->isAdFilterEnabled()) {
+        if (KHTMLGlobal::defaultHTMLSettings()->isAdFilterEnabled()) {
             action = new KAction( i18n( "Block Image..." ), this );
             d->m_actionCollection->addAction( "blockimage", action );
             connect( action, SIGNAL(triggered(bool)), this, SLOT( slotBlockImage() ) );
@@ -750,7 +750,7 @@ void KHTMLPopupGUIClient::slotSaveImageAs()
 void KHTMLPopupGUIClient::slotBlockHost()
 {
     QString name=d->m_imageURL.protocol()+"://"+d->m_imageURL.host()+"/*";
-    KHTMLFactory::defaultHTMLSettings()->addAdFilter( name );
+    KHTMLGlobal::defaultHTMLSettings()->addAdFilter( name );
     d->m_khtml->reparseConfiguration();
 }
 
@@ -763,7 +763,7 @@ void KHTMLPopupGUIClient::slotBlockImage()
                                          d->m_imageURL.url(),
                                          &ok);
     if ( ok ) {
-        KHTMLFactory::defaultHTMLSettings()->addAdFilter( url );
+        KHTMLGlobal::defaultHTMLSettings()->addAdFilter( url );
         d->m_khtml->reparseConfiguration();
     }
 }
@@ -776,7 +776,7 @@ void KHTMLPopupGUIClient::slotBlockIFrame()
                                                d->m_khtml->url().url(),
                                                &ok );
     if ( ok ) {
-        KHTMLFactory::defaultHTMLSettings()->addAdFilter( url );
+        KHTMLGlobal::defaultHTMLSettings()->addAdFilter( url );
         d->m_khtml->reparseConfiguration();
     }
 }
@@ -959,8 +959,8 @@ void KHTMLPopupGUIClient::saveURL( QWidget* parent, const KUrl &url, const KUrl
           bool downloadViaKIO = true;
           if ( !url.isLocalFile() )
           {
-            KConfigGroup cfg = KSharedConfig::openConfig("konquerorrc", KConfig::CascadeConfig)->group("HTML Settings");
-            QString downloadManger = cfg.readPathEntry("DownloadManager");
+            KConfigGroup cfg = KSharedConfig::openConfig("konquerorrc", KConfig::NoGlobals)->group("HTML Settings");
+            QString downloadManger = cfg.readPathEntry("DownloadManager", QString());
             if (!downloadManger.isEmpty())
             {
                 // then find the download manager location
@@ -1083,9 +1083,10 @@ void KHTMLZoomFactorAction::slotTriggered(QAction* action)
     int idx = selectableActionGroup()->actions().indexOf(action);
 
     if (idx == 0)
-        m_part->setZoomFactor(100);
+        m_part->setFontScaleFactor(100);
     else
-        m_part->setZoomFactor(fastZoomSizes[fastZoomSizeCount/2 + (m_direction ? 1 : -1)*idx]);
+        m_part->setFontScaleFactor(fastZoomSizes[fastZoomSizeCount/2 + (m_direction ? 1 : -1)*idx]);
+    setCurrentAction( 0L );
 }
 
 #include "khtml_ext.moc"
