@@ -241,6 +241,11 @@ CachedCSSStyleSheet::CachedCSSStyleSheet(const DOMString &url, const QString &st
     m_sheet = DOMString(stylesheet_data);
 }
 
+bool khtml::isAcceptableCSSMimetype( const DOM::DOMString& mimetype )
+{
+    // matches Mozilla's check (cf. nsCSSLoader.cpp)
+    return mimetype.isEmpty() || mimetype == "text/css" || mimetype == "application/x-unknown-content-type";
+}
 
 void CachedCSSStyleSheet::ref(CachedObjectClient *c)
 {
@@ -250,7 +255,7 @@ void CachedCSSStyleSheet::ref(CachedObjectClient *c)
 	if (m_hadError)
 	    c->error( m_err, m_errText );
 	else
-	    c->setStyleSheet( m_url, m_sheet, m_charset );
+	    c->setStyleSheet( m_url, m_sheet, m_charset, m_mimetype );
     }
 }
 
@@ -285,7 +290,7 @@ void CachedCSSStyleSheet::checkNotify()
     CDEBUG << "CachedCSSStyleSheet:: finishedLoading " << m_url.string() << endl;
 
     for (QHashIterator<CachedObjectClient*,CachedObjectClient*> it( m_clients ); it.hasNext();)
-        it.next().value()->setStyleSheet( m_url, m_sheet, m_charset );
+        it.next().value()->setStyleSheet( m_url, m_sheet, m_charset, m_mimetype );
 }
 
 
@@ -334,11 +339,13 @@ CachedScript::CachedScript(DocLoader* dl, const DOMString &url, KIO::CacheContro
     // load the file
     Cache::loader()->load(dl, this, false);
     m_loading = true;
+    m_hadError = false;
 }
 
 CachedScript::CachedScript(const DOMString &url, const QString &script_data)
     : CachedObject(url, Script, KIO::CC_Verify, script_data.length())
 {
+    m_hadError = false;
     m_loading = false;
     m_status = Persistent;
     m_script = DOMString(script_data);
@@ -374,7 +381,8 @@ void CachedScript::checkNotify()
 
 void CachedScript::error( int /*err*/, const char* /*text*/ )
 {
-    m_loading = false;
+    m_hadError = true;
+    m_loading  = false;
     checkNotify();
 }
 
@@ -1263,6 +1271,7 @@ void Loader::servePendingRequests()
         }
 
         connect( job, SIGNAL( result( KJob * ) ), this, SLOT( slotFinished( KJob * ) ) );
+        connect( job, SIGNAL( mimetype( KIO::Job *, const QString& ) ), this, SLOT( slotMimetype( KIO::Job *, const QString& ) ) );
         connect( job, SIGNAL( data( KIO::Job*, const QByteArray &)),
                  SLOT( slotData( KIO::Job*, const QByteArray &)));
 
@@ -1273,6 +1282,16 @@ void Loader::servePendingRequests()
     }
 }
 
+void Loader::slotMimetype( KIO::Job *j, const QString& s )
+{
+    KIO::TransferJob* job = static_cast<KIO::TransferJob*>(j);
+    Request *r = m_requestsLoading.value( j );
+    if (!r)
+        return;
+    CachedObject *o = r->object;
+    o->m_mimetype = s;
+}
+
 void Loader::slotFinished( KJob* job )
 {
   KIO::TransferJob* j = static_cast<KIO::TransferJob*>(job);
@@ -1756,7 +1775,7 @@ void Cache::insertInLRUList(CachedObject *object)
 // --------------------------------------
 
 void CachedObjectClient::updatePixmap(const QRect&, CachedImage *) {}
-void CachedObjectClient::setStyleSheet(const DOM::DOMString &/*url*/, const DOM::DOMString &/*sheet*/, const DOM::DOMString &/*charset*/) {}
+void CachedObjectClient::setStyleSheet(const DOM::DOMString &/*url*/, const DOM::DOMString &/*sheet*/, const DOM::DOMString &/*charset*/, const DOM::DOMString &/*mimetype*/) {}
 void CachedObjectClient::notifyFinished(CachedObject * /*finishedObj*/) {}
 void CachedObjectClient::error(int /*err*/, const QString &/*text*/) {}
 
