@@ -134,6 +134,7 @@ namespace khtml
 	int size() const { return m_size; }
 
         bool isLoaded() const { return !m_loading; }
+        bool hadError() const { return m_hadError; }
 
         bool free() const { return m_free; }
 
@@ -157,11 +158,15 @@ namespace khtml
         QString accept() const { return m_accept; }
         void setAccept(const QString &_accept) { m_accept = _accept; }
 
+        // the mimetype that was eventually used
+        QString mimetype() const { return m_mimetype; }
+
     protected:
         void setSize(int size);
         QHash<CachedObjectClient*,CachedObjectClient*> m_clients;
 	DOM::DOMString m_url;
         QString m_accept;
+        QString m_mimetype;
         Request *m_request;
 	Type m_type;
 	Status m_status;
@@ -181,9 +186,12 @@ namespace khtml
         CachedObject* m_prev;
         friend class Cache;
         friend class ::KHTMLPart;
+        friend class Loader;
     };
 
 
+    bool isAcceptableCSSMimetype( const DOM::DOMString& mimetype );
+
     /**
      * a cached style sheet. also used for loading xml documents.
      *
@@ -207,6 +215,7 @@ namespace khtml
         void setCharsetHint( const QString& charset ) { m_charsetHint = charset; }
         void setCharset( const QString& charset ) { m_charset = charset; }
 
+
     protected:
         void checkNotify();
 
@@ -433,6 +442,7 @@ namespace khtml
      */
     class Loader : public QObject
     {
+
 	Q_OBJECT
 
     public:
@@ -454,6 +464,7 @@ namespace khtml
 
     protected Q_SLOTS:
 	void slotFinished( KJob * );
+	void slotMimetype( KIO::Job *, const QString& s);
 	void slotData( KIO::Job *, const QByteArray & );
 	void servePendingRequests();
 
@@ -463,7 +474,7 @@ namespace khtml
 #ifdef HAVE_LIBJPEG
         // TODO KJPEGFormatType m_jpegloader;
 #endif
-        QTimer m_timer;
+        QTimer m_timer;     
     };
 
         /**
