@@ -359,12 +359,6 @@ inline QChar::Direction BidiIterator::direction() const
     RenderText *renderTxt = static_cast<RenderText *>( obj );
     if ( pos >= renderTxt->stringLength() )
         return QChar::DirON;
-#ifdef __GNUC__
-	#warning "KDE4: hack for testregression, remove when main branch"
-#endif
-    if (renderTxt->text()[pos].unicode() == 0x2D)
-        return QChar::DirET;
-
 
     return renderTxt->text()[pos].direction();
 }
@@ -787,6 +781,7 @@ void RenderBlock::computeHorizontalPositionsForLine(InlineFlowBox* lineBox, Bidi
                 // Only justify text with white-space: normal.
                 if (r->obj->style()->whiteSpace() == NORMAL) {
                     spaceAdd = (availableWidth - totWidth)*spaces/numSpaces;
+                    spaceAdd = qMax(0, spaceAdd);
                     static_cast<InlineTextBox*>(r->box)->setSpaceAdd(spaceAdd);
                     totWidth += spaceAdd;
                 }
@@ -1741,7 +1736,7 @@ RootInlineBox* RenderBlock::determineStartPosition(bool fullLayout, BidiIterator
         int cnt = 0;
         for (curr = firstRootBox(); curr && !curr->isDirty(); curr = curr->nextRootBox()) cnt++;
         if (curr) {
-            kDebug( 6040 ) << "found dirty line at " << cnt;
+            // kDebug( 6040 ) << "found dirty line at " << cnt;
             // We have a dirty line.
             if (RootInlineBox* prevRootBox = curr->prevRootBox()) {
                 // We have a previous line.
@@ -1751,7 +1746,7 @@ RootInlineBox* RenderBlock::determineStartPosition(bool fullLayout, BidiIterator
                     curr = prevRootBox;
             }
         } else {
-            kDebug( 6040 ) << "No dirty line found";
+            // kDebug( 6040 ) << "No dirty line found";
             // No dirty lines were found.
             // If the last line didn't break cleanly, treat it as dirty.
             if (lastRootBox() && !lastRootBox()->endsWithBreak())
@@ -1870,6 +1865,37 @@ static void setStaticPosition( RenderBlock* p, RenderObject *o, bool *needToSetS
       if (needToSetStaticY) *needToSetStaticY = nssy;
 }
 
+static inline bool requiresLineBox(BidiIterator& it)
+{
+    if (it.obj->isFloatingOrPositioned() || it.obj->isInlineFlow())
+        return false;
+    
+    if (it.obj->style()->preserveWS() || it.obj->isBR())
+        return true;
+
+    switch (it.current().unicode()) {
+        case 0x0009: // ASCII tab
+        case 0x000A: // ASCII line feed
+        case 0x000C: // ASCII form feed
+        case 0x0020: // ASCII space
+        case 0x200B: // Zero-width space
+            return false;
+    }
+    return true;
+}
+
+bool RenderBlock::inlineChildNeedsLineBox(RenderObject* inlineObj) // WC: generatesLineBoxesForInlineChild
+{
+    assert(inlineObj->parent() == this);
+
+    BidiIterator it(this, inlineObj, 0);
+    BidiState state;
+    while (!it.atEnd() && !requiresLineBox(it))
+        it.increment(state);
+
+    return !it.atEnd();
+}
+
 BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi)
 {
     int width = lineWidth(m_height);
@@ -1883,21 +1909,7 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
     BidiIterator posStart = start;
     bool hadPosStart = false;
 
-    // eliminate spaces at beginning of line
-    // remove leading spaces.  Any inline flows we encounter will be empty and should also
-    // be skipped.
-    while (!start.atEnd() && (start.obj->isInlineFlow() || (!start.obj->style()->preserveWS() && !start.obj->isBR() &&
-#ifndef QT_NO_UNICODETABLES
-        ( (start.current().unicode() == (ushort)0x0020) || // ASCII space
-          (start.current().unicode() == (ushort)0x0009) || // ASCII tab
-          (start.current().unicode() == (ushort)0x000A) || // ASCII line feed
-          (start.current().unicode() == (ushort)0x000C) || // ASCII form feed
-          (start.current().unicode() == (ushort)0x200B) || // Zero-width space
-          start.obj->isFloatingOrPositioned() )
-#else
-	      ( start.current() == ' ' || start.current() == '\n' || start.obj->isFloatingOrPositioned() )
-#endif
-            ))) {
+    while (!start.atEnd() && !requiresLineBox(start)) {
         if( start.obj->isFloating() || start.obj->isPosWithStaticDim()) {
             RenderObject *o = start.obj;
             // add to special objects...
@@ -2017,6 +2029,12 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
         } else if (o->isInlineFlow()) {
             // Only empty inlines matter.  We treat those similarly to replaced elements.
             KHTMLAssert(!o->firstChild());
+            if (o->isWordBreak()) {
+                w += tmpW;
+                tmpW = 0;
+                lBreak.obj = o;
+                lBreak.pos = 0;
+            }
             tmpW += o->marginLeft()+o->borderLeft()+o->paddingLeft()+
                     o->marginRight()+o->borderRight()+o->paddingRight();
         } else if ( o->isReplaced() || o->isGlyph() ) {
