@@ -417,7 +417,7 @@ void RenderBox::paintRootBoxDecorations(PaintInfo& paintInfo, int _tx, int _ty)
     // hence, paint the background even in the margin areas (unlike for every other element!)
     // I just love these little inconsistencies .. :-( (Dirk)
     QRect cr = paintInfo.r.intersected(QRect(bx, by, bw, bh));
-    paintBackgrounds(paintInfo.p, bgColor, bgLayer, cr, bx, by, bw, bh);
+    paintAllBackgrounds(paintInfo.p, bgColor, bgLayer, cr, bx, by, bw, bh);
 
     if(style()->hasBorder())
         paintBorder( paintInfo.p, _tx, _ty, w, h, style() );
@@ -440,21 +440,21 @@ void RenderBox::paintBoxDecorations(PaintInfo& paintInfo, int _tx, int _ty)
    // since the root could be inline and wrapped in an anonymous block.
 
    if (!isBody() || !document()->isHTMLDocument() || document()->documentElement()->renderer()->style()->hasBackground())
-        paintBackgrounds(paintInfo.p, style()->backgroundColor(), style()->backgroundLayers(), cr, _tx, _ty, w, h);
+        paintAllBackgrounds(paintInfo.p, style()->backgroundColor(), style()->backgroundLayers(), cr, _tx, _ty, w, h);
 
     if(style()->hasBorder()) {
         paintBorder(paintInfo.p, _tx, _ty, w, h, style());
     }
 }
 
-void RenderBox::paintBackgrounds(QPainter *p, const QColor& c, const BackgroundLayer* bgLayer, QRect clipr, int _tx, int _ty, int w, int height)
+void RenderBox::paintAllBackgrounds(QPainter *p, const QColor& c, const BackgroundLayer* bgLayer, QRect clipr, int _tx, int _ty, int w, int height)
 {
     if (!bgLayer) return;
-    paintBackgrounds(p, c, bgLayer->next(), clipr, _tx, _ty, w, height);
-    paintBackground(p, c, bgLayer, clipr, _tx, _ty, w, height);
+    paintAllBackgrounds(p, c, bgLayer->next(), clipr, _tx, _ty, w, height);
+    paintOneBackground(p, c, bgLayer, clipr, _tx, _ty, w, height);
 }
 
-void RenderBox::paintBackground(QPainter *p, const QColor& c, const BackgroundLayer* bgLayer, QRect clipr, int _tx, int _ty, int w, int height)
+void RenderBox::paintOneBackground(QPainter *p, const QColor& c, const BackgroundLayer* bgLayer, QRect clipr, int _tx, int _ty, int w, int height)
 {
     paintBackgroundExtended(p, c, bgLayer, clipr, _tx, _ty, w, height,
                             borderLeft(), borderRight(), paddingLeft(), paddingRight(), 
@@ -534,10 +534,19 @@ void RenderBox::paintBackgroundExtended(QPainter *p, const QColor &c, const Back
     if (!bgLayer->next() && isRoot())
     {
         KHTMLView* v = canvas()->view();
-        if (!v || !v->m_kwp->isRedirected()) {
-            if (bgColor.alpha() == 0)
-                bgColor = p->background().color();
-            bgColor.setAlpha(255);
+        if (bgColor.alpha() != 255) {
+            if (v && v->m_kwp->isRedirected()) {
+                RenderStyle* ws = v->m_kwp->renderWidget()->style();
+                // only set static background on transparent subframes if the underlying RenderWidget background
+                // has something to show through.
+                if (!ws || !ws->backgroundColor().isValid() || !ws->backgroundColor().alpha() == 255
+                        || ws->hasBackgroundImage())
+                    v->setHasStaticBackground();
+            } else {
+                if (bgColor.alpha() == 0)
+                    bgColor = p->background().color();
+                bgColor.setAlpha(255);
+            }
         }
     }
 
@@ -635,43 +644,13 @@ void RenderBox::paintBackgroundExtended(QPainter *p, const QColor &c, const Back
         else
         {
             //fixed
-            QRect vr = viewRect();
-            int pw = vr.width();
-            int ph = vr.height();
-            scaledImageWidth = pw;
-            scaledImageHeight = ph;
-            calculateBackgroundSize(bgLayer, scaledImageWidth, scaledImageHeight);
-            EBackgroundRepeat bgr = bgLayer->backgroundRepeat();
-
-            int xPosition = bgLayer->backgroundXPosition().minWidth(pw-scaledImageWidth);
-            if (bgr == NO_REPEAT || bgr == REPEAT_Y) {
-                cw = qMin(scaledImageWidth, pw - xPosition);
-                cx = vr.x() + xPosition;
-            } else {
-                cw = pw;
-                cx = vr.x();
-                if (scaledImageWidth > 0)
-                    sx = scaledImageWidth - xPosition % scaledImageWidth;
-            }
-
-            int yPosition = bgLayer->backgroundYPosition().minWidth(ph-scaledImageHeight);
-            if (bgr == NO_REPEAT || bgr == REPEAT_X) {
-                ch = qMin(scaledImageHeight, ph - yPosition);
-                cy = vr.y() + yPosition;
-            } else {
-                ch = ph;
-                cy = vr.y();
-                if (scaledImageHeight > 0)
-                    sy = scaledImageHeight - yPosition % scaledImageHeight;
-            }
-
-            QRect fix(cx, cy, cw, ch);
+            QRect fix = getFixedBackgroundImageRect(bgLayer, sx, sy, scaledImageWidth, scaledImageHeight);
             QRect ele(_tx, _ty, w, h);
             QRect b = fix.intersect(ele);
 
             //kDebug() <<" ele is " << ele << " b is " << b << " fix is " << fix;
-            sx+=b.x()-cx;
-            sy+=b.y()-cy;
+            sx+=b.x()-fix.x();
+            sy+=b.y()-fix.y();
             cx=b.x();cy=b.y();cw=b.width();ch=b.height();
         }
         // restrict painting to repaint-clip
@@ -698,6 +677,41 @@ void RenderBox::paintBackgroundExtended(QPainter *p, const QColor &c, const Back
 
 }
 
+QRect RenderBox::getFixedBackgroundImageRect( const BackgroundLayer* bgLayer, int& sx, int& sy, int& scaledImageWidth, int& scaledImageHeight )
+{
+    int cx, cy, cw, ch;
+    QRect vr = viewRect();
+    int pw = vr.width();
+    int ph = vr.height();
+    scaledImageWidth = pw;
+    scaledImageHeight = ph;
+    calculateBackgroundSize(bgLayer, scaledImageWidth, scaledImageHeight);
+    EBackgroundRepeat bgr = bgLayer->backgroundRepeat();
+
+    int xPosition = bgLayer->backgroundXPosition().minWidth(pw-scaledImageWidth);
+    if (bgr == NO_REPEAT || bgr == REPEAT_Y) {
+        cw = qMin(scaledImageWidth, pw - xPosition);
+        cx = vr.x() + xPosition;
+    } else {
+        cw = pw;
+        cx = vr.x();
+        if (scaledImageWidth > 0)
+            sx = scaledImageWidth - xPosition % scaledImageWidth;
+    }
+
+    int yPosition = bgLayer->backgroundYPosition().minWidth(ph-scaledImageHeight);
+    if (bgr == NO_REPEAT || bgr == REPEAT_X) {
+        ch = qMin(scaledImageHeight, ph - yPosition);
+        cy = vr.y() + yPosition;
+    } else {
+        ch = ph;
+        cy = vr.y();
+        if (scaledImageHeight > 0)
+            sy = scaledImageHeight - yPosition % scaledImageHeight;
+    }
+    return QRect(cx, cy, cw, ch);
+}
+
 void RenderBox::outlineBox(QPainter *p, int _tx, int _ty, const char *color)
 {
     p->setPen(QPen(QColor(color), 1, Qt::DotLine));
