@@ -4,6 +4,7 @@
  * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
  *           (C) 2003 Apple Computer, Inc.
  *           (C) 2005 Allan Sandfeld Jensen (kde@carewolf.com)
+ *           (C) 2007-2008 Germain Garand (germain@ebooksfrance.org)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -212,8 +213,11 @@ void RenderCanvas::setNeedsWidgetMasks( bool b )
     m_needsWidgetMasks = b;
     KHTMLWidget* k = dynamic_cast<KHTMLWidget*>(m_view);
     // ### should be reversible
-    if (k && b && k->m_kwp->isRedirected())
+    if (k && b && k->m_kwp->isRedirected()) {
         k->m_kwp->setIsRedirected(!b);
+        if (k->m_kwp->renderWidget())
+            k->m_kwp->renderWidget()->setNeedsLayout(true);
+    }
 }
 
 void RenderCanvas::updateDocumentSize()
@@ -289,6 +293,45 @@ void RenderCanvas::updateDocSizeAfterLayerTranslation( RenderObject* o, bool pos
     updateDocumentSize();
 }
 
+QRegion RenderCanvas::staticRegion() const
+{
+   QRegion ret = QRegion();
+
+   // position:fixed objects
+   if (m_positionedObjects) {
+       RenderObject* obj;
+       QListIterator<RenderObject*> it(*m_positionedObjects);
+        while (it.hasNext()) {
+            obj = it.next();
+            if (obj->style()->position() == FIXED && obj->layer()) {
+                ret += obj->layer()->paintedRegion(layer());
+                assert( m_fixedPosition.contains(obj) );
+            }
+        }
+   }
+
+   // background-attachment:fixed images
+   QSetIterator<RenderObject *> i(m_fixedBackground);
+   while (i.hasNext()) {
+       RenderObject *ro = i.next();
+       if (ro && ro->isBox()) {
+           int d1, d2, d3, d4;
+           const BackgroundLayer* bgLayer = ro->style()->backgroundLayers();
+           while (bgLayer) {
+               CachedImage* bg = bgLayer->backgroundAttachment() ? 0 : bgLayer->backgroundImage();
+               if (bg && bg->isComplete() && !bg->isTransparent() && !bg->isErrorImage()) {
+                   int xpos, ypos;
+                   absolutePosition(xpos,ypos);
+                   ret += static_cast<RenderBox*>(ro)->getFixedBackgroundImageRect(bgLayer, d1, d2, d3, d4)
+                             .intersected(QRect(xpos,ypos,ro->width(),ro->height()));
+               }
+               bgLayer = bgLayer->next();
+           }
+       }
+   }
+   return ret;
+}
+
 bool RenderCanvas::needsFullRepaint() const
 {
     return m_needsFullRepaint || m_pagedMode;
@@ -442,6 +485,26 @@ static QRect enclosingPositionedRect (RenderObject *n)
     return rect;
 }
 
+void RenderCanvas::addStaticObject( RenderObject*o, bool pos)
+{ 
+    QSet<RenderObject*>& set = pos ? m_fixedPosition : m_fixedBackground;
+    if (!o || !o->isBox() || set.contains(o))
+        return;
+    set.insert(o);
+    if (view())
+        view()->addStaticObject(pos);
+}
+
+void RenderCanvas::removeStaticObject( RenderObject*o, bool pos)
+{
+    QSet<RenderObject*>& set = pos ? m_fixedPosition : m_fixedBackground;
+    if (!o || !o->isBox() || !set.contains(o))
+        return;
+    set.remove(o); 
+    if (view())
+        view()->removeStaticObject(pos);
+}
+
 QRect RenderCanvas::selectionRect() const
 {
     RenderObject *r = m_selectionStart;
