@@ -242,6 +242,7 @@ RenderCheckBox::RenderCheckBox(HTMLInputElementImpl *element)
     b->setChecked(element->checked());
 
     connect(b,SIGNAL(stateChanged(int)),this,SLOT(slotStateChanged(int)));
+    m_ignoreStateChanged = false;
 }
 
 
@@ -260,14 +261,19 @@ void RenderCheckBox::calcMinMaxWidth()
 
 void RenderCheckBox::updateFromElement()
 {
-    if (widget()->isChecked() != element()->checked())
+    if (widget()->isChecked() != element()->checked()) {
+        m_ignoreStateChanged = true; // We don't want an onchange here,
+                                     // or us getting yanked in a recalcStyle in the process, etc.
         widget()->setChecked(element()->checked());
+        m_ignoreStateChanged = false;
+    }
 
     RenderButton::updateFromElement();
 }
 
 void RenderCheckBox::slotStateChanged(int state)
 {
+    if (m_ignoreStateChanged) return;
     element()->setChecked(state == Qt::Checked);
 
     ref();
@@ -294,7 +300,7 @@ bool RenderCheckBox::handleEvent(const DOM::EventImpl& ev)
 
 RenderRadioButton::RenderRadioButton(HTMLInputElementImpl *element)
     : RenderButton(element)
-{
+{    
     RadioButtonWidget* b = new RadioButtonWidget(view()->widget());
     b->setMouseTracking(true);
     b->setAutoExclusive(false);
@@ -304,11 +310,14 @@ RenderRadioButton::RenderRadioButton(HTMLInputElementImpl *element)
     b->setChecked(element->checked());
 
     connect(b,SIGNAL(toggled(bool)),this,SLOT(slotToggled(bool)));
+    m_ignoreToggled = false;
 }
 
 void RenderRadioButton::updateFromElement()
 {
+    m_ignoreToggled = true;
     widget()->setChecked(element()->checked());
+    m_ignoreToggled = false;
 
     RenderButton::updateFromElement();
 }
@@ -328,6 +337,9 @@ void RenderRadioButton::calcMinMaxWidth()
 
 void RenderRadioButton::slotToggled(bool activated)
 {
+    if (m_ignoreToggled)
+      return;
+
     if(activated) {
       ref();
       element()->onChange();
@@ -498,7 +510,7 @@ void LineEditWidget::slotCheckSpelling()
     Sonnet::Dialog *spellDialog = new Sonnet::Dialog(new Sonnet::BackgroundChecker(this), 0);
     connect(spellDialog, SIGNAL(replace( const QString&, int,const QString&)), this, SLOT(spellCheckerCorrected( const QString&, int,const QString&)));
     connect(spellDialog, SIGNAL(misspelling( const QString&, int)), this, SLOT(spellCheckerMisspelling(const QString &,int)));
-    connect(spellDialog, SIGNAL(done(const QString&)), this, SLOT(spellCheckerDone(const QString&)));
+    connect(spellDialog, SIGNAL(done(const QString&)), this, SLOT(slotSpellCheckDone(const QString&)));
     connect(spellDialog, SIGNAL(cancel()), this, SLOT(spellCheckerFinished()));
     connect(spellDialog, SIGNAL(stop()), this, SLOT(spellCheckerFinished()));
     spellDialog->setBuffer(text());
@@ -876,7 +888,7 @@ void RenderFieldset::paintBoxDecorations(PaintInfo& pI, int _tx, int _ty)
     _ty += yOff - borderTopExtra();
 
     QRect cr = QRect(_tx, _ty, w, h).intersected( pI.r );
-    paintBackground(pI.p, style()->backgroundColor(), style()->backgroundLayers(), cr, _tx, _ty, w, h);
+    paintOneBackground(pI.p, style()->backgroundColor(), style()->backgroundLayers(), cr, _tx, _ty, w, h);
 
     if ( style()->hasBorder() )
 	    paintBorderMinusLegend(pI.p, _tx, _ty, w, h, style(), legend->xPos(), legend->width(), legendBottom);
@@ -1608,13 +1620,13 @@ void TextAreaWidget::contextMenuEvent(QContextMenuEvent * e)
         popup->addSeparator();
 
         popup->addAction( m_findAction );
-        m_findAction->setEnabled( !toPlainText().isEmpty() );
+        m_findAction->setEnabled( !document()->isEmpty() );
 
         popup->addAction( m_findNextAction );
         m_findNextAction->setEnabled( m_find != 0 );
 
         popup->addAction( m_replaceAction );
-        m_replaceAction->setEnabled( !toPlainText().isEmpty() );
+        m_replaceAction->setEnabled( !document()->isEmpty() );
     }
 
     popup->exec(e->globalPos());
@@ -1757,7 +1769,7 @@ void TextAreaWidget::slotFindNext()
 
 void TextAreaWidget::slotFind()
 {
-    if( toPlainText().isEmpty() )  // saves having to track the text changes
+    if( document()->isEmpty() )  // saves having to track the text changes
         return;
 
     if ( m_findDlg ) {
@@ -1777,7 +1789,7 @@ void TextAreaWidget::slotFind()
 
 void TextAreaWidget::slotReplace()
 {
-    if( toPlainText().isEmpty() )  // saves having to track the text changes
+    if( document()->isEmpty() )  // saves having to track the text changes
         return;
 
     if ( m_repDlg ) {
