@@ -94,7 +94,7 @@ void RenderListItem::detach()
     RenderBlock::detach();
 }
 
-static RenderObject* getParentOfFirstLineBox(RenderObject* curr, RenderObject* marker)
+static RenderObject* getParentOfFirstLineBox(RenderBlock* curr, RenderObject* marker)
 {
     RenderObject* firstChild = curr->firstChild();
     if (!firstChild)
@@ -105,7 +105,7 @@ static RenderObject* getParentOfFirstLineBox(RenderObject* curr, RenderObject* m
         if (currChild == marker)
             continue;
 
-        if (currChild->isInline())
+        if (currChild->isInline()&& (!currChild->isInlineFlow() || curr->inlineChildNeedsLineBox(currChild)))
             return curr;
 
         if (currChild->isFloating() || currChild->isPositioned())
@@ -114,12 +114,12 @@ static RenderObject* getParentOfFirstLineBox(RenderObject* curr, RenderObject* m
         if (currChild->isTable() || !currChild->isRenderBlock())
             break;
 
-        if (currChild->style()->htmlHacks() && currChild->element() &&
+        if (curr->isListItem() && currChild->style()->htmlHacks() && currChild->element() &&
             (currChild->element()->id() == ID_UL || currChild->element()->id() == ID_OL ||
              currChild->element()->id() == ID_DIR || currChild->element()->id() == ID_MENU))
             break;
 
-        RenderObject* lineBox = getParentOfFirstLineBox(currChild, marker);
+        RenderObject* lineBox = getParentOfFirstLineBox(static_cast<RenderBlock*>(currChild), marker);
         if (lineBox)
             return lineBox;
     }
@@ -127,6 +127,13 @@ static RenderObject* getParentOfFirstLineBox(RenderObject* curr, RenderObject* m
     return 0;
 }
 
+static RenderObject* firstNonMarkerChild(RenderObject* parent)
+{
+    RenderObject* result = parent->firstChild();
+    while (result && result->isListMarker())
+        result = result->nextSibling();
+    return result;
+}
 
 void RenderListItem::updateMarkerLocation()
 {
@@ -150,7 +157,7 @@ void RenderListItem::updateMarkerLocation()
                 markerPar->removeChild(m_marker);
             if (!lineBoxParent)
                 lineBoxParent = this;
-            lineBoxParent->addChild(m_marker, lineBoxParent->firstChild());
+            lineBoxParent->addChild(m_marker, firstNonMarkerChild(lineBoxParent));
             m_deleteMarker = false;
             if (!m_marker->minMaxKnown())
                 m_marker->calcMinMaxWidth();
@@ -211,7 +218,7 @@ RenderListMarker::~RenderListMarker()
 
 void RenderListMarker::setStyle(RenderStyle *s)
 {
-    if ( style() && s->listStylePosition() != style()->listStylePosition() )
+    if ( style() && (s->listStylePosition() != style()->listStylePosition()  || s->listStyleType() != style()->listStyleType()) )
         setNeedsLayoutAndMinMaxRecalc();
 
     RenderBox::setStyle(s);
