@@ -184,7 +184,8 @@ RenderObject::RenderObject(DOM::NodeImpl* node)
       m_needsPageClear( false ),
       m_containsPageBreak( false ),
       m_hasOverflowClip(false),
-      m_inPosObjectList(false)
+      m_inPosObjectList(false),
+      m_doNotDelete(false)
 {
   assert( node );
   if (node->getDocument()->documentElement() == node) setIsRoot(true);
@@ -1358,6 +1359,31 @@ void RenderObject::setStyle(RenderStyle *style)
     if (style->hasAutoZIndex() && (isRoot() || style->opacity() < 1.0f))
         style->setZIndex( 0 );
 
+    if ( d > RenderStyle::Position &&
+         (style->hasFixedBackgroundImage() != (m_style && m_style->hasFixedBackgroundImage())
+            || (style->position() == FIXED) != (m_style && (m_style->position() == FIXED)))
+            && canvas() && canvas()->view() ) {
+       // some sort of fixed object is added or removed. Let's find out more and report to the canvas,
+       // so that it does some bookkeeping and optimizes the view's background display mode accordingly.
+       bool fixedBG = style->hasFixedBackgroundImage();
+       bool oldFixedBG = m_style && m_style->hasFixedBackgroundImage();
+       bool fixedPos = (style->position() == FIXED);
+       bool oldFixedPos = m_style && (m_style->position() == FIXED);
+       if (fixedBG != oldFixedBG) {
+           if (fixedBG) {
+               canvas()->addStaticObject(this);
+           } else {
+               canvas()->removeStaticObject(this);
+           }
+       }
+       if (fixedPos != oldFixedPos) {
+           if (fixedPos)
+               canvas()->addStaticObject( this, true /*positioned*/ );
+           else
+               canvas()->removeStaticObject( this, true );
+       }
+    }
+
     RenderStyle *oldStyle = m_style;
     m_style = style;
 
@@ -2327,17 +2353,29 @@ void RenderObject::updateWidgetMasks() {
 QRegion RenderObject::visibleFlowRegion(int x, int y) const
 {
     QRegion r;
+    bool returnSelf = false;
     for (RenderObject* ro=firstChild();ro;ro=ro->nextSibling()) {
-        if( !ro->layer() && !ro->isInlineFlow() && ro->style()->visibility() == VISIBLE) {
+        if( !ro->layer() && !ro->isFloating() && ro->style()->visibility() == VISIBLE) {
+            // ### fix horizontal float extent
             const RenderStyle *s = ro->style();
-            if (ro->isRelPositioned())
-                static_cast<const RenderBox*>(ro)->relativePositionOffset(x,y);
-            if ( s->backgroundImage() || s->backgroundColor().isValid() || s->hasBorder() )
-                r += QRect(x + ro->effectiveXPos(),y + ro->effectiveYPos(), ro->effectiveWidth(), ro->effectiveHeight());
-            else
+            int ow = s->outlineSize();
+            if (ro->isInlineFlow()) {
+                returnSelf = true;
+                break;
+            }
+            if ( s->backgroundImage() || s->backgroundColor().isValid() || s->hasBorder() || ro->isReplaced() ) {
+                r += QRect(x -ow +ro->effectiveXPos(),y -ow + ro->effectiveYPos(), 
+                                  ro->effectiveWidth()+ow*2, ro->effectiveHeight()+ow*2);
+            } else {
                 r += ro->visibleFlowRegion(x+ro->xPos(), y+ro->yPos());
+            }
         }
     }
+    if (returnSelf) {
+        int ow = style()->outlineSize();
+        r+= QRect(x-xPos()-ow +effectiveXPos(),y-yPos()-ow + effectiveYPos(),
+                               effectiveWidth()+ow*2, effectiveHeight()+ow*2);
+    }
     return r;
 }
 
