@@ -85,6 +85,7 @@
 #include <kio/job.h>
 
 #include <stdlib.h>
+#include <limits.h>
 
 using namespace DOM;
 using namespace khtml;
@@ -106,18 +107,22 @@ bool DOMImplementationImpl::hasFeature ( const DOMString &feature, const DOMStri
     // ### update when we (fully) support the relevant features
     QString lower = feature.string().toLower();
     if ((lower == "html" || lower == "xml") &&
-        (version.isEmpty() || version == "1.0" || version == "2.0" || version == "null"))
+        (version.isEmpty() || version == "1.0" || version == "2.0"))
         return true;
 
     // ## Do we support Core Level 3 ?
     if ((lower == "core" ) &&
-        (version.isEmpty() || version == "2.0" || version == "null"))
+        (version.isEmpty() || version == "2.0"))
+        return true;
+
+    if ((lower == "traversal") &&
+        (version.isEmpty() || version == "2.0"))
         return true;
 
     if ((lower == "events" || lower == "uievents" ||
          lower == "mouseevents" || lower == "mutationevents" ||
          lower == "htmlevents" || lower == "textevents" ) &&
-        (version.isEmpty() || version == "2.0" || version == "3.0" || version == "null"))
+        (version.isEmpty() || version == "2.0" || version == "3.0"))
         return true;
     return false;
 }
@@ -383,6 +388,7 @@ DocumentImpl::DocumentImpl(DOMImplementationImpl *_implementation, KHTMLView *v)
     m_documentElement = 0;
     m_cssTarget = 0;
     m_dynamicDomRestyler = new khtml::DynamicDomRestyler();
+    m_stateRestorePos = 0;
 }
 
 void DocumentImpl::removedLastRef()
@@ -1070,22 +1076,50 @@ ElementImpl *DocumentImpl::createHTMLElement( const DOMString &name )
     return n;
 }
 
-QString DocumentImpl::nextState()
+void DocumentImpl::attemptRestoreState(NodeImpl* n)
 {
-   QString state;
-   if (!m_state.isEmpty())
-   {
-      state = m_state.first();
-      m_state.erase(m_state.begin());
-   }
-   return state;
+    if (!n->isElementNode())
+        return;
+
+    ElementImpl* el = static_cast<ElementImpl*>(n);
+
+    if (m_stateRestorePos >= m_state.size())
+        return;
+
+    // Grab the state and element info..
+    QString idStr = m_state[m_stateRestorePos];
+    QString nmStr = m_state[m_stateRestorePos + 1];
+    QString tpStr = m_state[m_stateRestorePos + 2];
+    QString stStr = m_state[m_stateRestorePos + 3];
+
+    // Make sure it matches!
+    if (idStr.toUInt() != el->id())
+        return;
+    if (nmStr != el->getAttribute(ATTR_NAME).string())
+        return;
+    if (tpStr != el->getAttribute(ATTR_TYPE).string())
+        return;
+
+    m_stateRestorePos += 4;
+    if (!stStr.isNull())
+        el->restoreState(stStr);
 }
 
 QStringList DocumentImpl::docState()
 {
     QStringList s;
-    for (QListIterator<NodeImpl*> it(m_maintainsState); it.hasNext();)
-        s.append(it.next()->state());
+    for (QListIterator<NodeImpl*> it(m_maintainsState); it.hasNext();) {
+        NodeImpl* n = it.next();
+        if (!n->isElementNode())
+            continue;
+
+        ElementImpl* el = static_cast<ElementImpl*>(n);
+        // Encode the element ID, as well as the name and type attributes
+        s.append(QString::number(el->id()));
+        s.append(el->getAttribute(ATTR_NAME).string());
+        s.append(el->getAttribute(ATTR_TYPE).string());
+        s.append(el->state());
+    }
 
     return s;
 }
@@ -1361,7 +1395,7 @@ void DocumentImpl::open( bool clearEventListeners )
     m_tokenizer->begin();
 }
 
-HTMLElementImpl* DocumentImpl::body()
+HTMLElementImpl* DocumentImpl::body() const
 {
     NodeImpl *de = documentElement();
     if (!de)
@@ -1454,13 +1488,13 @@ void DocumentImpl::determineParseMode( const QString &/*str*/ )
 
 NodeImpl *DocumentImpl::nextFocusNode(NodeImpl *fromNode)
 {
-    unsigned short fromTabIndex;
+    short fromTabIndex;
 
     if (!fromNode) {
 	// No starting node supplied; begin with the top of the document
 	NodeImpl *n;
 
-	int lowestTabIndex = 65535;
+	int lowestTabIndex = SHRT_MAX + 1;
 	for (n = this; n != 0; n = n->traverseNextNode()) {
 	    if (n->isTabFocusable()) {
 		if ((n->tabIndex() > 0) && (n->tabIndex() < lowestTabIndex))
@@ -1468,7 +1502,7 @@ NodeImpl *DocumentImpl::nextFocusNode(NodeImpl *fromNode)
 	    }
 	}
 
-	if (lowestTabIndex == 65535)
+	if (lowestTabIndex == SHRT_MAX + 1)
 	    lowestTabIndex = 0;
 
 	// Go to the first node in the document that has the desired tab index
@@ -1495,7 +1529,7 @@ NodeImpl *DocumentImpl::nextFocusNode(NodeImpl *fromNode)
 	// tab index. For nodes with the same tab index as fromNode, we are only interested in those that come after
 	// fromNode in document order.
 	// If we don't find a suitable tab index, the next focus node will be one with a tab index of 0.
-	unsigned short lowestSuitableTabIndex = 65535;
+	int lowestSuitableTabIndex = SHRT_MAX + 1;
 	NodeImpl *n;
 
 	bool reachedFromNode = false;
@@ -1515,7 +1549,7 @@ NodeImpl *DocumentImpl::nextFocusNode(NodeImpl *fromNode)
 		reachedFromNode = true;
 	}
 
-	if (lowestSuitableTabIndex == 65535) {
+	if (lowestSuitableTabIndex == SHRT_MAX + 1) {
 	    // No next node with a tab index -> just take first node with tab index of 0
 	    NodeImpl *n = this;
 	    while (n && !(n->isTabFocusable() && n->tabIndex() == 0))
@@ -1569,7 +1603,7 @@ NodeImpl *DocumentImpl::previousFocusNode(NodeImpl *fromNode)
 	return 0;
     }
     else {
-	unsigned short fromTabIndex = fromNode->tabIndex();
+	short fromTabIndex = fromNode->tabIndex();
 
 	if (fromTabIndex == 0) {
 	    // Find the previous selectable node before fromNode (in document order) that doesn't have a tab index
@@ -1602,7 +1636,7 @@ NodeImpl *DocumentImpl::previousFocusNode(NodeImpl *fromNode)
 	    // tab index. For nodes with the same tab index as fromNode, we are only interested in those before
 	    // fromNode.
 	    // If we don't find a suitable tab index, then there will be no previous focus node.
-	    unsigned short highestSuitableTabIndex = 0;
+	    short highestSuitableTabIndex = 0;
 	    NodeImpl *n;
 
 	    bool reachedFromNode = false;
@@ -1826,13 +1860,26 @@ bool DocumentImpl::childTypeAllowed( unsigned short type )
     return false;
 }
 
-NodeImpl *DocumentImpl::cloneNode ( bool /*deep*/ )
+NodeImpl *DocumentImpl::cloneNode ( bool deep )
 {
-    // Spec says cloning Document nodes is "implementation dependent"
-    // so we do not support it...
-    return 0;
-}
+#if 0
+    NodeImpl *dtn = m_doctype->cloneNode(deep);
+    DocumentTypeImpl *dt = static_cast<DocumentTypeImpl*>(dtn);
+#endif
 
+    int exceptioncode;
+    DocumentImpl *clone = m_implementation->createDocument("",
+							   "",
+							   0, exceptioncode);
+    assert( exceptioncode == 0 );
+
+    // ### attributes, styles, ...
+
+    if (deep)
+	cloneChildNodes(clone);
+
+    return clone;
+}
 
 typedef const char* (*NameLookupFunction)(unsigned short id);
 typedef int (*IdLookupFunction)(const char *tagStr, int len);
@@ -1918,7 +1965,7 @@ NodeImpl::Id DocumentImpl::getId( NodeImpl::IdType _type, DOMStringImpl* _nsURI,
         if (!readonly && id && _prefix && _prefix->l) {
             // we were called in registration mode... check if the alias exists
             const QString px = QString::fromRawData( _prefix->s, _prefix->l );
-            QString qn("aliases: " + (cs ? px : px.toUpper()) + ":" + name);	//krazy:exclude=doublequote_chars DOM demands chars
+            QString qn("aliases: " + (cs ? px : px.toUpper()) + ":" + name);
             if (!map->ids.contains( qn )) {
                 map->ids.insert( qn, (void*)id );
             }
@@ -2087,6 +2134,11 @@ void DocumentImpl::updateStyleSelector(bool shallow)
         renderer()->setNeedsLayoutAndMinMaxRecalc();
 }
 
+bool DocumentImpl::readyForLayout() const
+{
+    return renderer() && haveStylesheetsLoaded() && (!isHTMLDocument() || (body() && body()->renderer()));
+}
+
 void DocumentImpl::recalcStyleSelector()
 {
     if ( !m_render || !attached() ) return;
@@ -2264,17 +2316,28 @@ void DocumentImpl::setActiveNode(NodeImpl* newActiveNode)
     if ( oldActiveNode ) oldActiveNode->deref();
 }
 
+void DocumentImpl::quietResetFocus()
+{
+    assert(m_focusNode != this);
+    if (m_focusNode) {
+        if (m_focusNode->active())
+            setActiveNode(0);
+
+        m_focusNode->setFocus(false);
+        m_focusNode->deref();
+    }
+    m_focusNode = 0;
+
+    //We're blurring. Better clear the Qt focus/give it to the view...
+    if (view())
+        view()->setFocus();
+}
+
 void DocumentImpl::setFocusNode(NodeImpl *newFocusNode)
 {
     // don't process focus changes while detaching
     if( !m_render ) return;
 
-    // We do want to blur if a widget is being detached,
-    // but we don't want to emit events since that
-    // triggers updateLayout() and may recurse detach()
-    bool widgetDetach = m_focusNode && m_focusNode != this &&
-              m_focusNode->renderer() && !m_focusNode->renderer()->parent();
-
     // Make sure newFocusNode is actually in this document
     if (newFocusNode && (newFocusNode->getDocument() != this))
         return;
@@ -2290,10 +2353,9 @@ void DocumentImpl::setFocusNode(NodeImpl *newFocusNode)
 
             oldFocusNode->setFocus(false);
 
-            if (!widgetDetach) {
-                oldFocusNode->dispatchHTMLEvent(EventImpl::BLUR_EVENT,false,false);
-                oldFocusNode->dispatchUIEvent(EventImpl::DOMFOCUSOUT_EVENT);
-            }
+            oldFocusNode->dispatchHTMLEvent(EventImpl::BLUR_EVENT,false,false);
+            oldFocusNode->dispatchUIEvent(EventImpl::DOMFOCUSOUT_EVENT);
+
             if ((oldFocusNode == this) && oldFocusNode->hasOneRef()) {
                 oldFocusNode->deref(); // deletes this
                 return;
@@ -2329,8 +2391,7 @@ void DocumentImpl::setFocusNode(NodeImpl *newFocusNode)
                 view()->setFocus();
         }
 
-        if (!widgetDetach)
-            updateRendering();
+        updateRendering();
     }
 }
 
@@ -2500,11 +2561,11 @@ void DocumentImpl::loadXML(const DOMString &source)
     dispatchHTMLEvent(EventImpl::LOAD_EVENT,false,false);
 }
 
-void DocumentImpl::setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheet, const DOM::DOMString &/*charset*/)
+void DocumentImpl::setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheet, const DOM::DOMString &/*charset*/, const DOM::DOMString &mimetype)
 {
     if (!m_hadLoadError) {
 	m_url = url.string();
-	loadXML(sheet);
+	loadXML(khtml::isAcceptableCSSMimetype(mimetype) ? sheet : "");
     }
 
     m_docLoading = false;
@@ -2659,10 +2720,8 @@ ElementImpl *DocumentImpl::ownerElement() const
     ChildFrame *childFrame = childPart->d->m_frame;
     if (!childFrame)
         return 0;
-    RenderPart *renderPart = childFrame->m_frame;
-    if (!renderPart)
-        return 0;
-    return static_cast<ElementImpl *>(renderPart->element());
+    ElementImpl *el = childFrame->m_partContainerElement;
+    return el;
 }
 
 DOMString DocumentImpl::domain() const
@@ -2711,6 +2770,7 @@ DOMString DocumentImpl::toString() const
 void DOM::DocumentImpl::setRestoreState( const QStringList &s)
 {
     m_state = s;
+    m_stateRestorePos = 0;
 }
 
 KHTMLView* DOM::DocumentImpl::view() const
@@ -2877,10 +2937,10 @@ DOMString DocumentTypeImpl::toString() const
     if (!m_subset.isEmpty()) {
 	result += " [";
 	result += m_subset;
-	result += "]";	//krazy:exclude=doublequote_chars DOM demands chars
+	result += "]";
     }
 
-    result += ">";	//krazy:exclude=doublequote_chars DOM demands chars
+    result += ">";
 
     return result;
 }
@@ -2903,9 +2963,12 @@ bool DocumentTypeImpl::childTypeAllowed( unsigned short /*type*/ )
 
 NodeImpl *DocumentTypeImpl::cloneNode ( bool /*deep*/ )
 {
-    // Spec says cloning Document nodes is "implementation dependent"
-    // so we do not support it...
-    return 0;
+    DocumentTypeImpl *clone = new DocumentTypeImpl(implementation(),
+						   0,
+						   name(), publicId(),
+						   systemId());
+    // ### copy entities etc.
+    return clone;
 }
 
 NamedNodeMapImpl * DocumentTypeImpl::entities() const
