@@ -237,7 +237,7 @@ public:
      * This method returns true if all top-level stylesheets have loaded (including
      * any \@imports that they may be loading).
      */
-    bool haveStylesheetsLoaded() { return m_pendingStylesheets <= 0 || m_ignorePendingStylesheets; }
+    bool haveStylesheetsLoaded() const { return m_pendingStylesheets <= 0 || m_ignorePendingStylesheets; }
 
     /**
      * Increments the number of pending sheets.  The \<link\> elements
@@ -260,15 +260,18 @@ public:
      * constructed from these which is used to create the a new style selector which collates all of the stylesheets
      * found and is used to calculate the derived styles for all rendering objects.
      *
-     * @param shallow If the stylesheet list for the document is unchanged, with only added or removed rules 
+     * @param shallow If the stylesheet list for the document is unchanged, with only added or removed rules
      * in existing sheets, then set this argument to true for efficiency.
      */
     void updateStyleSelector(bool shallow=false);
 
+    bool readyForLayout() const;
     void recalcStyleSelector();
     void rebuildStyleSelector();
 
-    QString nextState();
+    // Tries to restore the elements value from the doc state,
+    // if it seems like the same thing
+    void attemptRestoreState(NodeImpl* e);
 
     // Query all registered elements for their state
     QStringList docState();
@@ -401,6 +404,7 @@ public:
     NodeImpl* hoverNode() const { return m_hoverNode; }
     void setHoverNode(NodeImpl *newHoverNode);
     NodeImpl *focusNode() const { return m_focusNode; }
+    void quietResetFocus(); // Removes focus from active node without attempting to emit any events
     void setFocusNode(NodeImpl *newFocusNode);
     NodeImpl* activeNode() const { return m_activeNode; }
     void setActiveNode(NodeImpl *newActiveNode);
@@ -440,7 +444,7 @@ public:
     void load(const DOMString &uri);
     void loadXML(const DOMString &source);
     // from cachedObjectClient
-    void setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheet, const DOM::DOMString &charset);
+    void setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheet, const DOM::DOMString &charset, const DOM::DOMString &mimetype);
     void error(int err, const QString &text);
 
     typedef QMap<QString, ProcessingInstructionImpl*> LocalStyleRefs;
@@ -513,7 +517,7 @@ public:
 
     bool isURLAllowed(const QString& url) const;
 
-    HTMLElementImpl* body();
+    HTMLElementImpl* body() const;
 
     DOMString toString() const;
 
@@ -550,6 +554,7 @@ protected:
     khtml::CSSStyleSelector *m_styleSelector;
     KHTMLView *m_view;
     QStringList m_state;
+    int         m_stateRestorePos;
 
     khtml::DocLoader *m_docLoader;
     khtml::Tokenizer *m_tokenizer;
@@ -647,7 +652,7 @@ protected:
     DOMString m_title;
     DOMString m_preferredStylesheetSet;
     khtml::CachedCSSStyleSheet *m_loadingXMLDoc;
-    
+
     mutable ElementImpl* m_documentElement;
 
     //int m_decoderMibEnum;
