@@ -143,7 +143,7 @@ DOMString AttrImpl::name() const
         n = n.lower();
 
     if (m_prefix && m_prefix->l)
-        return DOMString(m_prefix) + ":" + n;	//krazy:exclude=doublequote_chars DOM demands chars
+        return DOMString(m_prefix) + ":" + n;
 
     return n;
 }
@@ -462,13 +462,16 @@ void ElementImpl::finishCloneNode( ElementImpl* clone, bool deep )
             if (!clone->m_hasCombinedStyle)
                 clone->createNonCSSDecl();
             if (m_style.combinedDecls->inlineDecls)
-                *clone->m_style.combinedDecls->inlineDecls = *m_style.combinedDecls->inlineDecls;
+                *(clone->getInlineStyleDecls()) = *m_style.combinedDecls->inlineDecls;
             *clone->m_style.combinedDecls->nonCSSDecls = *m_style.combinedDecls->nonCSSDecls;
         } else {
             *(clone->getInlineStyleDecls()) = *m_style.inlineDecls;
         }
     }
 
+    // ### fold above style cloning into this function?
+    clone->copyNonAttributeProperties(this);
+
     if (deep)
         cloneChildNodes(clone);
 }
@@ -518,6 +521,9 @@ void ElementImpl::removeAttribute( const DOMString &name, int& exceptioncode )
     if (!id) return;
 
     attributes(false)->removeNamedItem(id, false, name.implementation(), exceptioncode);
+    // it's allowed to remove attributes that don't exist.
+    if ( exceptioncode == DOMException::NOT_FOUND_ERR )
+        exceptioncode = 0;
 }
 
 AttrImpl* ElementImpl::getAttributeNode( const DOMString &name )
@@ -985,7 +991,7 @@ DOMString ElementImpl::openTagStartToString(bool expandurls) const
     if (attrMap) {
 	unsigned long numAttrs = attrMap->length();
 	for (unsigned long i = 0; i < numAttrs; i++) {
-	    result += " ";	//krazy:exclude=doublequote_chars DOM demands chars
+	    result += " ";
 
 	    AttributeImpl *attribute = attrMap->attrAt(i);
 	    AttrImpl *attr = attribute->attr();
@@ -1025,7 +1031,7 @@ DOMString ElementImpl::selectionToString(NodeImpl *selectionStart, NodeImpl *sel
     DOMString result = openTagStartToString();
 
     if (hasChildNodes()) {
-	result += ">";	//krazy:exclude=doublequote_chars DOM demands chars
+	result += ">";
 
 	for (NodeImpl *child = firstChild(); child != NULL; child = child->nextSibling()) {
 	    result += child->selectionToString(selectionStart, selectionEnd, startOffset, endOffset, found); // this might set found to true
@@ -1036,7 +1042,7 @@ DOMString ElementImpl::selectionToString(NodeImpl *selectionStart, NodeImpl *sel
 
 	result += "</";
 	result += tagName();
-	result += ">";	//krazy:exclude=doublequote_chars DOM demands chars
+	result += ">";
     } else {
 	result += " />";
     }
@@ -1049,7 +1055,7 @@ DOMString ElementImpl::toString() const
     QString result = openTagStartToString().string(); //Accumulate in QString, since DOMString can't append well.
 
     if (hasChildNodes()) {
-	result += ">";	//krazy:exclude=doublequote_chars DOM demands chars
+	result += ">";
 
 	for (NodeImpl *child = firstChild(); child != NULL; child = child->nextSibling()) {
 	    DOMString kid = child->toString();
@@ -1058,7 +1064,7 @@ DOMString ElementImpl::toString() const
 
 	result += "</";
 	result += tagName().string();
-	result += ">";	//krazy:exclude=doublequote_chars DOM demands chars
+	result += ">";
     } else if (result.length() == 1) {
 	// ensure we do not get results like < /> can happen when serialize document
         result = "";
@@ -1138,7 +1144,7 @@ DOMString XMLElementImpl::tagName() const
         tn = tn.upper();
 
     if (m_prefix)
-        return DOMString(m_prefix) + ":" + tn;	//krazy:exclude=doublequote_chars DOM demands chars
+        return DOMString(m_prefix) + ":" + tn;
 
     return tn;
 }
