@@ -209,6 +209,8 @@ public:
     // State of the element.
     virtual QString state() { return QString(); }
 
+    virtual void copyNonAttributeProperties(const ElementImpl* /*source*/) {}
+
     virtual void attach();
     virtual void close();
     virtual void detach();
@@ -373,18 +375,29 @@ inline bool checkQualifiedName(const DOMString &qualifiedName, const DOMString &
     int colonpos = -1;
     uint i;
     DOMStringImpl *qname = qualifiedName.implementation();
-    for (i = 0 ; i < qname->l ; i++) {
+    for (i = 0 ; i < (qname ? qname->l : 0); i++) {
         if ((*qname)[i] == QLatin1Char(':')) {
             colonpos = i;
             break;
         }
     }
 
+    bool hasXMLPrefix = colonpos == 3 && (*qname)[0] == QLatin1Char('x') &&
+      (*qname)[1] == QLatin1Char('m') && (*qname)[2] == QLatin1Char('l');
+    bool hasXMLNSPrefix = colonpos == 5 && (*qname)[0] == QLatin1Char('x') &&
+      (*qname)[1] == QLatin1Char('m') && (*qname)[2] == QLatin1Char('l') &&
+      (*qname)[3] == QLatin1Char('n') && (*qname)[4] == QLatin1Char('s');
+
     if ((!qualifiedName.isNull() && Element::khtmlMalformedQualifiedName(qualifiedName)) ||
         (colonpos >= 0 && namespaceURI.isNull()) ||
+        colonpos == 0 || // prefix has to consist of at least a letter
         (qualifiedName.isNull() && !namespaceURI.isNull()) ||
-        (colonpos == 3 && qualifiedName[0] == QLatin1Char('x') && qualifiedName[1] == QLatin1Char('m') && qualifiedName[2] == QLatin1Char('l') &&
-         namespaceURI != "http://www.w3.org/XML/1998/namespace")) {
+        (hasXMLPrefix &&
+         namespaceURI != "http://www.w3.org/XML/1998/namespace") ||
+        (hasXMLNSPrefix &&
+         namespaceURI != "http://www.w3.org/2000/xmlns/") ||
+        (namespaceURI == "http://www.w3.org/2000/xmlns/" &&
+         !hasXMLNSPrefix && qualifiedName != "xmlns")) {
         if (pExceptioncode)
             *pExceptioncode = DOMException::NAMESPACE_ERR;
         return false;
