@@ -944,9 +944,9 @@ KLauncher::createArgs( KLaunchRequest *request, const KService::Ptr service ,
   for(QStringList::ConstIterator it = params.begin();
       it != params.end(); ++it)
   {
-     request->arg_list.append((*it).toLocal8Bit());
+     request->arg_list.append(*it);
   }
-  request->cwd = QFile::encodeName(service->path());
+  request->cwd = service->path();
 }
 
 ///// IO-Slave functions
@@ -1030,7 +1030,7 @@ KLauncher::requestSlave(const QString &protocol,
     arg_list << protocol;
     arg_list << mConnectionServer.address();
     arg_list << app_socket;
-    name = "kioslave";
+    name = KStandardDirs::findExe("kioslave");
 #else
     QString arg1 = protocol;
     QString arg2 = mConnectionServer.address();
