@@ -101,42 +101,29 @@ JobUiDelegate *Job::ui() const
     return static_cast<JobUiDelegate*>( uiDelegate() );
 }
 
-void Job::addSubjob(Job *job, bool inheritMetaData)
+bool Job::addSubjob(KJob *jobBase)
 {
-    Q_D(Job);
-    //kDebug(7007) << "addSubjob(" << job << ") this = " << this;
+    //kDebug(7007) << "addSubjob(" << jobBase << ") this=" << this;
 
-    KCompositeJob::addSubjob( job );
+    bool ok = KCompositeJob::addSubjob( jobBase );
+    KIO::Job *job = dynamic_cast<KIO::Job*>( jobBase );
+    if (ok && job) {
+        // Forward information from that subjob.
+        connect( job, SIGNAL(speed( KJob*, unsigned long )),
+                 SLOT(slotSpeed(KJob*, unsigned long)) );
 
-    // Forward information from that subjob.
-    connect( job, SIGNAL(speed( KJob*, unsigned long )),
-             SLOT(slotSpeed(KJob*, unsigned long)) );
-
-    if (inheritMetaData)
-        job->mergeMetaData(d->m_outgoingMetaData);
-
-    if (ui() && job->ui()) {
-        job->ui()->setWindow( ui()->window() );
-        job->ui()->updateUserTimestamp( ui()->userTimestamp() );
+        if (ui() && job->ui()) {
+            job->ui()->setWindow( ui()->window() );
+            job->ui()->updateUserTimestamp( ui()->userTimestamp() );
+        }
     }
+    return ok;
 }
 
-bool Job::removeSubjob( KJob *jobBase, bool mergeMetaData )
+bool Job::removeSubjob( KJob *jobBase )
 {
-    Q_D(Job);
-    KIO::Job *job = dynamic_cast<KIO::Job*>( jobBase );
-
-    if ( job == 0 )
-    {
-        return false;
-    }
-
-    //kDebug(7007) << "removeSubjob(" << job << ") this = " << this << "  subjobs = " << subjobs().count();
-    // Merge metadata from subjob
-    if ( mergeMetaData )
-        d->m_incomingMetaData += job->metaData();
-
-    return KCompositeJob::removeSubjob( job );
+    //kDebug(7007) << "removeSubjob(" << jobBase << ") this=" << this << "subjobs=" << subjobs().count();
+    return KCompositeJob::removeSubjob( jobBase );
 }
 
 void JobPrivate::emitMoving(KIO::Job * job, const KUrl &src, const KUrl &dest)
@@ -341,7 +328,7 @@ bool SimpleJob::doKill()
     Q_D(SimpleJob);
     Scheduler::cancelJob( this ); // deletes the slave if not 0
     d->m_slave = 0; // -> set to 0
-    return true;
+    return Job::doKill();
 }
 
 bool SimpleJob::doSuspend()
@@ -409,7 +396,7 @@ void SimpleJobPrivate::start(Slave *slave)
                 SLOT( slotWarning( const QString & ) ) );
 
     q->connect( slave, SIGNAL( infoMessage( const QString & ) ),
-                SLOT( slotInfoMessage( const QString & ) ) );
+                SLOT( _k_slotSlaveInfoMessage( const QString & ) ) );
 
     q->connect( slave, SIGNAL( connected() ),
                 SLOT( slotConnected() ) );
@@ -521,9 +508,9 @@ void SimpleJob::slotWarning( const QString & errorText )
     emit warning( this, errorText );
 }
 
-void SimpleJob::slotInfoMessage( const QString & msg )
+void SimpleJobPrivate::_k_slotSlaveInfoMessage( const QString & msg )
 {
-    emit infoMessage( this, msg );
+    emit q_func()->infoMessage( q_func(), msg );
 }
 
 void SimpleJobPrivate::slotConnected()
@@ -545,9 +532,6 @@ void SimpleJobPrivate::slotProcessedSize( KIO::filesize_t size )
     Q_Q(SimpleJob);
     //kDebug(7007) << "SimpleJob::slotProcessedSize " << KIO::number(size);
     q->setProcessedAmount(KJob::Bytes, size);
-    if ( size > q->totalAmount(KJob::Bytes) ) {
-        q->setTotalAmount(KJob::Bytes, size); // safety
-    }
 }
 
 void SimpleJobPrivate::slotSpeed( unsigned long speed )
@@ -1050,9 +1034,6 @@ void TransferJob::sendAsyncData(const QByteArray &dataForSlave)
        {
            KIO::filesize_t size = processedAmount(KJob::Bytes)+dataForSlave.size();
            setProcessedAmount(KJob::Bytes, size);
-           if ( size > totalAmount(KJob::Bytes) ) {
-               setTotalAmount(KJob::Bytes, size); // safety
-           }
        }
     }
 
@@ -1737,18 +1718,18 @@ public:
 FileCopyJob::FileCopyJob(FileCopyJobPrivate &dd)
     : Job(dd)
 {
-    Q_D(FileCopyJob);
-    if (!d->m_move)
-        JobPrivate::emitCopying( this, d->m_src, d->m_dest );
-    else
-        JobPrivate::emitMoving( this, d->m_src, d->m_dest );
-
     //kDebug(7007) << "FileCopyJob::FileCopyJob()";
     QTimer::singleShot(0, this, SLOT(slotStart()));
 }
 
 void FileCopyJobPrivate::slotStart()
 {
+    Q_Q(FileCopyJob);
+    if (!m_move)
+        JobPrivate::emitCopying( q, m_src, m_dest );
+    else
+        JobPrivate::emitMoving( q, m_src, m_dest );
+
    if ( m_move )
    {
       // The if() below must be the same as the one in startBestCopyMethod
@@ -1912,9 +1893,6 @@ void FileCopyJobPrivate::slotProcessedSize( KJob *, qulonglong size )
 {
     Q_Q(FileCopyJob);
     q->setProcessedAmount(KJob::Bytes, size);
-    if ( size > q->totalAmount(KJob::Bytes) ) {
-        slotTotalSize( q, size ); // safety
-    }
 }
 
 void FileCopyJobPrivate::slotTotalSize( KJob*, qulonglong size )
@@ -1926,8 +1904,13 @@ void FileCopyJobPrivate::slotTotalSize( KJob*, qulonglong size )
     }
 }
 
-void FileCopyJobPrivate::slotPercent( KJob*, unsigned long pct ) {
-  Q_Q(FileCopyJob); if ( pct > q->percent() ) { q->setPercent( pct ); } }
+void FileCopyJobPrivate::slotPercent( KJob*, unsigned long pct )
+{
+  Q_Q(FileCopyJob);
+  if ( pct > q->percent() ) {
+      q->setPercent( pct );
+  }
+}
 
 void FileCopyJobPrivate::startDataPump()
 {
