@@ -37,7 +37,7 @@ namespace KIO {
 
     /**
      * Flags for the job properties.
-     * Not all flags are supported in all cases. Please see documentation of 
+     * Not all flags are supported in all cases. Please see documentation of
      * the calling function!
      */
     enum JobFlag {
@@ -287,10 +287,8 @@ namespace KIO {
          * the finish signal is emitted by the slave.
          *
 	 * @param job the subjob to add
-         * @param inheritMetaData if true, the subjob will
-         * inherit the meta data from this job.
          */
-        virtual void addSubjob( Job *job, bool inheritMetaData=true );
+        virtual bool addSubjob( KJob *job );
 
         /**
          * Mark a sub job as being done.
@@ -298,10 +296,8 @@ namespace KIO {
          * KDE4 change: this doesn't terminate the parent job anymore, call emitResult to do that.
 	 *
 	 * @param job the subjob to remove
-         * @param mergeMetaData if set, the metadata received by the subjob is
-         *                      merged into this job.
          */
-        bool removeSubjob( KJob *job, bool mergeMetaData = false );
+        virtual bool removeSubjob( KJob *job );
 
     private:
         /**
@@ -385,13 +381,6 @@ namespace KIO {
          */
         virtual void slotWarning( const QString & );
 
-        /**
-         * Called on a slave's info message.
-	 * @param s the info message
-	 * @see infoMessage()
-         */
-        virtual void slotInfoMessage( const QString &s );
-
         /**
          * MetaData from the slave is received.
 	 * @param _metaData the meta data
@@ -419,6 +408,7 @@ namespace KIO {
         Q_PRIVATE_SLOT(d_func(), void slotProcessedSize( KIO::filesize_t data_size ))
         Q_PRIVATE_SLOT(d_func(), void slotSpeed( unsigned long speed ))
         Q_PRIVATE_SLOT(d_func(), void slotTotalSize( KIO::filesize_t data_size ))
+        Q_PRIVATE_SLOT(d_func(), void _k_slotSlaveInfoMessage(const QString&))
 
         Q_DECLARE_PRIVATE(SimpleJob)
     };
