@@ -31,6 +31,7 @@
 #include <QFileInfo>
 #include <QDir>
 #include <sys/types.h>
+#include <dirent.h>
 
 class KDirModelNode;
 class KDirModelDirNode;
@@ -163,15 +164,15 @@ QPair<int /*row*/, KDirModelNode*> KDirModelPrivate::nodeForUrl(const KUrl& _url
     if (url == nodeUrl)
         return qMakePair(0, static_cast<KDirModelNode *>(m_rootNode));
 
-    const QString urlStr = url.url();
+    const QString pathStr = url.path();
     KDirModelDirNode* dirNode = m_rootNode;
 
-    if ( !urlStr.startsWith(nodeUrl.url()) ) {
+    if ( !pathStr.startsWith(nodeUrl.path()) ) {
         return qMakePair(0, static_cast<KDirModelNode*>(0));
     }
 
     for (;;) {
-        Q_ASSERT( urlStr.startsWith(nodeUrl.url()) );
+        Q_ASSERT( pathStr.startsWith(nodeUrl.path()) );
         bool foundChild = false;
         QList<KDirModelNode *>::const_iterator it = dirNode->m_childNodes.begin();
         const QList<KDirModelNode *>::const_iterator end = dirNode->m_childNodes.end();
@@ -182,7 +183,8 @@ QPair<int /*row*/, KDirModelNode*> KDirModelPrivate::nodeForUrl(const KUrl& _url
                 //kDebug(7008) << "Found! " << u;
                 return qMakePair(row, *it);
             }
-            if ( urlStr.startsWith(u.url()+'/') ) {
+            // This used to be urlStr.startsWith(u.url()+'/'), but KUrl::url() is a slow operation.
+            if ( pathStr.startsWith(u.path()+'/') ) {
                 //kDebug(7008) << "going into " << node->item().url();
                 Q_ASSERT( isDir(*it) );
                 dirNode = static_cast<KDirModelDirNode *>( *it );
@@ -480,8 +482,26 @@ QVariant KDirModel::data( const QModelIndex & index, int role ) const
                 if (count == ChildCountUnknown && item.isReadable()) {
                     const QString path = item.localPath();
                     if (!path.isEmpty()) {
+#if 0 // slow
                         QDir dir(path);
                         count = dir.entryList(QDir::AllEntries|QDir::NoDotAndDotDot|QDir::System).count();
+#else
+                        DIR* dir = ::opendir(QFile::encodeName(path));
+                        if (dir) {
+                            count = 0;
+                            struct dirent *dirEntry = 0;
+                            while ((dirEntry = ::readdir(dir))) {
+                                if (dirEntry->d_name[0] == '.') {
+                                    if (dirEntry->d_name[1] == '\0') // skip "."
+                                        continue;
+                                    if (dirEntry->d_name[1] == '.' && dirEntry->d_name[2] == '\0') // skip ".."
+                                        continue;
+                                }
+                                ++count;
+                            }
+                            ::closedir(dir);
+                        }
+#endif
                         //kDebug(7008) << "child count for " << path << ":" << count;
                         dirNode->setChildCount(count);
                     }
