@@ -39,8 +39,6 @@
 
 #include <kdebug.h>
 #include <kfilemetainfo.h>
-#include <ksambashare.h>
-#include <knfsshare.h>
 #include <kglobal.h>
 #include <kglobalsettings.h>
 #include <kiconloader.h>
@@ -51,6 +49,10 @@
 #include <kdesktopfile.h>
 #include <kmountpoint.h>
 #include <kconfiggroup.h>
+#ifndef Q_OS_WIN
+#include <knfsshare.h>
+#include <ksambashare.h>
+#endif
 
 class KFileItemPrivate : public QSharedData
 {
@@ -754,6 +756,7 @@ QStringList KFileItem::overlays() const
         names.append("hidden");
     }
 
+#ifndef Q_OS_WIN
     if( S_ISDIR( d->m_fileMode ) && d->m_bIsLocalUrl)
     {
         if (KSambaShare::instance()->isDirectoryShared( d->m_url.path() ) ||
@@ -763,6 +766,7 @@ QStringList KFileItem::overlays() const
             names.append("share");
         }
     }
+#endif  // Q_OS_WIN
 
     if ( d->m_pMimeType && d->m_pMimeType->is("application/x-gzip") && d->m_url.fileName().endsWith( QLatin1String( ".gz" ) ) ) {
         names.append("zio");
