@@ -173,12 +173,13 @@ void PreviewJobPrivate::startPreview()
     const KService::List plugins = KServiceTypeTrader::self()->query("ThumbCreator");
     QMap<QString, KService::Ptr> mimeMap;
 
-    for (KService::List::ConstIterator it = plugins.begin(); it != plugins.end(); ++it)
+    for (KService::List::ConstIterator it = plugins.begin(); it != plugins.end(); ++it) {
         if (!enabledPlugins || enabledPlugins->contains((*it)->desktopEntryName()))
-    {
-        const QStringList mimeTypes = (*it)->serviceTypes();
-        for (QStringList::ConstIterator mt = mimeTypes.begin(); mt != mimeTypes.end(); ++mt)
-            mimeMap.insert(*mt, *it);
+        {
+            const QStringList mimeTypes = (*it)->serviceTypes();
+            for (QStringList::ConstIterator mt = mimeTypes.begin(); mt != mimeTypes.end(); ++mt)
+                mimeMap.insert(*mt, *it);
+        }
     }
 
     // Look for images and store the items in our todo list :)
@@ -256,7 +257,7 @@ void PreviewJobPrivate::startPreview()
 
   // Read configuration value for the maximum allowed size
     KConfigGroup cg( KGlobal::config(), "PreviewSettings" );
-    maximumSize = cg.readEntry( "MaximumSize", 1024*1024LL /* 1MB */ );
+    maximumSize = cg.readEntry( "MaximumSize", 5*1024*1024LL /* 5MB */ );
 
     if (bNeedCache)
     {
