@@ -41,23 +41,22 @@
 #include <kstandarddirs.h>
 #include <kwalletentry.h>
 #include <kwindowsystem.h>
+#include <kpluginfactory.h>
+#include <kpluginloader.h>
 
 #include <QtCore/QDir>
-#include <QtGui/QLabel>
-#include <QtGui/QLayout>
-#include <QtGui/QPushButton>
+#include <QtGui/QTextDocument> // Qt::escape
 #include <QtCore/QRegExp>
-#include <QtGui/QTextDocument>
+#include <QtCore/QTimer>
 
 #include <assert.h>
 
 #include "kwalletdadaptor.h"
 
-extern "C" {
-   KDE_EXPORT KDEDModule *create_kwalletd() {
-	   return new KWalletD();
-   }
-}
+K_PLUGIN_FACTORY(KWalletDFactory,
+                 registerPlugin<KWalletD>();
+    )
+K_EXPORT_PLUGIN(KWalletDFactory("kwalletd"))
 
 class KWalletTransaction {
 	public:
@@ -77,8 +76,8 @@ class KWalletTransaction {
 		bool modal;
 };
 
-KWalletD::KWalletD()
-: KDEDModule(), _failed(0) {
+KWalletD::KWalletD(QObject* parent, const QList<QVariant>&)
+	: KDEDModule(parent), _failed(0) {
 	srand(time(0));
 	_showingFailureNotify = false;
 	_timeouts = new KTimeout();
@@ -231,7 +230,7 @@ int KWalletD::openPath(const QString& path, qlonglong wId, const QString& appid)
 	}
 
 	// FIXME: setup transaction
-	int rc = internalOpen(appid, path, true, wId, false);
+	int rc = internalOpen(appid, path, true, (WId)wId, false);
 	return rc;
 }
 
@@ -306,7 +305,8 @@ int KWalletD::doTransactionOpen(const QString& appid, const QString& wallet, qlo
 	if (_firstUse && !wallets().contains(KWallet::Wallet::LocalWallet())) {
 		// First use wizard
 		KWalletWizard *wiz = new KWalletWizard(0);
-		setupDialog( wiz, wId, appid, modal );
+		wiz->setWindowTitle(i18n("KDE Wallet Service"));
+		setupDialog( wiz, (WId)wId, appid, modal );
 		int rc = wiz->exec();
 		if (rc == QDialog::Accepted) {
 			bool useWallet = wiz->field("useWallet").toBool();
@@ -505,7 +505,7 @@ bool KWalletD::isAuthorizedApp(const QString& appid, const QString& wallet, WId
 
 	if (!implicitAllow(wallet, thisApp)) {
 		KConfigGroup cfg = KSharedConfig::openConfig("kwalletrc")->group("Auto Allow");
-		if (!cfg.entryIsImmutable(wallet)) {
+		if (!cfg.isEntryImmutable(wallet)) {
 		    KBetterThanKDialog *dialog = new KBetterThanKDialog;
 		    if (appid.isEmpty()) {
 			dialog->setLabel(i18n("<qt>KDE has requested access to the open wallet '<b>%1</b>'.</qt>", Qt::escape(wallet)));
@@ -523,7 +523,7 @@ bool KWalletD::isAuthorizedApp(const QString& appid, const QString& wallet, WId
 			KConfigGroup cfg = KSharedConfig::openConfig("kwalletrc")->group("Auto Allow");
 			QStringList apps = cfg.readEntry(wallet, QStringList());
 			if (!apps.contains(thisApp)) {
-				if (cfg.entryIsImmutable(wallet)) {
+				if (cfg.isEntryImmutable(wallet)) {
 					return false;
 				}
 				apps += thisApp;
@@ -592,7 +592,7 @@ void KWalletD::doTransactionChangePassword(const QString& appid, const QString&
 	if (!w) {
 		handle = doTransactionOpen(appid, wallet, wId, false);
 		if (-1 == handle) {
-			KMessageBox::sorryWId(wId, i18n("Unable to open wallet. The wallet must be opened in order to change the password."), i18n("KDE Wallet Service"));
+			KMessageBox::sorryWId((WId)wId, i18n("Unable to open wallet. The wallet must be opened in order to change the password."), i18n("KDE Wallet Service"));
 			return;
 		}
 
@@ -606,19 +606,19 @@ void KWalletD::doTransactionChangePassword(const QString& appid, const QString&
 	kpd->setPrompt(i18n("<qt>Please choose a new password for the wallet '<b>%1</b>'.</qt>", Qt::escape(wallet)));
 	kpd->setCaption(i18n("KDE Wallet Service"));
 	kpd->setAllowEmptyPasswords(true);
-	setupDialog( kpd, wId, appid, false );
+	setupDialog( kpd, (WId)wId, appid, false );
 	if (kpd->exec() == KDialog::Accepted) {
 		QString p = kpd->password();
 		if (!p.isNull()) {
 			_passwords[wallet] = p.toUtf8();
 			int rc = w->close(p.toUtf8());
 			if (rc < 0) {
-				KMessageBox::sorryWId(wId, i18n("Error re-encrypting the wallet. Password was not changed."), i18n("KDE Wallet Service"));
+				KMessageBox::sorryWId((WId)wId, i18n("Error re-encrypting the wallet. Password was not changed."), i18n("KDE Wallet Service"));
 				reclose = true;
 			} else {
 				rc = w->open(p.toUtf8());
 				if (rc < 0) {
-					KMessageBox::sorryWId(wId, i18n("Error reopening the wallet. Data may be lost."), i18n("KDE Wallet Service"));
+					KMessageBox::sorryWId((WId)wId, i18n("Error reopening the wallet. Data may be lost."), i18n("KDE Wallet Service"));
 					reclose = true;
 				}
 			}
