@@ -71,9 +71,10 @@
 #include <QtCore/QVarLengthArray>
 #include <QtCore/QCoreApplication>
 #include <QtCore/QRegExp>
+#include <QtCore/QFile>
 #ifdef Q_WS_WIN
-#include <QDir>
-#include <QFileInfo>
+#include <QtCore/QDir>
+#include <QtCore/QFileInfo>
 #endif
 
 #include <kdebug.h>
@@ -83,7 +84,6 @@
 #include <kconfiggroup.h>
 #include <ktemporaryfile.h>
 #include <klocale.h>
-#include <QtCore/QFile>
 #include <limits.h>
 #include <kshell.h>
 #include <kmountpoint.h>
@@ -759,7 +759,13 @@ void FileProtocol::put( const KUrl& url, int _mode, KIO::JobFlags _flags )
         if( (_flags & KIO::Overwrite) && S_ISLNK( buff_orig.st_mode ) )
           remove( _dest_orig.data() );
 
+#ifdef Q_OS_WIN
+        if ( MoveFileExA( _dest.data(),
+                          _dest_orig.data(),
+                          MOVEFILE_REPLACE_EXISTING|MOVEFILE_COPY_ALLOWED ) == 0 )
+#else
         if ( ::rename( _dest.data(), _dest_orig.data() ) )
+#endif
         {
             kWarning(7101) << " Couldn't rename " << _dest << " to " << _dest_orig;
             error( KIO::ERR_CANNOT_RENAME_PARTIAL, dest_orig );
@@ -799,6 +805,7 @@ void FileProtocol::put( const KUrl& url, int _mode, KIO::JobFlags _flags )
     finished();
 }
 
+#ifndef Q_OS_WIN
 
 void FileProtocol::copy( const KUrl &src, const KUrl &dest,
                          int _mode, JobFlags _flags )
@@ -1169,6 +1176,8 @@ void FileProtocol::del( const KUrl& url, bool isfile)
     finished();
 }
 
+#endif  // Q_OS_WIN
+
 QString FileProtocol::getUserName( uid_t uid ) const
 {
     if ( !mUsercache.contains( uid ) ) {
@@ -1318,6 +1327,7 @@ void FileProtocol::stat( const KUrl & url )
     finished();
 }
 
+#ifndef Q_OS_WIN
 void FileProtocol::listDir( const KUrl& url)
 {
     kDebug(7101) << "========= LIST " << url.url() << " =========";
@@ -1330,38 +1340,6 @@ void FileProtocol::listDir( const KUrl& url)
 	return;
     }
     QByteArray _path( QFile::encodeName(url.toLocalFile()) );
-#ifdef Q_WS_WIN
-    QDir dir( _path );
-    dir.setFilter(QDir::AllEntries);
-    dir.setSorting(QDir::Size | QDir::Reversed);
-
-    if ( !dir.exists() ) {
-        kDebug(7101) << "========= ERR_DOES_NOT_EXIST  =========";
-        error( KIO::ERR_DOES_NOT_EXIST, _path );
-        return;
-    }
-
-   	// don't knwo how to check
-   	// error( KIO::ERR_IS_FILE, url.path() );
-
-    if ( !dir.isReadable() ) {
-        kDebug(7101) << "========= ERR_CANNOT_ENTER_DIRECTORY =========";
-        error( KIO::ERR_CANNOT_ENTER_DIRECTORY, _path );
-        return;
-    }
-    QFileInfoList list = dir.entryInfoList();
-    totalSize( list.size() );
-
-    UDSEntry entry;
-    for (int i = 0; i < list.size(); ++i) {
-        QFileInfo fileInfo = list.at(i);
-        entry.clear();
-        if ( createUDSEntry( fileInfo.fileName(),
-                             fileInfo.absoluteFilePath().toAscii() /* we can use the filename as relative path*/,
-                             entry, 2, true ) )
-            listEntry( entry, false);
-    }
-#else
     KDE_struct_stat buff;
     if ( KDE_stat( _path.data(), &buff ) == -1 ) {
 	error( KIO::ERR_DOES_NOT_EXIST, _path );
@@ -1432,17 +1410,15 @@ void FileProtocol::listDir( const KUrl& url)
                              entry, 2, true ) )
           listEntry( entry, false);
     }
-#endif
 
     listEntry( entry, true ); // ready
 
     kDebug(7101) << "============= COMPLETED LIST ============";
 
-#ifndef Q_WS_WIN
     chdir(path_buffer);
-#endif
     finished();
 }
+#endif  // !Q_OS_WIN
 
 /*
 void FileProtocol::testDir( const QString& path )
