@@ -1,7 +1,8 @@
 // -*- mode: c++; c-basic-offset: 4 -*-
 /*
  *  This file is part of the KDE libraries
- *  Copyright (C) 2003, 2004, 2005, 2006 Apple Computer, Inc.
+ *  Copyright (C) 2003, 2004, 2005, 2006, 2007 Apple Computer, Inc.
+ *  Copyright (C) 2007 Maksim Orlovich <maksim@kde.org>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -35,8 +36,10 @@
 
 #include <pthread.h>
 #include <mach/mach_port.h>
+#include <mach/mach_init.h>
 #include <mach/task.h>
 #include <mach/thread_act.h>
+#include <mach/vm_map.h>
 
 #elif PLATFORM(WIN_OS)
 
@@ -44,7 +47,11 @@
 
 #elif PLATFORM(UNIX)
 
+#include <stdlib.h>
+#include <sys/types.h>
+#include <sys/mman.h>
 #include <pthread.h> //krazy:exclude=includes (yes, it's duplicate, but in a different #if branch)
+#include <unistd.h>
 
 #if HAVE(PTHREAD_NP_H)
 #include <pthread_np.h>
@@ -54,94 +61,254 @@
 
 #define DEBUG_COLLECTOR 0
 
+#if HAVE(VALGRIND_MEMCHECK_H) && !defined(NDEBUG)
+   #include <valgrind/memcheck.h>
+   #if defined(VALGRIND_MAKE_MEM_DEFINED)
+      #define VG_DEFINED(p) VALGRIND_MAKE_MEM_DEFINED(&p, sizeof(void*))
+   #else
+      #define VG_DEFINED(p)
+   #endif
+#else
+   #define VG_DEFINED(p)
+#endif
+
+
 using std::max;
 
 namespace KJS {
 
 // tunable parameters
-const size_t MINIMUM_CELL_SIZE = 48;
-const size_t BLOCK_SIZE = (8 * 4096);
 const size_t SPARE_EMPTY_BLOCKS = 2;
 const size_t MIN_ARRAY_SIZE = 14;
 const size_t GROWTH_FACTOR = 2;
 const size_t LOW_WATER_FACTOR = 4;
-const size_t ALLOCATIONS_PER_COLLECTION = 1000;
-
-// derived constants
-const size_t CELL_ARRAY_LENGTH = (MINIMUM_CELL_SIZE / sizeof(double)) + (MINIMUM_CELL_SIZE % sizeof(double) != 0 ? sizeof(double) : 0);
-const size_t CELL_SIZE = CELL_ARRAY_LENGTH * sizeof(double);
-const size_t CELLS_PER_BLOCK = ((BLOCK_SIZE * 8 - sizeof(uint32_t) * 8 - sizeof(void *) * 8) / (CELL_SIZE * 8));
+const size_t ALLOCATIONS_PER_COLLECTION = 4000;
 
 
+// A whee bit like a WTF::Vector, but perfectly POD, so can be used in global context
+// w/o worries. 
+struct BlockList {
+    CollectorBlock** m_data;
+    size_t m_used;
+    size_t m_capacity;
 
-struct CollectorCell {
-  union {
-    double memory[CELL_ARRAY_LENGTH];
-    struct {
-      void *zeroIfFree;
-      ptrdiff_t next;
-    } freeCell;
-  } u;
-};
+    CollectorBlock* operator[](int pos) {
+        return m_data[pos];
+    }
 
+    size_t used() const {
+        return m_used;
+    }
 
-struct CollectorBlock {
-  CollectorCell cells[CELLS_PER_BLOCK];
-  uint32_t usedCells;
-  CollectorCell *freeList;
+    void append(CollectorBlock* block) {
+        if (m_used == m_capacity) {
+            m_capacity = max(MIN_ARRAY_SIZE, m_capacity * GROWTH_FACTOR);
+            m_data = static_cast<CollectorBlock **>(fastRealloc(m_data, m_capacity * sizeof(CollectorBlock *)));
+        }
+        m_data[m_used] = block;
+        ++m_used;
+    }
+
+    void remove(CollectorBlock* block) {
+        size_t c;
+        for (c = 0; c < m_used; ++c)
+            if (m_data[c] == block)
+                break;
+
+        if (c == m_used)
+            return;
+
+        // Move things up, and shrink..
+        --m_used;
+        for (; c < m_used; ++c)
+            m_data[c] = m_data[c+1];
+    }
 };
 
 struct CollectorHeap {
-  CollectorBlock **blocks;
-  size_t numBlocks;
-  size_t usedBlocks;
+  // This contains the list of both normal and oversize blocks
+  BlockList allBlocks;
+
+  // Just the normal blocks
+  BlockList blocks;
+
   size_t firstBlockWithPossibleSpace;
 
-  CollectorCell **oversizeCells;
-  size_t numOversizeCells;
-  size_t usedOversizeCells;
+  // The oversize block handling is a bit tricky, since we do not wish to slow down
+  // the usual paths. Hence, we do the following:
+  // 1) We set the marked bits for any extension portions of the block.
+  //    this way the stack GC doesn't have to do anything special if a pointer
+  //    happens to go that way.
+  // 2) We keep track of an allBlocks list to help the stack GC tests things.
+  //
+  // The oversize heap itself represents blocks as follows:
+  // 1) free: marked = false, allocd = false, trailer = false, zeroIfFree = 0
+  // 2) alloc'd, head: marked = depends, allocd = true, trailer = false, zeroIfFree is uncertain
+  // 3) alloc'd, trailer: marked = true (see above), allocd = true, trailer = true, zeroIfFree is uncertain
+  //
+  // For now, we don't use a freelist, so performance may be quite bad if
+  // this is used heavily; this is just because the cell does not have
+  // back and forward links; which we need since we can pick a non-first cell
+  // ### actually, it may be possible to shuffle the list. Not now, though
+  BlockList oversizeBlocks;
 
   size_t numLiveObjects;
   size_t numLiveObjectsAtLastCollect;
 };
 
-static CollectorHeap heap = {NULL, 0, 0, 0, NULL, 0, 0, 0, 0};
+static CollectorHeap heap;
 
 bool Collector::memoryFull = false;
 
+static CollectorBlock* allocateBlock()
+{
+#if PLATFORM(DARWIN)
+    vm_address_t address = 0;
+    vm_map(current_task(), &address, BLOCK_SIZE, BLOCK_OFFSET_MASK, VM_FLAGS_ANYWHERE, MEMORY_OBJECT_NULL, 0, FALSE, VM_PROT_DEFAULT, VM_PROT_DEFAULT, VM_INHERIT_DEFAULT);
+#elif PLATFORM(WIN_OS)
+     // windows virtual address granularity is naturally 64k
+    LPVOID address = VirtualAlloc(NULL, BLOCK_SIZE, MEM_COMMIT | MEM_RESERVE, PAGE_READWRITE);
+#elif HAVE(POSIX_MEMALIGN)
+    void* address;
+    posix_memalign(address, BLOCK_SIZE, BLOCK_SIZE);
+    memset(reinterpret_cast<void*>(address), 0, BLOCK_SIZE);
+#else
+    static size_t pagesize = getpagesize();
+
+    size_t extra = 0;
+    if (BLOCK_SIZE > pagesize)
+        extra = BLOCK_SIZE - pagesize;
+
+    void* mmapResult = mmap(NULL, BLOCK_SIZE + extra, PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANON, -1, 0);
+    uintptr_t address = reinterpret_cast<uintptr_t>(mmapResult);
+
+    size_t adjust = 0;
+    if ((address & BLOCK_OFFSET_MASK) != 0)
+        adjust = BLOCK_SIZE - (address & BLOCK_OFFSET_MASK);
+
+    if (adjust > 0)
+        munmap(reinterpret_cast<void*>(address), adjust);
+
+    if (adjust < extra)
+        munmap(reinterpret_cast<void*>(address + adjust + BLOCK_SIZE), extra - adjust);
+
+    address += adjust;
+    memset(reinterpret_cast<void*>(address), 0, BLOCK_SIZE);
+#endif
+    CollectorBlock* ptr = reinterpret_cast<CollectorBlock*>(address);
+    heap.allBlocks.append(ptr); // Register with the heap
+    return ptr;
+}
+
+static void freeBlock(CollectorBlock* block)
+{
+    // Unregister the block first
+    heap.allBlocks.remove(block);
+    
+#if PLATFORM(DARWIN)
+    vm_deallocate(current_task(), reinterpret_cast<vm_address_t>(block), BLOCK_SIZE);
+#elif PLATFORM(WIN_OS)
+    VirtualFree(block, BLOCK_SIZE, MEM_RELEASE);
+#elif HAVE(POSIX_MEMALIGN)
+    free(block);
+#else
+    munmap(block, BLOCK_SIZE);
+#endif
+}
+
+static void* allocOversize(size_t s)
+{
+  size_t cellsNeeded = (s + (CELL_SIZE - 1)) / CELL_SIZE;
+
+  // printf("allocOversize, size:%d, cellsNeeded:%d\n", s, cellsNeeded);
+
+  // We are not oversize enough to deal with things close to 64K in size
+  assert(cellsNeeded <= CELLS_PER_BLOCK);
+
+  // Look through the blocks, see if one has enough, and where.
+  CollectorBlock* sufficientBlock = 0;
+  size_t startOffset = -1;
+  for (size_t b = 0; b < heap.oversizeBlocks.used() && !sufficientBlock; ++b) {
+    CollectorBlock* candidate = heap.oversizeBlocks[b];
+    if (cellsNeeded <= CELLS_PER_BLOCK - candidate->usedCells) {
+      // Well, there is a chance we will fit.. Let's see if we can find a batch long enough..
+      for (size_t i = 0; i < CELLS_PER_BLOCK; i++) {
+        if (i % 32 == 0 && candidate->allocd.bits[i/32] == 0xFFFFFFFF) {
+          // Can skip this and 31 other cells
+          i += 31;
+          continue;
+        }
+
+        if (candidate->allocd.get(i))
+            continue;
+
+        // This cell is free -- are there enough free cells after it?
+        startOffset = i;
+        size_t last = i + cellsNeeded - 1;
+
+        if (last >= CELLS_PER_BLOCK) // No way it will fit
+          break;
+        
+        ++i;
+        while (i <= last && !candidate->allocd.get(i))
+          ++i;
+
+        // Did we get through enough?
+        if (i == last + 1) {
+          sufficientBlock = candidate;
+          break;
+        }
+
+        // Not enough room -- and the current entry has a non-zero zeroIfFree,
+        // so we should go on at i + 1 on next iteration
+
+      } // for each cell per block.
+    } // if enough room in block
+  } // for each block
+
+  if (!sufficientBlock) {
+    sufficientBlock = allocateBlock();
+    startOffset     = 0;
+    heap.oversizeBlocks.append(sufficientBlock);
+  }
+
+  sufficientBlock->usedCells += cellsNeeded;
+  
+  // Set proper bits for trailers and the head
+  sufficientBlock->allocd.set(startOffset);
+  for (size_t t = startOffset + 1; t < startOffset + cellsNeeded; ++t) {
+    sufficientBlock->trailer.set(t);
+    sufficientBlock->marked.set(t);
+    sufficientBlock->allocd.set(t);
+  }
+
+  void* result = sufficientBlock->cells + startOffset;
+  memset(result, 0, s);
+  heap.numLiveObjects = heap.numLiveObjects + 1;
+  return result;
+}
+
 void* Collector::allocate(size_t s)
 {
   assert(JSLock::lockCount() > 0);
 
   // collect if needed
   size_t numLiveObjects = heap.numLiveObjects;
-  if (numLiveObjects - heap.numLiveObjectsAtLastCollect >= ALLOCATIONS_PER_COLLECTION) {
+  size_t numLiveObjectsAtLastCollect = heap.numLiveObjectsAtLastCollect;  
+  size_t numNewObjects = numLiveObjects - numLiveObjectsAtLastCollect;
+
+  if (numNewObjects >= ALLOCATIONS_PER_COLLECTION && numNewObjects >= numLiveObjectsAtLastCollect) {
     collect();
     numLiveObjects = heap.numLiveObjects;
   }
 
   if (s > CELL_SIZE) {
-    // oversize allocator
-    size_t usedOversizeCells = heap.usedOversizeCells;
-    size_t numOversizeCells = heap.numOversizeCells;
-
-    if (usedOversizeCells == numOversizeCells) {
-      numOversizeCells = max(MIN_ARRAY_SIZE, numOversizeCells * GROWTH_FACTOR);
-      heap.numOversizeCells = numOversizeCells;
-      heap.oversizeCells = static_cast<CollectorCell **>(fastRealloc(heap.oversizeCells, numOversizeCells * sizeof(CollectorCell *)));
-    }
-    
-    void *newCell = fastCalloc(s, 1);
-    heap.oversizeCells[usedOversizeCells] = static_cast<CollectorCell *>(newCell);
-    heap.usedOversizeCells = usedOversizeCells + 1;
-    heap.numLiveObjects = numLiveObjects + 1;
-
-    return newCell;
+    return allocOversize(s);
   }
 
   // slab allocator
 
-  size_t usedBlocks = heap.usedBlocks;
+  size_t usedBlocks = heap.blocks.used();
 
   size_t i = heap.firstBlockWithPossibleSpace;
   CollectorBlock *targetBlock;
@@ -161,19 +328,11 @@ void* Collector::allocate(size_t s)
   } else {
 allocateNewBlock:
     // didn't find one, need to allocate a new block
-    size_t numBlocks = heap.numBlocks;
-    if (usedBlocks == numBlocks) {
-      numBlocks = max(MIN_ARRAY_SIZE, numBlocks * GROWTH_FACTOR);
-      heap.numBlocks = numBlocks;
-      heap.blocks = static_cast<CollectorBlock **>(fastRealloc(heap.blocks, numBlocks * sizeof(CollectorBlock *)));
-    }
-
-    targetBlock = static_cast<CollectorBlock *>(fastCalloc(1, sizeof(CollectorBlock)));
+    targetBlock = allocateBlock();
     targetBlock->freeList = targetBlock->cells;
     targetBlockUsedCells = 0;
-    heap.blocks[usedBlocks] = targetBlock;
-    heap.usedBlocks = usedBlocks + 1;
-    heap.firstBlockWithPossibleSpace = usedBlocks;
+    heap.blocks.append(targetBlock);
+    heap.firstBlockWithPossibleSpace = usedBlocks; // previous usedBlocks -> new one's index
   }
 
   // find a free spot in the block and detach it from the free list
@@ -200,6 +359,7 @@ struct Collector::Thread {
 
 pthread_key_t registeredThreadKey;
 pthread_once_t registeredThreadKeyOnce = PTHREAD_ONCE_INIT;
+
 Collector::Thread *registeredThreads;
 
 static void destroyRegisteredThread(void *data)
@@ -245,8 +405,8 @@ void Collector::registerThread()
 
 #define IS_POINTER_ALIGNED(p) (((intptr_t)(p) & (sizeof(char *) - 1)) == 0)
 
-// cells are 8-byte aligned
-#define IS_CELL_ALIGNED(p) (((intptr_t)(p) & 7) == 0)
+// cell size needs to be a power of two for this to be valid
+#define IS_CELL_ALIGNED(p) (((intptr_t)(p) & CELL_MASK) == 0)
 
 void Collector::markStackObjectsConservatively(void *start, void *end)
 {
@@ -263,34 +423,30 @@ void Collector::markStackObjectsConservatively(void *start, void *end)
   char **p = (char **)start;
   char **e = (char **)end;
 
-  size_t usedBlocks = heap.usedBlocks;
-  CollectorBlock **blocks = heap.blocks;
-  size_t usedOversizeCells = heap.usedOversizeCells;
-  CollectorCell **oversizeCells = heap.oversizeCells;
+  // We use allBlocks here since we want to mark oversize cells as well.
+  // Their trailers will have the mark bit set already, to avoid trouble.
+  size_t usedBlocks = heap.allBlocks.used();
+  CollectorBlock **blocks = heap.allBlocks.m_data;
 
   const size_t lastCellOffset = sizeof(CollectorCell) * (CELLS_PER_BLOCK - 1);
 
   while (p != e) {
     char *x = *p++;
+    VG_DEFINED(x);
     if (IS_CELL_ALIGNED(x) && x) {
+      uintptr_t offset = reinterpret_cast<uintptr_t>(x) & BLOCK_OFFSET_MASK;
+      CollectorBlock* blockAddr = reinterpret_cast<CollectorBlock*>(x - offset);
       for (size_t block = 0; block < usedBlocks; block++) {
-        size_t offset = x - reinterpret_cast<char *>(blocks[block]);
-        if (offset <= lastCellOffset && offset % sizeof(CollectorCell) == 0)
-          goto gotGoodPointer;
-      }
-      for (size_t i = 0; i != usedOversizeCells; i++)
-        if (x == reinterpret_cast<char *>(oversizeCells[i]))
-          goto gotGoodPointer;
-      continue;
-
-gotGoodPointer:
-      if (((CollectorCell *)x)->u.freeCell.zeroIfFree != 0) {
-        JSCell *imp = reinterpret_cast<JSCell *>(x);
-        if (!imp->marked())
-          imp->mark();
-      }
-    }
-  }
+        if ((blocks[block] == blockAddr) & (offset <= lastCellOffset)) {
+          if (((CollectorCell *)x)->u.freeCell.zeroIfFree != 0) {
+            JSCell *imp = reinterpret_cast<JSCell *>(x);
+            if (!imp->marked())
+              imp->mark();
+          }
+        } // if valid block
+      } // for each block
+    } // if cell-aligned
+  } // for each pointer
 }
 
 void Collector::markCurrentThreadConservatively()
@@ -482,13 +638,21 @@ bool Collector::collect()
   markStackObjectsConservatively();
   markProtectedObjects();
   List::markProtectedLists();
+#if USE(MULTIPLE_THREADS)
+  if (!currentThreadIsMainThread)
+    markMainThreadOnlyObjects();
+#endif
 
   // SWEEP: delete everything with a zero refcount (garbage) and unmark everything else
 
+  // Note: if a cell has zeroIfFree == 0, it is either free,
+  // or in the middle of being constructed and has not yet
+  // had its vtable filled. Hence, such cells should not be cleaned up
+
   size_t emptyBlocks = 0;
   size_t numLiveObjects = heap.numLiveObjects;
 
-  for (size_t block = 0; block < heap.usedBlocks; block++) {
+  for (size_t block = 0; block < heap.blocks.used(); block++) {
     CollectorBlock *curBlock = heap.blocks[block];
 
     size_t usedCells = curBlock->usedCells;
@@ -499,9 +663,7 @@ bool Collector::collect()
       for (size_t i = 0; i < CELLS_PER_BLOCK; i++) {
         CollectorCell *cell = curBlock->cells + i;
         JSCell *imp = reinterpret_cast<JSCell *>(cell);
-        if (imp->m_marked) {
-          imp->m_marked = false;
-        } else if (currentThreadIsMainThread || imp->m_destructorIsThreadSafe) {
+        if (!curBlock->marked.get(i) && currentThreadIsMainThread) {
           // special case for allocated but uninitialized object 
           // (We don't need this check earlier because nothing prior this point assumes the object has a valid vptr.) 
           if (cell->u.freeCell.zeroIfFree == 0)
@@ -524,9 +686,7 @@ bool Collector::collect()
           ++minimumCellsToProcess;
         } else {
           JSCell *imp = reinterpret_cast<JSCell *>(cell);
-          if (imp->m_marked) {
-            imp->m_marked = false;
-          } else if (currentThreadIsMainThread || imp->m_destructorIsThreadSafe) {
+          if (!curBlock->marked.get(i) && currentThreadIsMainThread) {
             imp->~JSCell();
             --usedCells;
             --numLiveObjects;
@@ -540,6 +700,7 @@ bool Collector::collect()
       }
     }
 
+    curBlock->marked.clearAll();
     curBlock->usedCells = usedCells;
     curBlock->freeList = freeList;
 
@@ -547,17 +708,12 @@ bool Collector::collect()
       emptyBlocks++;
       if (emptyBlocks > SPARE_EMPTY_BLOCKS) {
 #if !DEBUG_COLLECTOR
-        fastFree(curBlock);
+        freeBlock(curBlock);
 #endif
         // swap with the last block so we compact as we go
-        heap.blocks[block] = heap.blocks[heap.usedBlocks - 1];
-        heap.usedBlocks--;
+        heap.blocks.m_data[block] = heap.blocks[heap.blocks.used() - 1];
+        heap.blocks.m_used--;
         block--; // Don't move forward a step in this case
-
-        if (heap.numBlocks > MIN_ARRAY_SIZE && heap.usedBlocks < heap.numBlocks / LOW_WATER_FACTOR) {
-          heap.numBlocks = heap.numBlocks / GROWTH_FACTOR;
-          heap.blocks = (CollectorBlock **)fastRealloc(heap.blocks, heap.numBlocks * sizeof(CollectorBlock *));
-        }
       }
     }
   }
@@ -565,36 +721,75 @@ bool Collector::collect()
   if (heap.numLiveObjects != numLiveObjects)
     heap.firstBlockWithPossibleSpace = 0;
 
-  size_t cell = 0;
-  while (cell < heap.usedOversizeCells) {
-    JSCell *imp = (JSCell *)heap.oversizeCells[cell];
+  // Now sweep oversize blocks.
+  emptyBlocks = 0;
+  for (size_t ob = 0; ob < heap.oversizeBlocks.used(); ++ob) {
+    CollectorBlock* curBlock = heap.oversizeBlocks[ob];
+    CollectorCell *freeList = curBlock->freeList;
+    size_t usedCells = curBlock->usedCells;
 
-    if (!imp->m_marked && (currentThreadIsMainThread || imp->m_destructorIsThreadSafe)) {
-      if (heap.oversizeCells[cell]->u.freeCell.zeroIfFree == 0) //If partly initialized, better don't touch it!
+    // Go through the cells
+    for (size_t i = 0; i < CELLS_PER_BLOCK; i++) {
+      if (i % 32 == 0 && curBlock->allocd.bits[i/32] == 0) {
+        // Nothing used around here, skip this and 31 next cells
+        i += 31;
         continue;
-      imp->~JSCell();
-#if DEBUG_COLLECTOR
-      heap.oversizeCells[cell]->u.freeCell.zeroIfFree = 0;
-#else
-      fastFree(imp);
-#endif
-
-      // swap with the last oversize cell so we compact as we go
-      heap.oversizeCells[cell] = heap.oversizeCells[heap.usedOversizeCells - 1];
-
-      heap.usedOversizeCells--;
-      numLiveObjects--;
-
-      if (heap.numOversizeCells > MIN_ARRAY_SIZE && heap.usedOversizeCells < heap.numOversizeCells / LOW_WATER_FACTOR) {
-        heap.numOversizeCells = heap.numOversizeCells / GROWTH_FACTOR;
-        heap.oversizeCells = (CollectorCell **)fastRealloc(heap.oversizeCells, heap.numOversizeCells * sizeof(CollectorCell *));
       }
-    } else {
-      imp->m_marked = false;
-      cell++;
-    }
-  }
 
+      CollectorCell *cell = curBlock->cells + i;
+      if (cell->u.freeCell.zeroIfFree == 0)
+        continue;
+
+      if (!curBlock->allocd.get(i))
+        continue;
+
+      JSCell *imp = reinterpret_cast<JSCell *>(cell);
+
+      // Live and trailer cells will all have the mark set,
+      // so we only have to collect with it clear --
+      // and we already took care of those that are
+      // already free (or being initialized) above
+      if (!curBlock->marked.get(i)) {
+        // Free this block...
+        imp->~JSCell();
+        --numLiveObjects;
+        --usedCells;
+
+        // Mark the block and the trailers as available
+        cell->u.freeCell.zeroIfFree = 0;
+        curBlock->allocd.clear(i);
+
+        ++i; // Go to the potential trailer..
+        while (curBlock->trailer.get(i) && i < CELLS_PER_BLOCK) {
+          --usedCells;
+          curBlock->allocd.clear(i);
+          curBlock->trailer.clear(i);
+          curBlock->marked.clear (i);
+          curBlock->cells[i].u.freeCell.zeroIfFree = 0;
+          ++i;
+        }
+        --i; // Last item we processed.
+      } else {
+        // If this is not a trailer cell, clear the mark
+        if (!curBlock->trailer.get(i))
+          curBlock->marked.clear(i);
+      }
+    } // each cell
+    curBlock->usedCells = usedCells;
+    curBlock->freeList = freeList;
+    if (usedCells == 0) {
+      emptyBlocks++;
+      if (emptyBlocks > SPARE_EMPTY_BLOCKS) {
+        freeBlock(curBlock);
+
+        // swap with the last block so we compact as we go
+        heap.oversizeBlocks.m_data[ob] = heap.oversizeBlocks[heap.oversizeBlocks.used() - 1];
+        heap.oversizeBlocks.m_used--;
+        ob--; // Don't move forward a step in this case
+      }
+    }
+  } // each oversize block
+    
   bool deleted = heap.numLiveObjects != numLiveObjects;
 
   heap.numLiveObjects = numLiveObjects;
