@@ -60,57 +60,50 @@ extern YYLTYPE kjsyylloc; // global bison variable holding token info
 // a bridge for yacc from the C world to C++
 int kjsyylex()
 {
-  return Lexer::curr()->lex();
+  return lexer().lex();
 }
 
 namespace KJS {
 
-static Lexer* currLexer = 0;
-
 static bool isDecimalDigit(int c);
 
+static const size_t initialReadBufferCapacity = 32;
+static const size_t initialStringTableCapacity = 64;
+
+Lexer& lexer()
+{
+    // ASSERT(JSLock::currentThreadIsHoldingLock());
+
+    // FIXME: We'd like to avoid calling new here, but we don't currently 
+    // support tearing down the Lexer at app quit time, since that would involve
+    // tearing down its UString data members without holding the JSLock.
+    static Lexer* staticLexer = new Lexer;
+    return *staticLexer;
+}
+
 Lexer::Lexer()
-  : yylineno(1),
-    size8(128), size16(128), restrKeyword(false),
-    eatNextIdentifier(false), stackToken(-1), lastToken(-1), pos(0),
-    code(0), length(0),
+  : yylineno(1)
+  , restrKeyword(false)
+  , eatNextIdentifier(false)
+  , stackToken(-1)
+  , lastToken(-1)
+  , pos(0)
+  , code(0)
+  , length(0)
 #ifndef KJS_PURE_ECMA
-    bol(true),
+  , bol(true)
 #endif
-    current(0), next1(0), next2(0), next3(0),
-    strings(0), numStrings(0), stringsCapacity(0),
-    identifiers(0), numIdentifiers(0), identifiersCapacity(0)
+  , current(0)
+  , next1(0)
+  , next2(0)
+  , next3(0)
 {
-  // allocate space for read buffers
-  buffer8 = new char[size8];
-  buffer16 = new KJS::UChar[size16];
-  currLexer = this;
+    m_buffer8.reserveCapacity(initialReadBufferCapacity);
+    m_buffer16.reserveCapacity(initialReadBufferCapacity);
+    m_strings.reserveCapacity(initialStringTableCapacity);
+    m_identifiers.reserveCapacity(initialStringTableCapacity);
 }
 
-Lexer::~Lexer()
-{
-  doneParsing();
-  delete [] buffer8;
-  delete [] buffer16;
-}
-
-Lexer *Lexer::curr()
-{
-  if (!currLexer) {
-    // create singleton instance
-    currLexer = new Lexer();
-  }
-  return currLexer;
-}
-
-#ifdef KJS_DEBUG_MEM
-void Lexer::globalClear()
-{
-  delete currLexer;
-  currLexer = 0L;
-}
-#endif
-
 void Lexer::setCode(const UString &sourceURL, int startingLineNumber, const KJS::UChar *c, unsigned int len)
 {
   yylineno = 1 + startingLineNumber;
@@ -170,7 +163,8 @@ int Lexer::lex()
   int token = 0;
   state = Start;
   unsigned short stringType = 0; // either single or double quotes
-  pos8 = pos16 = 0;
+  m_buffer8.clear();
+  m_buffer16.clear();
   done = false;
   terminator = false;
   skipLF = false;
@@ -227,7 +221,7 @@ int Lexer::lex()
         record16(current);
         state = InIdentifierOrKeyword;
       } else if (current == '\\') {
-        state = InIdentifierUnicodeEscapeStart;
+        state = InIdentifierStartUnicodeEscapeStart;
       } else if (current == '0') {
         record8(current);
         state = InNum0;
@@ -356,7 +350,7 @@ int Lexer::lex()
       if (isIdentPart(current))
         record16(current);
       else if (current == '\\')
-        state = InIdentifierUnicodeEscapeStart;
+        state = InIdentifierPartUnicodeEscapeStart;
       else
         setDone(state == InIdentifierOrKeyword ? IdentifierOrKeyword : Identifier);
       break;
@@ -433,20 +427,45 @@ int Lexer::lex()
       } else
         setDone(Number);
       break;
-    case InIdentifierUnicodeEscapeStart:
+    case InIdentifierStartUnicodeEscapeStart:
       if (current == 'u')
-        state = InIdentifierUnicodeEscape;
+        state = InIdentifierStartUnicodeEscape;
       else
         setDone(Bad);
       break;
-    case InIdentifierUnicodeEscape:
-      if (isHexDigit(current) && isHexDigit(next1) && isHexDigit(next2) && isHexDigit(next3)) {
-        record16(convertUnicode(current, next1, next2, next3));
-        shift(3);
-        state = InIdentifier;
-      } else {
+    case InIdentifierPartUnicodeEscapeStart:
+      if (current == 'u')
+        state = InIdentifierPartUnicodeEscape;
+      else
         setDone(Bad);
+      break;
+    case InIdentifierStartUnicodeEscape:
+      if (!isHexDigit(current) || !isHexDigit(next1) || !isHexDigit(next2) || !isHexDigit(next3)) {
+        setDone(Bad);
+        break;
       }
+      token = convertUnicode(current, next1, next2, next3).uc;
+      shift(3);
+      if (!isIdentStart(token)) {
+        setDone(Bad);
+        break;
+      }
+      record16(token);
+      state = InIdentifier;
+      break;
+    case InIdentifierPartUnicodeEscape:
+      if (!isHexDigit(current) || !isHexDigit(next1) || !isHexDigit(next2) || !isHexDigit(next3)) {
+        setDone(Bad);
+        break;
+      }
+      token = convertUnicode(current, next1, next2, next3).uc;
+      shift(3);
+      if (!isIdentPart(token)) {
+        setDone(Bad);
+        break;
+      }
+      record16(token);
+      state = InIdentifier;
       break;
     default:
       assert(!"Unhandled state in switch statement");
@@ -466,37 +485,37 @@ int Lexer::lex()
     state = Bad;
 
   // terminate string
-  buffer8[pos8] = '\0';
+  m_buffer8.append('\0');
 
 #ifdef KJS_DEBUG_LEX
   fprintf(stderr, "line: %d ", lineNo());
-  fprintf(stderr, "yytext (%x): ", buffer8[0]);
-  fprintf(stderr, "%s ", buffer8);
+  fprintf(stderr, "yytext (%x): ", m_buffer8[0]);
+  fprintf(stderr, "%s ", m_buffer8.data());
 #endif
 
   double dval = 0;
   if (state == Number) {
-    dval = kjs_strtod(buffer8, 0L);
+    dval = kjs_strtod(m_buffer8.data(), 0L);
   } else if (state == Hex) { // scan hex numbers
-    const char *p = buffer8 + 2;
+    const char *p = m_buffer8.data() + 2;
     while (char c = *p++) {
       dval *= 16;
       dval += convertHex(c);
     }
 
     if (dval >= mantissaOverflowLowerBound)
-      dval = parseIntOverflow(buffer8 + 2, p - (buffer8 + 3), 16);
+      dval = parseIntOverflow(m_buffer8.data() + 2, p - (m_buffer8.data() + 3), 16);
 
     state = Number;
   } else if (state == Octal) {   // scan octal number
-    const char *p = buffer8 + 1;
+    const char *p = m_buffer8.data() + 1;
     while (char c = *p++) {
       dval *= 8;
       dval += c - '0';
     }
 
     if (dval >= mantissaOverflowLowerBound)
-      dval = parseIntOverflow(buffer8 + 1, p - (buffer8 + 2), 8);
+      dval = parseIntOverflow(m_buffer8.data() + 1, p - (m_buffer8.data() + 2), 8);
 
     state = Number;
   }
@@ -541,7 +560,7 @@ int Lexer::lex()
     }
     break;
   case IdentifierOrKeyword:
-    if ((token = Lookup::find(&mainTable, buffer16, pos16)) < 0) {
+    if ((token = Lookup::find(&mainTable, m_buffer16.data(), m_buffer16.size())) < 0) {
   case Identifier:
       // Lookup for keyword failed, means this is an identifier
       // Apply anonymous-function hack below (eat the identifier)
@@ -550,7 +569,7 @@ int Lexer::lex()
         token = lex();
         break;
       }
-      kjsyylval.ident = makeIdentifier(buffer16, pos16);
+      kjsyylval.ident = makeIdentifier(m_buffer16);
       token = IDENT;
       break;
     }
@@ -565,7 +584,7 @@ int Lexer::lex()
       restrKeyword = true;
     break;
   case String:
-    kjsyylval.ustr = makeUString(buffer16, pos16);
+    kjsyylval.ustr = makeUString(m_buffer16);
     token = STRING;
     break;
   case Number:
@@ -611,6 +630,7 @@ bool Lexer::isWhiteSpace() const
     case 0x202F:  // NARROW NO-BREAK SPACE
     case 0x205F:  // MEDIUM MATHEMATICAL SPACE
     case 0x3000:  // IDEOGRAPHIC SPACE
+    case 0xFEFF:  // BOM/ZWNBS
         return true;
     default:
         return false;
@@ -853,45 +873,26 @@ KJS::UChar Lexer::convertUnicode(int c1, int c2, int c3, int c4)
 
 void Lexer::record8(int c)
 {
-  assert(c >= 0);
-  assert(c <= 0xff);
-
-  // enlarge buffer if full
-  if (pos8 >= size8 - 1) {
-    char *tmp = new char[2 * size8];
-    memcpy(tmp, buffer8, size8 * sizeof(char));
-    delete [] buffer8;
-    buffer8 = tmp;
-    size8 *= 2;
-  }
-
-  buffer8[pos8++] = (char) c;
+    ASSERT(c >= 0);
+    ASSERT(c <= 0xff);
+    m_buffer8.append(c);
 }
 
 void Lexer::record16(int c)
 {
-  ASSERT(c >= 0);
-  ASSERT(c <= USHRT_MAX);
-  record16(UChar(static_cast<unsigned short>(c)));
+    ASSERT(c >= 0);
+    ASSERT(c <= USHRT_MAX);
+    record16(UChar(static_cast<unsigned short>(c)));
 }
 
 void Lexer::record16(KJS::UChar c)
 {
-  // enlarge buffer if full
-  if (pos16 >= size16 - 1) {
-    KJS::UChar *tmp = new KJS::UChar[2 * size16];
-    memcpy(tmp, buffer16, size16 * sizeof(KJS::UChar));
-    delete [] buffer16;
-    buffer16 = tmp;
-    size16 *= 2;
-  }
-
-  buffer16[pos16++] = c;
+    m_buffer16.append(c);
 }
 
 bool Lexer::scanRegExp()
 {
-  pos16 = 0;
+  m_buffer16.clear();
   bool lastWasEscape = false;
   bool inBrackets = false;
 
@@ -910,12 +911,11 @@ bool Lexer::scanRegExp()
         record16(current);
         lastWasEscape =
             !lastWasEscape && (current == '\\');
-    }
-    else { // end of regexp
-      pattern = UString(buffer16, pos16);
-      pos16 = 0;
-      shift(1);
-      break;
+    } else { // end of regexp
+        m_pattern = UString(m_buffer16);
+        m_buffer16.clear();
+        shift(1);
+        break;
     }
     shift(1);
   }
@@ -924,58 +924,48 @@ bool Lexer::scanRegExp()
     record16(current);
     shift(1);
   }
-  flags = UString(buffer16, pos16);
+  m_flags = UString(m_buffer16);
 
   return true;
 }
 
 
-void Lexer::doneParsing()
+void Lexer::clear()
 {
-  for (unsigned i = 0; i < numIdentifiers; i++) {
-    delete identifiers[i];
-  }
-  fastFree(identifiers);
-  identifiers = 0;
-  numIdentifiers = 0;
-  identifiersCapacity = 0;
+    deleteAllValues(m_strings);
+    Vector<UString*> newStrings;
+    newStrings.reserveCapacity(initialStringTableCapacity);
+    m_strings.swap(newStrings);
+    deleteAllValues(m_identifiers);
+    Vector<KJS::Identifier*> newIdentifiers;
+    newIdentifiers.reserveCapacity(initialStringTableCapacity);
+    m_identifiers.swap(newIdentifiers);
 
-  for (unsigned i = 0; i < numStrings; i++) {
-    delete strings[i];
-  }
-  fastFree(strings);
-  strings = 0;
-  numStrings = 0;
-  stringsCapacity = 0;
+    Vector<char> newBuffer8;
+    newBuffer8.reserveCapacity(initialReadBufferCapacity);
+    m_buffer8.swap(newBuffer8);
+
+    Vector<UChar> newBuffer16;
+    newBuffer16.reserveCapacity(initialReadBufferCapacity);
+    m_buffer16.swap(newBuffer16);
+
+    m_pattern = 0;
+    m_flags = 0;
+    m_sourceURL = 0;
 }
 
-const int initialCapacity = 64;
-const int growthFactor = 2;
-
-// FIXME: this completely ignores its parameters, instead using buffer16 and pos16 - wtf?
-Identifier *Lexer::makeIdentifier(KJS::UChar*, unsigned int)
+Identifier* Lexer::makeIdentifier(const Vector<KJS::UChar>& buffer)
 {
-  if (numIdentifiers == identifiersCapacity) {
-    identifiersCapacity = (identifiersCapacity == 0) ? initialCapacity : identifiersCapacity *growthFactor;
-    identifiers = (KJS::Identifier **)fastRealloc(identifiers, sizeof(KJS::Identifier *) * identifiersCapacity);
-  }
-
-  KJS::Identifier *identifier = new KJS::Identifier(buffer16, pos16);
-  identifiers[numIdentifiers++] = identifier;
-  return identifier;
+    KJS::Identifier* identifier = new KJS::Identifier(buffer.data(), buffer.size());
+    m_identifiers.append(identifier);
+    return identifier;
 }
 
-// FIXME: this completely ignores its parameters, instead using buffer16 and pos16 - wtf?
-UString *Lexer::makeUString(KJS::UChar*, unsigned int)
+UString* Lexer::makeUString(const Vector<KJS::UChar>& buffer)
 {
-  if (numStrings == stringsCapacity) {
-    stringsCapacity = (stringsCapacity == 0) ? initialCapacity : stringsCapacity *growthFactor;
-    strings = (UString **)fastRealloc(strings, sizeof(UString *) * stringsCapacity);
-  }
-
-  UString *string = new UString(buffer16, pos16);
-  strings[numStrings++] = string;
-  return string;
+    UString* string = new UString(buffer);
+    m_strings.append(string);
+    return string;
 }
 
-}
+} // namespace KJS
