@@ -121,7 +121,7 @@ Node::Node()
 #ifndef NDEBUG
     ++NodeCounter::count;
 #endif
-  m_line = Lexer::curr()->lineNo();
+  m_line = lexer().lineNo();
   if (!newNodes)
       newNodes = new HashSet<Node*>;
   newNodes->add(this);
@@ -2776,8 +2776,8 @@ void ParameterNode::recurseVisit(NodeVisitor *visitor)
 
 FunctionBodyNode::FunctionBodyNode(SourceElementsNode *s)
     : BlockNode(s)
-    , m_sourceURL(Lexer::curr()->sourceURL())
-    , m_sourceId(Parser::sid)
+    , m_sourceURL(lexer().sourceURL())
+    , m_sourceId(parser().sourceId())
     , m_builtSymbolList(false)
 {
 
@@ -2885,7 +2885,7 @@ FunctionImp* FuncDeclNode::makeFunctionObject(ExecState *exec)
   FunctionImp *func = new DeclaredFunctionImp(exec, ident, body.get(), context->scopeChain());
 
   JSObject *proto = exec->lexicalInterpreter()->builtinObject()->construct(exec, List::empty());
-  proto->put(exec, exec->propertyNames().constructor, func, ReadOnly | DontDelete | DontEnum);
+  proto->put(exec, exec->propertyNames().constructor, func, DontEnum);
   func->put(exec, exec->propertyNames().prototype, proto, Internal|DontDelete);
 
   func->put(exec, exec->propertyNames().length, jsNumber(body->numParams()), ReadOnly|DontDelete|DontEnum);
@@ -2924,7 +2924,7 @@ JSValue *FuncExprNode::evaluate(ExecState *exec)
 
   FunctionImp *func = new DeclaredFunctionImp(exec, ident, body.get(), context->scopeChain());
   JSObject *proto = exec->lexicalInterpreter()->builtinObject()->construct(exec, List::empty());
-  proto->put(exec, exec->propertyNames().constructor, func, ReadOnly|DontDelete|DontEnum);
+  proto->put(exec, exec->propertyNames().constructor, func, DontEnum);
   func->put(exec, exec->propertyNames().prototype, proto, Internal|DontDelete);
 
   if (named) {
