@@ -83,6 +83,7 @@ namespace KJS {
     GetterSetterImp() : getter(0), setter(0) { }
       
     virtual JSValue *toPrimitive(ExecState *exec, JSType preferred = UnspecifiedType) const;
+    virtual bool getPrimitiveNumber(ExecState*, double& number, JSValue*& value);
     virtual bool toBoolean(ExecState *exec) const;
     virtual double toNumber(ExecState *exec) const;
     virtual UString toString(ExecState *exec) const;
@@ -416,6 +417,7 @@ namespace KJS {
     virtual void getPropertyNames(ExecState*, PropertyNameArray&);
 
     virtual JSValue *toPrimitive(ExecState *exec, JSType preferredType = UnspecifiedType) const;
+    virtual bool getPrimitiveNumber(ExecState*, double& number, JSValue*& value);
     virtual bool toBoolean(ExecState *exec) const;
     virtual double toNumber(ExecState *exec) const;
     virtual UString toString(ExecState *exec) const;
@@ -443,6 +445,7 @@ namespace KJS {
     void putDirectFunction(InternalFunctionImp*, int attr = 0);
 
     void fillGetterPropertySlot(PropertySlot& slot, JSValue **location);
+    void fillDirectLocationSlot(PropertySlot& slot, JSValue **location);
 
     void defineGetter(ExecState *exec, const Identifier& propertyName, JSObject *getterFunc);
     void defineSetter(ExecState *exec, const Identifier& propertyName, JSObject *setterFunc);
@@ -547,6 +550,16 @@ inline bool JSObject::inherits(const ClassInfo *info) const
     return false;
 }
 
+inline void JSObject::fillDirectLocationSlot(PropertySlot& slot,
+                                             JSValue **location)
+{
+    if (_prop.hasGetterSetterProperties() &&
+        (*location)->type() == GetterSetterType)
+        fillGetterPropertySlot(slot, location);
+    else
+        slot.setValueSlot(this, location);
+}
+
 // this method is here to be after the inline declaration of JSObject::inherits
 inline bool JSCell::isObject(const ClassInfo *info) const
 {
