@@ -339,7 +339,7 @@ static JSValue *replace(ExecState *exec, const UString &source, JSValue *pattern
     reg->prepareMatch(source);
     do {
       int *ovector;
-      UString matchString = regExpObj->performMatch(reg, source, startPosition, &matchIndex, &ovector);
+      UString matchString = regExpObj->performMatch(reg, exec, source, startPosition, &matchIndex, &ovector);
       if (matchIndex == -1)
         break;
       int matchLen = matchString.size();
@@ -537,7 +537,8 @@ JSValue *StringProtoFunc::callAsFunction(ExecState* exec, JSObject* thisObj, con
     }
     RegExpObjectImp* regExpObj = static_cast<RegExpObjectImp*>(exec->lexicalInterpreter()->builtinRegExp());
     reg->prepareMatch(u);
-    UString mstr = regExpObj->performMatch(reg, u, 0, &pos);
+    UString mstr = regExpObj->performMatch(reg, exec, u, 0, &pos);
+    
     if (id == Search) {
       result = jsNumber(pos);
     } else {
@@ -560,7 +561,7 @@ JSValue *StringProtoFunc::callAsFunction(ExecState* exec, JSObject* thisObj, con
             list.append(jsString(mstr));
           lastIndex = pos;
           pos += mstr.isEmpty() ? 1 : mstr.size();
-          mstr = regExpObj->performMatch(reg, u, pos, &pos);
+          mstr = regExpObj->performMatch(reg, exec, u, pos, &pos);
         }
         if (imp)
           imp->put(exec, "lastIndex", jsNumber(lastIndex), DontDelete|DontEnum);
@@ -610,17 +611,20 @@ JSValue *StringProtoFunc::callAsFunction(ExecState* exec, JSObject* thisObj, con
     if (a0->isObject() && static_cast<JSObject *>(a0)->inherits(&RegExpImp::info)) {
       RegExp *reg = static_cast<RegExpImp *>(a0)->regExp();
       reg->prepareMatch(u);
-      if (u.isEmpty() && !reg->match(u, 0).isNull()) {
+      bool error = false;
+      if (u.isEmpty() && !reg->match(u, &error, 0).isNull()) {
+        reg->doneMatch();
+        
         // empty string matched by regexp -> empty array
         res->put(exec, exec->propertyNames().length, jsNumber(0));
         break;
       }
       pos = 0;
-      while (static_cast<uint32_t>(i) != limit && pos < u.size()) {
+      while (!error && static_cast<uint32_t>(i) != limit && pos < u.size()) {
         // TODO: back references
         int mpos;
         int *ovector = 0L;
-        UString mstr = reg->match(u, pos, &mpos, &ovector);
+        UString mstr = reg->match(u, &error, pos, &mpos, &ovector);
         delete [] ovector; ovector = 0L;
         if (mpos < 0)
           break;
@@ -632,6 +636,9 @@ JSValue *StringProtoFunc::callAsFunction(ExecState* exec, JSObject* thisObj, con
         }
       }
       reg->doneMatch();
+      if (error)
+        RegExpObjectImp::throwRegExpError(exec);
+      
     } else {
       u2 = a0->toString(exec);
       if (u2.isEmpty()) {
@@ -657,23 +664,28 @@ JSValue *StringProtoFunc::callAsFunction(ExecState* exec, JSObject* thisObj, con
     res->put(exec, exec->propertyNames().length, jsNumber(i));
     }
     break;
-  case Substr: {
-    double d = a0->toInteger(exec);
-    double d2 = a1->toInteger(exec);
-    if (!(d >= 0)) { // true for NaN
-      d += len;
-      if (!(d >= 0)) // true for NaN
-        d = 0;
+  case Substr: { //B.2.3
+    // Note: NaN is effectively handled as 0 here for both length
+    // and start, hence toInteger does fine, and removes worries
+    // about weird comparison results below.
+    int len = s.size();
+    double start  = a0->toInteger(exec);
+    double length = a1->isUndefined() ? len : a1->toInteger(exec);
+
+    if (start >= len)
+      return jsString("");
+    if (length < 0)
+      return jsString("");
+    if (start < 0) {
+      start += s.size();
+      if (start < 0)
+       start = 0;
     }
-    if (isNaN(d2))
-      d2 = len - d;
-    else {
-      if (d2 < 0)
-        d2 = 0;
-      if (d2 > len - d)
-        d2 = len - d;
-    }
-    result = jsString(s.substr(static_cast<int>(d), static_cast<int>(d2)));
+
+    if (length > len - start)
+      length = len - start;
+
+    result = jsString(s.substr(static_cast<int>(start), static_cast<int>(length)));
     break;
   }
   case Substring: {
