@@ -160,7 +160,7 @@ static unsigned short almostUChar;
 UString::Rep UString::Rep::null = { 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0 };
 UString::Rep UString::Rep::empty = { 0, 0, 1, 0, 0, 0, reinterpret_cast<UChar*>(&almostUChar), 0, 0, 0, 0 };
 const int normalStatBufferSize = 4096;
-static char *statBuffer = 0;
+static char *statBuffer = 0; // FIXME: This buffer is never deallocated.
 static int statBufferSize = 0;
 
 UString::Rep&
@@ -446,6 +446,15 @@ UString::UString(UChar* c, int length, bool copy)
     m_rep.assign(Rep::create(c, length));
 }
 
+UString::UString(const Vector<UChar>& buffer)
+{
+    if (!buffer.size())
+        m_rep = &Rep::empty;
+    else
+        m_rep = Rep::createCopying(buffer.data(), buffer.size());
+}
+
+
 UString::UString(const UString &a, const UString &b)
 {
   int aSize = a.size();
@@ -909,15 +918,6 @@ char *UString::ascii() const
   return statBuffer;
 }
 
-#ifdef KJS_DEBUG_MEM
-void UString::globalClear()
-{
-  delete [] statBuffer;
-  statBuffer = 0;
-  statBufferSize = 0;
-}
-#endif
-
 UString& UString::operator=(Empty)
 {
   m_rep = &Rep::empty;
