@@ -280,12 +280,6 @@ private:
   * @li @p copy : Copy selected items to clipboard and store 'cut' in clipboard.
   * @li @p paste : Paste clipboard into view URL.
   * @li @p pasteTo(const KUrl &) : Paste clipboard into given URL.
-  * @li @p rename : Rename item in place.
-  * @li @p trash : Move selected items to trash.
-  * @li @p del : Delete selected items (couldn't call it delete!).
-  * @li @p shred : Shred selected items (secure deletion) - DEPRECATED.
-  * @li @p properties : Show file/document properties.
-  * @li @p editMimeType : show file/document's mimetype properties.
   * @li @p searchProvider : Lookup selected text at default search provider
   *
   * [normal actions]
@@ -310,8 +304,6 @@ private:
   * and having parameters.
   *
   * [configuration slot]
-  * @li @p setSaveViewPropertiesLocally( bool ): If @p true, view properties are saved into .directory
-  *                                       otherwise, they are saved globally.
   * @li @p disableScrolling: no scrollbars
   */
 class KPARTS_EXPORT BrowserExtension : public QObject
@@ -344,7 +336,7 @@ public:
       NoDeletion=0x0040, /**< deletion, trashing and renaming not allowed (e.g. parent dir not writeable).
        *            (this is only needed if the protocol itself supports deletion, unlike e.g. HTTP) */
       IsLink=0x0080, /**< show "Bookmark This Link" and other link-related actions (linkactions merging group) */
-      ShowUrlOperations=0x0100, /**< show copy,cut,paste, as well as rename+trash+delete, if NoDeletion is not set. */
+      ShowUrlOperations=0x0100, /**< show copy, paste, as well as cut if NoDeletion is not set. */
       ShowProperties=0x200     /**< show "Properties" action (usually done by directory views) */
   };
 
