@@ -537,7 +537,7 @@ bool ReadOnlyPart::openUrl( const KUrl &url )
     if ( d->m_url.isLocalFile() )
     {
         emit started( 0 );
-        d->m_file = d->m_url.path();
+        d->m_file = d->m_url.toLocalFile();
         // set the mimetype only if it was not already set (for example, by the host application)
         if (d->m_arguments.mimeType().isEmpty())
         {
@@ -752,7 +752,10 @@ bool ReadWritePart::queryClose()
     QString docName = url().fileName();
     if (docName.isEmpty()) docName = i18n( "Untitled" );
 
-    int res = KMessageBox::warningYesNoCancel( widget(),
+    QWidget *parentWidget=widget();
+    if(!parentWidget) parentWidget=QApplication::activeWindow();
+
+    int res = KMessageBox::warningYesNoCancel( parentWidget,
                                                i18n( "The document \"%1\" has been modified.\n"
                                                      "Do you want to save your changes or discard them?" ,  docName ),
                                                i18n( "Close Document" ), KStandardGuiItem::save(), KStandardGuiItem::discard() );
@@ -767,7 +770,7 @@ bool ReadWritePart::queryClose()
         {
             if (d->m_url.isEmpty())
             {
-                KUrl url = KFileDialog::getSaveUrl();
+                KUrl url = KFileDialog::getSaveUrl(KUrl(), QString(), parentWidget);
                 if (url.isEmpty())
                     return false;
 
