@@ -298,7 +298,7 @@ ActionCollectionView::ActionCollectionView(QWidget* parent)
 
     d->collection = new KActionCollection(this);
 
-    KAction* runaction = new KAction(KIcon("launch"), i18n("Run"), this);
+    KAction* runaction = new KAction(KIcon("system-run"), i18n("Run"), this);
     runaction->setObjectName("run");
     runaction->setToolTip( i18n("Execute the selected script.") );
     runaction->setEnabled(false);
@@ -312,7 +312,7 @@ ActionCollectionView::ActionCollectionView(QWidget* parent)
     d->collection->addAction("stop", stopaction);
     connect(stopaction, SIGNAL(triggered()), this, SLOT(slotStop()));
 
-    KAction* editaction = new KAction(KIcon("edit"), i18n("Edit..."), this);
+    KAction* editaction = new KAction(KIcon("document-properties"), i18n("Edit..."), this);
     editaction->setObjectName("edit");
     editaction->setToolTip( i18n("Edit selected script.") );
     editaction->setEnabled(false);
