@@ -1,54 +1,57 @@
 /*
- *
- * $Id: sourceheader 511311 2006-02-19 14:51:05Z trueg $
- *
  * This file is part of the Nepomuk KDE project.
  * Copyright (C) 2006-2007 Sebastian Trueg <trueg@kde.org>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
+ * modify it under the terms of the GNU Library General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; see the file COPYING.LIB.  If not, write to
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include "resourcedata.h"
 #include "resourcemanager.h"
-#include "tools.h"
 #include "generated/resource.h"
+#include "tools.h"
 
 #include <Soprano/Statement>
 #include <Soprano/StatementIterator>
 #include <Soprano/Model>
+#include <Soprano/Vocabulary/RDFS>
+#include <Soprano/Vocabulary/RDF>
 
 #include "ontology/class.h"
 
-#include <kdebug.h>
+#include <QtCore/QFile>
 
+#include <kdebug.h>
+#include <kurl.h>
 
 using namespace Soprano;
 
 
-typedef QMap<QString, Nepomuk::ResourceData*> ResourceDataHash;
+typedef QHash<QString, Nepomuk::ResourceData*> ResourceDataHash;
 
 Q_GLOBAL_STATIC( ResourceDataHash, initializedData )
 Q_GLOBAL_STATIC( ResourceDataHash, kickoffData )
 
-// FIXME: use Soprano::Vocabulary::RDFS::RESOURCE()
-static const char* s_defaultType = "http://www.w3.org/2000/01/rdf-schema#Resource";
+
+static int s_dataCnt = 0;
+
 
 static Nepomuk::Variant nodeToVariant( const Soprano::Node& node )
 {
     if ( node.isResource() ) {
-        return Nepomuk::Variant( Nepomuk::Resource( node.toString() ) );
+        return Nepomuk::Variant( Nepomuk::Resource( node.uri() ) );
     }
     else if ( node.isLiteral() ) {
         return Nepomuk::Variant( node.literal().variant() );
@@ -59,19 +62,36 @@ static Nepomuk::Variant nodeToVariant( const Soprano::Node& node )
 }
 
 
-Nepomuk::ResourceData::ResourceData( const QString& uriOrId, const QString& type_ )
+Nepomuk::ResourceData::ResourceData( const QUrl& uri, const QString& uriOrId, const QUrl& type )
     : m_kickoffUriOrId( uriOrId ),
-      m_type( type_ ),
+      m_uri( uri ),
+      m_type( type ),
       m_ref(0),
-      m_proxyData(0)
+      m_proxyData(0),
+      m_cacheDirty(true)
 {
-    if( m_type.isEmpty() && !uriOrId.isEmpty() )
-        m_type = s_defaultType;
+    if( m_type.isEmpty() && !( uriOrId.isEmpty() && uri.isEmpty() ) )
+        m_type = Soprano::Vocabulary::RDFS::Resource();
+
+    // TODO: handle the caching in a decent Cache class and not this ugly.
+    if ( s_dataCnt >= 1000 ) {
+        for( ResourceDataHash::iterator rdIt = initializedData()->begin();
+             rdIt != initializedData()->end(); ++rdIt ) {
+            ResourceData* data = rdIt.value();
+            if ( !data->cnt() ) {
+                data->deleteData();
+                break;
+            }
+        }
+    }
+
+    ++s_dataCnt;
 }
 
 
 Nepomuk::ResourceData::~ResourceData()
 {
+    --s_dataCnt;
     if( m_proxyData )
         m_proxyData->deref();
 }
@@ -89,11 +109,11 @@ QString Nepomuk::ResourceData::uri() const
 {
     if( m_proxyData )
         return m_proxyData->uri();
-    return m_uri;
+    return m_uri.toString();
 }
 
 
-QString Nepomuk::ResourceData::type() const
+QUrl Nepomuk::ResourceData::type() const
 {
     if( m_proxyData )
         return m_proxyData->type();
@@ -110,12 +130,11 @@ void Nepomuk::ResourceData::deleteData()
     m_proxyData = 0;
 
     if( !m_uri.isEmpty() )
-        initializedData()->remove( m_uri );
+        initializedData()->remove( m_uri.toString() );
     if( !m_kickoffUriOrId.isEmpty() )
         kickoffData()->remove( m_kickoffUriOrId );
 
-    // FIXME: crashes
-    // delete this;
+    deleteLater();
 }
 
 
@@ -128,7 +147,7 @@ QHash<QString, Nepomuk::Variant> Nepomuk::ResourceData::allProperties()
     QHash<QString, Variant> props;
 
     if ( determineUri() ) {
-        Soprano::StatementIterator it = model->listStatements( Soprano::Statement( QUrl(m_uri), Soprano::Node(), Soprano::Node() ) );
+        Soprano::StatementIterator it = model->listStatements( Soprano::Statement( m_uri, Soprano::Node(), Soprano::Node() ) );
         while ( it.next() ) {
             Statement statement = *it;
             if ( props.contains( statement.predicate().toString() ) ) {
@@ -151,7 +170,21 @@ bool Nepomuk::ResourceData::hasProperty( const QString& uri )
         return m_proxyData->hasProperty( uri );
 
     if ( determineUri() ) {
-        return ResourceManager::instance()->mainModel()->containsAnyStatement( Soprano::Statement( QUrl( m_uri ), QUrl( uri ), Soprano::Node() ) );
+        return ResourceManager::instance()->mainModel()->containsAnyStatement( Soprano::Statement( m_uri, QUrl( uri ), Soprano::Node() ) );
+    }
+    else {
+        return false;
+    }
+}
+
+
+bool Nepomuk::ResourceData::hasType( const QUrl& uri )
+{
+    if( m_proxyData )
+        return m_proxyData->hasType( uri );
+
+    if ( determineUri() ) {
+        return ResourceManager::instance()->mainModel()->containsAnyStatement( Soprano::Statement( m_uri, Soprano::Vocabulary::RDF::type(), QUrl( uri ) ) );
     }
     else {
         return false;
@@ -164,24 +197,32 @@ Nepomuk::Variant Nepomuk::ResourceData::property( const QString& uri )
     if( m_proxyData )
         return m_proxyData->property( uri );
 
-    Variant v;
-
-    if ( determineUri() ) {
-        Soprano::Model* model = ResourceManager::instance()->mainModel();
-        Soprano::StatementIterator it = model->listStatements( Soprano::Statement( QUrl(m_uri), QUrl(uri), Soprano::Node() ) );
-
-        while ( it.next() ) {
-            Statement statement = *it;
-            if ( !v.isValid() ) {
-                v = nodeToVariant( statement.object() );
-            }
-            else {
-                v.append( nodeToVariant( statement.object() ) );
-            }
+    if ( load() ) {
+        QHash<QUrl, Variant>::iterator it = m_cache.find( uri );
+        if ( it == m_cache.end() ) {
+            return Variant();
+        }
+        else {
+            return *it;
         }
-        it.close();
     }
-    return v;
+    else {
+        return Variant();
+    }
+
+//     Variant v;
+
+//     if ( determineUri() ) {
+//         Soprano::Model* model = ResourceManager::instance()->mainModel();
+//         Soprano::StatementIterator it = model->listStatements( Soprano::Statement( m_uri, QUrl(uri), Soprano::Node() ) );
+
+//         while ( it.next() ) {
+//             Statement statement = *it;
+//             v.append( nodeToVariant( statement.object() ) );
+//         }
+//         it.close();
+//     }
+//     return v;
 }
 
 
@@ -193,11 +234,19 @@ bool Nepomuk::ResourceData::store()
 
             // save type
             // FIXME: handle multiple types
-            statements.append( Statement( QUrl(m_uri), QUrl(Nepomuk::typePredicate()), QUrl(m_type) ) );
+            statements.append( Statement( m_uri, Soprano::Vocabulary::RDF::type(), m_type ) );
 
             // save the kickoff identifier (other identifiers are stored via setProperty)
             if ( !m_kickoffIdentifier.isEmpty() ) {
-                statements.append( Statement( QUrl(m_uri), QUrl(Resource::identifierUri()), LiteralValue(m_kickoffIdentifier) ) );
+                statements.append( Statement( m_uri, QUrl(Resource::identifierUri()), LiteralValue(m_kickoffIdentifier) ) );
+            }
+
+            // HACK: make sure that files have proper fileUrl properties so long as we do not have a File class for
+            // Dolphin and co.
+            if ( QFile::exists( m_uri.toLocalFile())) {
+                statements.append( Statement( m_uri,
+                                              QUrl("http://freedesktop.org/standards/xesam/1.0/core#url"),
+                                              LiteralValue( m_uri.toLocalFile() ) ) );
             }
 
             return ResourceManager::instance()->mainModel()->addStatements( statements ) == Soprano::Error::ErrorNone;
@@ -212,6 +261,35 @@ bool Nepomuk::ResourceData::store()
 }
 
 
+bool Nepomuk::ResourceData::load()
+{
+    if ( m_cacheDirty ) {
+        m_cache.clear();
+
+        if ( determineUri() ) {
+            Soprano::Model* model = ResourceManager::instance()->mainModel();
+            Soprano::StatementIterator it = model->listStatements( Soprano::Statement( m_uri, Soprano::Node(), Soprano::Node() ) );
+
+            while ( it.next() ) {
+                Statement statement = *it;
+                if ( statement.predicate().uri() != Soprano::Vocabulary::RDF::type() ) {
+                    m_cache[statement.predicate().uri()].append( nodeToVariant( statement.object() ) );
+                }
+            }
+
+            m_cacheDirty = false;
+            return true;
+        }
+        else {
+            return false;
+        }
+    }
+    else {
+        return true;
+    }
+}
+
+
 void Nepomuk::ResourceData::setProperty( const QString& uri, const Nepomuk::Variant& value )
 {
     if( m_proxyData )
@@ -221,10 +299,9 @@ void Nepomuk::ResourceData::setProperty( const QString& uri, const Nepomuk::Vari
     if ( store() ) {
         Soprano::Model* model = ResourceManager::instance()->mainModel();
 
-        // step 1: remove all the existing stuff
-        model->removeAllStatements( Statement( QUrl(m_uri), QUrl(uri), Node() ) );
+        QList<Statement> resourceStatements;
 
-        // step 2: make sure resource values are in the store
+        // make sure resource values are in the store
         if ( value.simpleType() == qMetaTypeId<Resource>() ) {
             QList<Resource> l = value.toResourceList();
             for( QList<Resource>::iterator resIt = l.begin(); resIt != l.end(); ++resIt ) {
@@ -232,18 +309,18 @@ void Nepomuk::ResourceData::setProperty( const QString& uri, const Nepomuk::Vari
             }
         }
 
-        // step 3: add the actual property statements
+        // add the actual property statements
 
         // one-to-one Resource
         if( value.isResource() ) {
-            model->addStatement( Statement( QUrl(m_uri), QUrl(uri), QUrl( value.toResource().uri() ) ) );
+            resourceStatements.append( Statement( m_uri, QUrl(uri), QUrl( value.toResource().uri() ) ) );
         }
 
         // one-to-many Resource
         else if( value.isResourceList() ) {
             const QList<Resource>& l = value.toResourceList();
             for( QList<Resource>::const_iterator resIt = l.constBegin(); resIt != l.constEnd(); ++resIt ) {
-                model->addStatement( Statement( QUrl(m_uri), QUrl(uri), QUrl( (*resIt).uri() ) ) );
+                resourceStatements.append( Statement( m_uri, QUrl(uri), QUrl( (*resIt).uri() ) ) );
             }
         }
 
@@ -251,14 +328,32 @@ void Nepomuk::ResourceData::setProperty( const QString& uri, const Nepomuk::Vari
         else if( value.isList() ) {
             QList<Node> nl = Nepomuk::valuesToRDFNodes( value );
             for( QList<Node>::const_iterator nIt = nl.constBegin(); nIt != nl.constEnd(); ++nIt ) {
-                model->addStatement( Statement( QUrl(m_uri), QUrl(uri), *nIt ) );
+                resourceStatements.append( Statement( m_uri, QUrl(uri), *nIt ) );
             }
         }
 
         // one-to-one literal
         else {
-            model->addStatement( Statement( QUrl(m_uri), QUrl(uri),
-                                            Nepomuk::valueToRDFNode( value ) ) );
+            resourceStatements.append( Statement( m_uri, QUrl(uri),
+                                                  Nepomuk::valueToRDFNode( value ) ) );
+        }
+
+        // update the cache for now
+        m_cache[uri] = value;
+
+        // update the store
+        QList<Statement> existingStatements = model->listStatements( Soprano::Statement( m_uri, QUrl(uri), Node() ) ).allStatements();
+        for ( QList<Statement>::const_iterator it = existingStatements.constBegin();
+              it != existingStatements.constEnd(); ++it ) {
+            if ( !resourceStatements.contains( *it ) ) {
+                model->removeStatement( *it );
+            }
+        }
+        for ( QList<Statement>::const_iterator it = resourceStatements.constBegin();
+              it != resourceStatements.constEnd(); ++it ) {
+            if ( !existingStatements.contains( *it ) ) {
+                model->addStatement( *it );
+            }
         }
     }
 }
@@ -270,7 +365,7 @@ void Nepomuk::ResourceData::removeProperty( const QString& uri )
         return m_proxyData->removeProperty( uri );
 
     if ( determineUri() ) {
-        ResourceManager::instance()->mainModel()->removeAllStatements( Statement( QUrl(m_uri), QUrl(uri), Node() ) );
+        ResourceManager::instance()->mainModel()->removeAllStatements( Statement( m_uri, QUrl(uri), Node() ) );
     }
 }
 
@@ -282,9 +377,9 @@ void Nepomuk::ResourceData::remove( bool recursive )
 
     if ( determineUri() ) {
         Soprano::Model* model = ResourceManager::instance()->mainModel();
-        model->removeAllStatements( Statement( QUrl(m_uri), Node(), Node() ) );
+        model->removeAllStatements( Statement( m_uri, Node(), Node() ) );
         if ( recursive ) {
-            model->removeAllStatements( Statement( Node(), Node(), QUrl(m_uri) ) );
+            model->removeAllStatements( Statement( Node(), Node(), m_uri ) );
         }
     }
 }
@@ -296,7 +391,7 @@ bool Nepomuk::ResourceData::exists()
         return m_proxyData->exists();
 
     if( determineUri() ) {
-        return ResourceManager::instance()->mainModel()->containsAnyStatement( Statement( QUrl( m_uri ), Node(), Node() ) );
+        return ResourceManager::instance()->mainModel()->containsAnyStatement( Statement( m_uri, Node(), Node() ) );
     }
     else
         return false;
@@ -358,50 +453,65 @@ bool Nepomuk::ResourceData::determineUri()
             // TODO: basically it is perfectly valid to store both types in the first case
             //
             Q_ASSERT( !m_type.isEmpty() );
-            const Nepomuk::Class* wantedType = Nepomuk::Class::load( m_type );
-            if ( wantedType && m_type != s_defaultType ) {
-                while ( it.next() ) {
-                    // get the type of the stored resource
-                    StatementIterator resourceSl = model->listStatements( Statement( it.current().subject(),
-                                                                                     QUrl( typePredicate() ),
-                                                                                     Node() ) );
-                    if ( resourceSl.next() ) {
-                        const Nepomuk::Class* storedType = Nepomuk::Class::load( resourceSl.current().object().uri() );
-                        if ( storedType ) {
-                            if ( storedType == wantedType ) {
-                                // great. :)
-                                m_uri = it.current().subject().toString();
-                            }
-                            else if ( wantedType->isSubClassOf( storedType ) ) {
-                                // Keep the type that is further down the hierarchy
-                                m_type = wantedType->uri().toString();
-                                m_uri = it.current().subject().toString();
-                                break;
-                            }
-                            else if ( storedType->isSubClassOf( wantedType ) ) {
-                                // just use the existing data with the finer grained type
-                                m_uri = it.current().subject().toString();
-                                break;
+            if ( it.next() ) {
+                const Nepomuk::Class* wantedType = Nepomuk::Class::load( m_type );
+                if ( wantedType && m_type != Soprano::Vocabulary::RDFS::Resource() ) {
+                    do {
+                        // get the type of the stored resource
+                        StatementIterator resourceSl = model->listStatements( Statement( it.current().subject(),
+                                                                                         Soprano::Vocabulary::RDF::type(),
+                                                                                         Node() ) );
+                        if ( resourceSl.next() && resourceSl.current().object().isResource() ) {
+                            const Nepomuk::Class* storedType = Nepomuk::Class::load( resourceSl.current().object().uri() );
+                            if ( storedType ) {
+                                if ( storedType == wantedType ) {
+                                    // great. :)
+                                    m_uri = it.current().subject().uri();
+                                }
+                                else if ( wantedType->isSubClassOf( storedType ) ) {
+                                    // Keep the type that is further down the hierarchy
+                                    m_type = wantedType->uri();
+                                    m_uri = it.current().subject().uri();
+                                    break;
+                                }
+                                else if ( storedType->isSubClassOf( wantedType ) ) {
+                                    // just use the existing data with the finer grained type
+                                    m_uri = it.current().subject().uri();
+                                    break;
+                                }
                             }
                         }
-                    }
+                    } while ( it.next() );
+                }
+                else {
+                    m_uri = it.current().subject().uri();
+                    kDebug(300004) << k_funcinfo << " kickoff identifier " << kickoffUriOrId() << " already exists with URI " << uri();
+                    updateType();
                 }
-            }
-            else if ( it.next() ) {
-                m_uri = it.current().subject().toString();
-                kDebug(300004) << k_funcinfo << " kickoff identifier " << kickoffUriOrId() << " already exists with URI " << uri();
-                updateType();
             }
 
             it.close();
 
             if ( m_uri.isEmpty() ) {
                 //
-                // The resource does not exist, create a new one
+                // The resource does not exist, create a new one:
+                // If the kickoffUriOrId is a valid URI we use it as such, otherwise we create a new URI
+                // Special case: files: paths are always converted to URIs
                 //
-                m_kickoffIdentifier = kickoffUriOrId();
-                m_uri = ResourceManager::instance()->generateUniqueUri();
-                kDebug(300004) << " kickoff identifier " << kickoffUriOrId() << " seems fresh. Generated new URI " << uri();
+                QUrl uri( kickoffUriOrId() );
+                if ( uri.isValid() && !uri.scheme().isEmpty() ) {
+                    m_uri = uri;
+                }
+                else if ( QFile::exists( kickoffUriOrId() ) ) {
+                    // KURL defaults to schema "file:"
+                    m_uri = KUrl::fromPath( kickoffUriOrId() );
+                }
+                else {
+                    m_kickoffIdentifier = kickoffUriOrId();
+                    m_uri = ResourceManager::instance()->generateUniqueUri();
+                }
+
+                kDebug(300004) << " kickoff identifier " << kickoffUriOrId() << " seems fresh. Generated new URI " << m_uri;
             }
         }
 
@@ -431,13 +541,13 @@ void Nepomuk::ResourceData::updateType()
     Soprano::Model* model = ResourceManager::instance()->mainModel();
 
     // get the type of the stored resource
-    StatementIterator typeStatements = model->listStatements( Statement( QUrl( m_uri ),
-                                                                         QUrl( typePredicate() ),
+    StatementIterator typeStatements = model->listStatements( Statement( m_uri,
+                                                                         Soprano::Vocabulary::RDF::type(),
                                                                          Node() ) );
-    if ( typeStatements.next() ) {
+    if ( typeStatements.next() && typeStatements.current().object().isResource() ) {
         // FIXME: handle multple types, maybe select the one type that fits best
-        QString storedType = typeStatements.current().object().toString();
-        if ( m_type == s_defaultType ) {
+        QUrl storedType = typeStatements.current().object().uri();
+        if ( m_type == Soprano::Vocabulary::RDFS::Resource() ) {
             m_type = storedType;
         }
         else {
@@ -447,7 +557,7 @@ void Nepomuk::ResourceData::updateType()
             if ( wantedTypeClass && storedTypeClass ) {
                 // Keep the type that is further down the hierarchy
                 if ( wantedTypeClass->isSubClassOf( storedTypeClass ) ) {
-                    m_type = wantedTypeClass->uri().toString();
+                    m_type = wantedTypeClass->uri();
                 }
             }
         }
@@ -474,9 +584,56 @@ bool Nepomuk::ResourceData::operator==( const ResourceData& other ) const
 }
 
 
-Nepomuk::ResourceData* Nepomuk::ResourceData::data( const QString& uriOrId, const QString& type )
+Nepomuk::ResourceData* Nepomuk::ResourceData::data( const QUrl& uri, const QUrl& type )
 {
-    Q_ASSERT( !uriOrId.isEmpty() );
+    if ( uri.isEmpty() ) {
+        return 0;
+    }
+
+    // default to "file" scheme, i.e. we do not allow an empty scheme
+    if ( uri.scheme().isEmpty() ) {
+        QUrl fileUri( uri );
+        fileUri.setScheme( "file" );
+        return data( fileUri, type );
+    }
+
+    ResourceDataHash::iterator it = initializedData()->find( uri.toString() );
+
+    //
+    // The uriOrId has no local representation yet -> create one
+    //
+    if( it == initializedData()->end() ) {
+        kDebug(300004) << "No existing ResourceData instance found for uri " << uri;
+        //
+        // The actual URI is already known here
+        //
+        ResourceData* d = new ResourceData( uri, QString(), type );
+        initializedData()->insert( uri.toString(), d );
+
+        return d;
+    }
+    else {
+        //
+        // Reuse the already existing ResourceData object
+        //
+        return it.value();
+    }
+}
+
+
+Nepomuk::ResourceData* Nepomuk::ResourceData::data( const QString& uriOrId, const QUrl& type )
+{
+    if ( uriOrId.isEmpty() ) {
+        return 0;
+    }
+
+    // special case: files (only absolute paths for now)
+    if ( uriOrId[0] == '/' ) {
+        ResourceDataHash::iterator it = initializedData()->find( "file://" + uriOrId );
+        if ( it != initializedData()->end() ) {
+            return *it;
+        }
+    }
 
     ResourceDataHash::iterator it = initializedData()->find( uriOrId );
 
@@ -489,7 +646,7 @@ Nepomuk::ResourceData* Nepomuk::ResourceData::data( const QString& uriOrId, cons
         it = kickoffData()->find( uriOrId );
 
         // check if the type matches (see determineUri for details)
-        if ( !type.isEmpty() && type != s_defaultType ) {
+        if ( !type.isEmpty() && type != Soprano::Vocabulary::RDFS::Resource() ) {
             const Nepomuk::Class* wantedType = Nepomuk::Class::load( type );
             if ( wantedType ) {
                 while ( it != kickoffData()->end() &&
@@ -520,7 +677,7 @@ Nepomuk::ResourceData* Nepomuk::ResourceData::data( const QString& uriOrId, cons
         //
         // Every new ResourceData object ends up in the kickoffdata since its actual URI is not known yet
         //
-        ResourceData* d = new ResourceData( uriOrId, type );
+        ResourceData* d = new ResourceData( QUrl(), uriOrId, type );
         kickoffData()->insert( uriOrId, d );
 
         return d;
@@ -576,6 +733,12 @@ QList<Nepomuk::ResourceData*> Nepomuk::ResourceData::allResourceData()
          rdIt != kickoffData()->end(); ++rdIt ) {
         l.append( rdIt.value() );
     }
+    for( ResourceDataHash::iterator rdIt = initializedData()->begin();
+         rdIt != initializedData()->end(); ++rdIt ) {
+        l.append( rdIt.value() );
+    }
 
     return l;
 }
+
+#include "resourcedata.moc"
