@@ -302,35 +302,37 @@ void MediaObject::setQueue(const QList<MediaSource> &sources)
 {
     K_D(MediaObject);
     d->sourceQueue.clear();
-    d->sourceQueue << sources;
+    enqueue(sources);
 }
 
 void MediaObject::setQueue(const QList<QUrl> &urls)
 {
     K_D(MediaObject);
     d->sourceQueue.clear();
-    foreach (const QUrl &url, urls) {
-        d->sourceQueue << url;
-    }
+    enqueue(urls);
 }
 
 void MediaObject::enqueue(const MediaSource &source)
 {
     K_D(MediaObject);
-    d->sourceQueue << source;
+    if (d->mediaSource.type() == MediaSource::Invalid) {
+        setCurrentSource(source);
+    } else {
+        d->sourceQueue << source;
+    }
 }
 
 void MediaObject::enqueue(const QList<MediaSource> &sources)
 {
-    K_D(MediaObject);
-    d->sourceQueue << sources;
+    foreach (const MediaSource &m, sources) {
+        enqueue(m);
+    }
 }
 
 void MediaObject::enqueue(const QList<QUrl> &urls)
 {
-    K_D(MediaObject);
     foreach (const QUrl &url, urls) {
-        d->sourceQueue << url;
+        enqueue(url);
     }
 }
 
@@ -369,7 +371,10 @@ void MediaObjectPrivate::streamError(Phonon::ErrorType type, const QString &text
 void MediaObjectPrivate::_k_stateChanged(Phonon::State newstate, Phonon::State oldstate)
 {
     Q_Q(MediaObject);
-    Q_ASSERT(mediaSource.type() == MediaSource::Url);
+    if (mediaSource.type() != MediaSource::Url) {
+        // special handling only necessary for URLs because of the fallback
+        emit q->stateChanged(newstate, oldstate);
+    }
     if (errorOverride) {
         errorOverride = false;
         if (newstate == ErrorState) {
@@ -401,7 +406,7 @@ void MediaObjectPrivate::_k_stateChanged(Phonon::State newstate, Phonon::State o
             break;
         default:
             pError() << "backend MediaObject reached ErrorState after " << oldstate
-                << ". It seems a KioMediaStream won't help here, trying anyway.";
+                << ". It seems a KioMediaStream will not help here, trying anyway.";
             emit q->stateChanged(Phonon::LoadingState, oldstate);
             break;
         }
@@ -444,9 +449,20 @@ void MediaObjectPrivate::_k_aboutToFinish()
             return;
         }
     }
-    mediaSource = sourceQueue.dequeue();
+
+    mediaSource = sourceQueue.head();
     pINTERFACE_CALL(setNextSource(mediaSource));
-    emit q->currentSourceChanged(mediaSource);
+}
+
+void MediaObjectPrivate::_k_currentSourceChanged(const MediaSource &source)
+{
+    Q_Q(MediaObject);
+    pDebug() << Q_FUNC_INFO;
+
+    if (!sourceQueue.isEmpty() && sourceQueue.head() == source)
+        sourceQueue.dequeue();
+
+    emit q->currentSourceChanged(source);
 }
 
 void MediaObjectPrivate::setupBackendObject()
@@ -470,6 +486,8 @@ void MediaObjectPrivate::setupBackendObject()
     QObject::connect(m_backendObject, SIGNAL(totalTimeChanged(qint64)), q, SIGNAL(totalTimeChanged(qint64)));
     QObject::connect(m_backendObject, SIGNAL(metaDataChanged(const QMultiMap<QString, QString> &)),
             q, SLOT(_k_metaDataChanged(const QMultiMap<QString, QString> &)));
+    QObject::connect(m_backendObject, SIGNAL(currentSourceChanged(const MediaSource&)), 
+        q, SLOT(_k_currentSourceChanged(const MediaSource&)));
 
     // set up attributes
     pINTERFACE_CALL(setTickInterval(tickInterval));
